Package: scMultiome
Title: Collection of Public Single-Cell Multiome (scATAC + scRNAseq)
        Datasets
Version: 1.10.0
Authors@R: c(
    person(given = "Xiaosai", family ="Yao", email = "xiaosai.yao@gmail.com", role = c("cre","aut"),  comment = c(ORCID = "0000-0001-9729-0726")),
    person(given = "Aleksander", family ="Chlebowski", email = "aleksander.chlebowski@contractors.roche.com", role = "aut"),
    person(given = "Aaron", family ="Lun", email = "lun.aaron@gene.com", role = "aut"),
    person(given = "Shiqi", family ="Xie", email = "xie.shiqi@gene.com", role = "ctb"),
    person(given = "Tomasz", family = "Włodarczyk", email = "tomasz.wlodarczyk@contractors.roche.com", role = "aut")
  )
Description: Single cell multiome data, containing chromatin accessibility
    (scATAC-seq) and gene expression (scRNA-seq) information analyzed with
    the ArchR package and presented as MultiAssayExperiment objects.
License: CC BY-SA 4.0
Depends: AnnotationHub, ExperimentHub (>= 2.8.1), MultiAssayExperiment,
        SingleCellExperiment, SummarizedExperiment
Imports: AzureStor, GenomicRanges, HDF5Array, S4Vectors, checkmate,
        methods, rhdf5, alabaster.matrix
Suggests: BiocGenerics, IRanges, Matrix, knitr, rmarkdown, rstudioapi,
        testthat (>= 3.0.0), devtools, BiocStyle, ExperimentHubData
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
biocViews: PackageTypeData, ExperimentHub, SingleCellData,
        ExpressionData, SequencingData, Homo_sapiens_Data, CellCulture,
        Tissue, GEO
git_url: https://git.bioconductor.org/packages/scMultiome
git_branch: RELEASE_3_22
git_last_commit: 26da406
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 16:42:45 UTC; biocbuild
Author: Xiaosai Yao [cre, aut] (ORCID: <https://orcid.org/0000-0001-9729-0726>),
  Aleksander Chlebowski [aut],
  Aaron Lun [aut],
  Shiqi Xie [ctb],
  Tomasz Włodarczyk [aut]
Maintainer: Xiaosai Yao <xiaosai.yao@gmail.com>

# Generated by roxygen2: do not edit by hand

export(AR_drug)
export(PBMC_10x)
export(TEADi_resistance)
export(assertHDF5)
export(colonHealthy)
export(dummyMAE)
export(dummySCE)
export(hematopoiesis)
export(listDatasets)
export(loadExp)
export(loadMAE)
export(prostateENZ)
export(reprogramSeq)
export(saveExp)
export(saveMAE)
export(testFile)
export(tfBinding)
export(tfMotifs)
export(uploadFile)
exportMethods(saveExp)
import(AnnotationHub)
import(ExperimentHub)
import(MultiAssayExperiment)
import(SingleCellExperiment)
import(SummarizedExperiment)

# scMultiome


## Motivation

Single cell data is gaining sophistication - Cells can be measured in multiple modalities including gene expression, chromatin accessibility, cell surface markers and protein expression. These orthogonal measures of the same or matched cells enable a holistic construction of the cell state. However it has been challenging to share multiomic data, especially in an integrated format that consolidates the multiple layers of measurements. The `MultiAssayExperiment` provides a framework to package the various modalities into a single dataset on a per cell basis.

The `scMultiome` package is a collection of public single cell multiome data sets pre-processed and packaged into `MultiAssayExperiment` objects for downstream analysis. It also provides basic functions to save the `MultiAssayExperiment` as `.hdf5` files so that users can load only the desired modalities into memory.


## Installation:
```
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")

BiocManager::install("scMultiome")
```


## Package Contents

To list currently available data sets,

```
listDatasets()
```

or see package help with 

```
?scMultiome
```

## Tutorial

For a quick start, please see our [vignette](https://bioconductor.org/packages/release/data/experiment/vignettes/scMultiome/inst/doc/scMultiome.html)

## Data Format

Current multiomic data sets consist of gene expression and chromatin accessibility but can be extended to include any other modalities. The current data sets are either paired multiomic data sets or unpaired data sets with data integration performed by the `ArchR` [package](https://www.archrproject.com/). 


The `MultiAssayExperiment` object is saved in hdf5 files. The nature of the hdf5 format allows the MAEs to be split into individual experiments and store them in one file, so you can choose freely which ones to load. Experiments, usually `SingleCellExperiment` objects, are disassembled into parts, which are saved in the hdf5 hierarchy. Assays are saved as sparse arrays to save storage.

Upon loading, selected experiments are reassembled and wrapped into an MAE object. Assays are represented by `DelayedMatrix` objects to save memory.

_NOTE: These data sets can be quite large. See `listDatasets()` to avoid surprises._




## Data Storage and Access

Data sets in `MultiAssayExperiment` form are saved to disk with the `saveMAE` function. It creates an hdf5 file and dispatches individual experiments to `saveExp`. `saveExp` disassembles and saves experiments to the file.

Each data set is accessed by its own accessor function. Accessor functions call the internal `retrieve` function to query `ExperimentHub` for the resource in question, which causes the file to be downloaded and cached. The path to the cached file is passed to `loadMAE`, which in turn calls `loadExp` for each experiment specified in the accessor call. `loadExp` reads experiment components and reassembles them, and then `loadMAE` builds a `MultiAssayExperiment` object. This object is the final return value of the accessor function.

![](inst/images/scMultiome.png)
![](inst/images/scMultiome.functions.svg)



## Package Structure:

```
.
├── README.md                       this file
├── DESCRIPTION                     package metadata
├── NAMESPACE                       namespace information
├── R/                              functions
├── man/                            help files
├── tests/                          unit test files
├── vignettes/                      vignettes
├── images/                         images
├── inst/
│   ├── images/                     image files used in documentation
│   ├── extdata/                    external data, including data set metadata
│   └── scripts/                    scripts, including ones to create data set metadata
└── scMultiome.Rproj                RStudio project file           
```


## Contributing

To add your public data sets to the package, review the vignette _Adding Data Sets_ and contact the package maintainer.

#' custom classes
#'
#' Additional class definitions.
#'
#' @section Classes:
#' \itemize{
#' \item{\code{SingleCellAccessibilityExperiment}, contains \code{SingleCellExperiment}}
#' }
#'
#' @keywords internal
#'
#' @name customClasses
#'
#' @rdname customClasses
setClass("SingleCellAccessibilityExperiment", contains = "SingleCellExperiment")

# ExperimentHub metadata
metadata.prostateENZ <- data.frame(
    Title = "LNCaP Cells Treated with Enzalutamide",
    Description = "Multiome data (scATAC and scRNAseq) for enzalutamide-treated LNCaP cells. Measured (unpaired) with Illumina NextSeq 500 (GPL18573). Results stored in GEO, acc.nos: GSE168667 and GSE168668.",
    BiocVersion = "3.17",
    Genome = "hg38",
    SourceType = "tar.gz",
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE168667&format=file, https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE168668&format=file", # refers to raw data; optional
    SourceVersion = "Sep 08 2021",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE
    DataProvider = "Tampere University", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "MultiAssayExperiment",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/prostateENZ.h5"
)

# dataset manifest metadata
manifest.prostateENZ <- data.frame(
    Call = "prostateENZ()",
    Author = "Taavitsainen",
    Title = "LNCaP Cells Treated with Enzalutamide",
    Species = "Homo sapiens",
    Lineage = "Prostate",
    Multiome = "unpaired",
    CellNumber = "15522",
    DiskSize = "2.9 GB",
    Version = "2022-09-06"
)

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.TEADi_resistance <-
  data.frame(
    Title = "Resistance of TEAD inhibitor to drug",
    Description = "Cooperation between the Hippo and MAPK pathway activation drives acquired resistance to TEAD inhibition",
    BiocVersion = "3.20",
    Genome = "hg38",
    SourceType = "HDF5", # refers to raw data
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE247442", # refers to raw data
    SourceVersion = "2022-05-19", # no commas!
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    TaxonomyId = "9606", # e.g. "9606"
    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE
    DataProvider = "Genentech, Inc.", # refers to raw data
    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
    RDataClass = "MultiAssayExperiment", # class that is returned from hub
    DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
    # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
    RDataPath = "scMultiome/TEADi_resistance.h5"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.TEADi_resistance <-
  data.frame(
    Call = "TEADi_resistance()",
    Author = "Xiaosai Yao",
    Title = "Resistance of TEAD inhibitor to drug",
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    Lineage = "H226 cell line",
    CellNumber = "4952",
    Multiome = "paired",
    DiskSize = "825.4 MB",
    Version = "2024-09-17"
)

#'
#' scATAC-seq and unpaired scRNA-seq of hematopoetic cells
#'
#' Example scATAC-seq data of hematopoietic cells included in ArchR package was integrated with scRNAseq. ScATAC-seq data was obtained from GSE139369 and scRNA-seq obtained from https://jeffgranja.s3.amazonaws.com/ArchR/TestData/scRNA-Hematopoiesis-Granja-2019.rds
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the hg19 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 17889 rows and 10250 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 22217 rows and 10250 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 150046 rows and 10250 columns}
#'   \item{\strong{TileMatrix500}: SingleCellExperiment with 5762078 rows and 10250 columns}
#' }
#'
#' @references
#' Single-cell multiomic analysis identifies regulatory programs in mixed-phenotype acute leukemia
#' associated with prostate cancer relapse. 
#' Granja \emph{et al.}, \emph{Nature Biotechnology} 2019 Dec;37(12):1458-1465. 
#' \href{https://www.nature.com/articles/s41587-019-0332-7}{doi: 10.1038/s41587-019-0332-7}
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-hematopoiesis.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' hematopoiesis(metadata = TRUE)
#' # download data
#' \dontrun{
#' hematopoiesis()
#' }
#' @export
#'
hematopoiesis <-
    function(metadata = FALSE,
             experiments = c("TileMatrix500",
                             "GeneScoreMatrix",
                             "GeneIntegrationMatrix",
                             "PeakMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("hematopoiesis", metadata, experiments, verbose = FALSE)
    }

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.PBMC_10x <-
  data.frame(
    Title = "PBMC Data Set",
    Description = "PBMC from a Healthy Donor - Granulocytes Removed Through Cell Sorting (10k)",
    BiocVersion = "3.20",
    Genome = "hg38",
    SourceType = "HDF5",
    SourceUrl = "https://cf.10xgenomics.com/samples/cell-arc/1.0.0/pbmc_granulocyte_sorted_10k/",
    SourceVersion = "2021-05-03",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "10X Genomics",
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "MultiAssayExperiment",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/PBMC_10x.h5"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.PBMC_10x <-
  data.frame(
    Call = "PBMC_10x()",
    Author = "10X Genomics",
    Title = "PBMC Data Set",
    Species = "Homo sapiens",
    Lineage = "Blood",
    CellNumber = "11898",
    Multiome = "paired",
    DiskSize = "1.2 GB",
    Version = "2024-09-17"
)

#'
#' reprogramSeq
#'
#' scMultiome data of LNCaP infected with FOXA1, NKX2-1, GATA6
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix500}: SingleCellAccessibilityExperiment with 6062095 rows and 3903 columns}
#'   \item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36438 rows and 3903 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 3903 columns}
#'   \item{\strong{NEPCMatrix}: SingleCellExperiment with 2 rows and 3903 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 126602 rows and 3903 columns}
#'   \item{\strong{TF_bindingMatrix}: SingleCellExperiment with 1274 rows and 3903 columns}
#' }
#'
#' @references
#' Genentech dataset
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-reprogramSeq.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' reprogramSeq(metadata = TRUE)
#' # download data
#' \dontrun{
#' reprogramSeq()
#' }
#'
#' @export
#'
reprogramSeq <-
    function(metadata = FALSE,
             experiments = c("TileMatrix500",
                             "GeneExpressionMatrix",
                             "GeneScoreMatrix",
                             "NEPCMatrix",
                             "PeakMatrix",
                             "TF_bindingMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("reprogramSeq", metadata, experiments, verbose = FALSE)
    }

#'
#' AR-dependent gene expression in prostate cancer cells
#'
#' Single cell gene expression and ATACseq from prostate cancer cell lines (LNCaP, VCaP, DU145, MDA-PCA-2B, 22Rv1 and NCI-H660).
#' after 24h of drug treatment (enzalutamide, ARV110 (AR degrader) or SMARCA2_4.1 (SMARCA2/4 degrader) or DMSO control.
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the Hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix}: SingleCellExperiment with 6068436 rows and 23118 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 57765 rows and 23118 columns}
#'   \item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36451 rows and 23118 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 237856 rows and 23118 columns}
#'   \item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 23118 columns}
#'   \item{\strong{TFPeaksDeviationsMatrix}: SingleCellExperiment with 1533 rows and 23118 columns}
#' }
#'
#' @references
#' Tomasz Włodarczyk, Aaron Lun, Diana Wu, Shreya Menon, Shushan Toneyan, Kerstin Seidel, Liang Wang,
#' Jenille Tan, Shang-Yang Chen, Timothy Keyes, Aleksander Chlebowski, Yu Guo, Ciara Metcalfe,
#' Marc Hafner, Christian W. Siebel, M. Ryan Corces, Robert Yauch, Shiqi Xie, Xiaosai Yao. 2023.
#' "Inference of single-cell transcription factor activity to dissect mechanisms of lineage
#' plasticity and drug response" bioRxiv 2023.11.27.568955; doi: https://doi.org/10.1101/2023.11.27.568955
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-AR_drug.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' AR_drug(metadata = TRUE)
#'
#' # download data
#'  \dontrun{
#' AR_drug()
#' }
#'
#' @export
#'
AR_drug <-
    function(metadata = FALSE,
             experiments = c("TileMatrix",
                             "GeneScoreMatrix",
                             "GeneExpressionMatrix",
                             "PeakMatrix",
                             "MotifMatrix",
                             "TFPeaksDeviationsMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("AR_drug", metadata, experiments, verbose = FALSE)
    }

#'
#' 10k PBMC data
#'
#' PBMC from a Healthy Donor downloaded from 10x Genomics. Granulocytes were removed by cell sorting.
#' Paired ATAC and Gene Expression libraries were generated from the isolated nuclei.
#' Targeted nuclei recovery was 10,000.
#' Data source: https://www.10xgenomics.com/datasets/pbmc-from-a-healthy-donor-granulocytes-removed-through-cell-sorting-10-k-1-standard-2-0-0
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the Hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 9702 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 9702 columns}
#'   \item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36438 rows and 9702 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 159290 rows and 9702 columns}
#'   \item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 9702 columns}
#' }
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-PBMC_10x.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' PBMC_10x(metadata = TRUE)
#' # download data
#'  \dontrun{
#' PBMC_10x()
#' }
#'
#' @export
#'
PBMC_10x <-
    function(metadata = FALSE,
             experiments = c("TileMatrix",
                             "GeneScoreMatrix",
                             "GeneExpressionMatrix",
                             "PeakMatrix",
                             "MotifMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("PBMC_10x", metadata, experiments, verbose = FALSE)
    }

#' retrieve data set or its metadata
#'
#' Retrieve and return the data or metadata for the currently queried data set.
#'
#' This is a generic accessor function that is used by user-level accessor functions
#' to access data sets or their metadata.
#'
#' @param dataset character string specifying the data set name
#' @param metadata logical flag specifying whether to return the resource or only its metadata
#' @param experiments character string specifying which experiments to extract
#' @param verbose logical flag specifying loading verbosity
#'
#' @return
#' If \code{metadata = FALSE}, a \code{MultiAssayExperiment},
#' otherwise an \code{ExperimentHub} object.
#'
#' @import AnnotationHub
#' @import ExperimentHub
#'
#' @keywords internal
#'
retrieve <- function(dataset, metadata, experiments, verbose = FALSE) {
    checkmate::assertString(dataset)
    checkmate::assertFlag(metadata)
    checkmate::assertCharacter(experiments)
    checkmate::assertFlag(verbose)

    # query database for the data set
    eh <- AnnotationHub::query(ExperimentHub::ExperimentHub(), c("scMultiome", dataset))
    # backstop for missing data set
    if (length(eh) == 0L) stop("resource ", dataset, " not found in ExperimentHub")
    # retrieve meatadata or data (i.e. path to file in bucket)
    ans <- if (metadata) {
        eh[1]
    } else {
        loadMAE(file = eh[[1]], experiments, verbose)
    }

    return(ans)
}

test_that("resource is returned", {
    testthat::expect_s4_class(retrieve("reprogramSeq", metadata = TRUE, experiments = "NEPCMatrix"),
                              "ExperimentHub")
    testthat::expect_s4_class(retrieve("reprogramSeq", metadata = FALSE, experiments = "NEPCMatrix"),
                              "MultiAssayExperiment")
})

test_that("missing resources signal errors", {
    expect_error(retrieve("nonexistentdataset",
                          metadata = FALSE,
                          experiments = c("experiment1", "experiment2")))
    expect_error(retrieve("nonexistentdataset",
                          metadata = TRUE,
                          experiments = c("experiment1", "experiment2")))
})

#' create data set list
#'
#' Automatically creates the data set list of the package.
#'
#' This is an internal helper function for developers and will not be called directly.
#' It creates the file inst/scripts/datasetList.Rmd, which is incorporated
#' into the package help page to automatically list the current data sets.
#'
#' @param metadata a \code{data.frame} containing data set metadata
#'
#' @return Invisible TRUE.
#'
#' @keywords internal
#'
makeDataSetList <- function(metadata) {
    checkmate::assertDataFrame(metadata)

    names <- tools::file_path_sans_ext(basename(metadata[["RDataPath"]]))
    titles <- metadata[["Title"]]

    fileName <- file.path(system.file("scripts", package = "scMultiome"), "datasetList.Rmd")
    file.create(fileName)
    writeLines(con = fileName, text = c(
        header(),
        mapply(makeLine, name = names, title = titles),
        footer())
    )

    return(invisible(TRUE))
}


makeLine <- function(name, title) {
    checkmate::assertString(name)
    checkmate::assertString(title)

    sprintf("+ **%s**: %s", name, title)
}


header <- function() {
    c(
        "---",
        "title: \"Data Set List\"",
        "output:",
        "  BiocStyle::html_document:",
        "    titlecaps: false",
        "    toc_float: true",
        "---",
        "",
        "```{r setup, include = FALSE}",
        "knitr::opts_chunk$set(echo = TRUE, eval = FALSE)",
        "```",
        ""
    )
}


footer <- function() {
    c(
        ""
    )
}

fileName <- file.path(system.file("scripts", package = "scMultiome"), "datasetList.Rmd")

test_that("file is created", {
    metadata <- file.path(system.file("extdata", "metadata.csv", package = "scMultiome"))
    metadata <- read.csv(metadata)
    expect_true(makeDataSetList(metadata))
    expect_true(file.exists(fileName))
})

# The purpose of this script is to describe how all datasets were obtained.

### Excerpt from ExperimentHub's vignette:
# A script describing the steps involved in making the data object(s).
# It can be code, pseudo-code, or text but should include
# where the original data were downloaded from, pre-processing, and how the final R object was made.
# Include a description of any steps performed outside of R with third party software.
# Output of the [process described in the] script should be files on disk ready to be pushed to data server.
###

# The creation of all datasets is described in detail in respective help files.
# Run listDatasets() or ?scMultiome to view a list of available datasets.
# See ?<DATASET_NAME> for details on particular datasets, e.g. ?prostateENZ.

# This script creates package metadata displayed in ExperimentHub.
# It also creates the data set manifest used by the listDatasets function.
# It also creates the data set list that will appear in the package help page.
# ExperimentHub DEMANDS that the metadata be in a data frame that is written to a csv file.
# See https://bioconductor.org/packages/3.15/bioc/vignettes/HubPub/inst/doc/CreateAHubPackage.html#overview for details.
# Describe your data set in a file called /inst/scripts/make-metadata-<DATASET_NAME>.R
#   by creating two 1-row data frames called metadata.<DATASET_NAME> and manifest.<DATASET_NAME>.
# See ?ExperimentHubData::makeExperimentHubMetadata and ?listDatasets for field details.

# When your file is ready, run this script.

# discover metadata files
files <- list.files(path = system.file("scripts", package = "scMultiome"),
                    full.names = TRUE,
                    pattern = "make-metadata-.+\\.R")
# create metadata items
lapply(files, source)


# collate manifest
manifest.all <- ls(envir = .GlobalEnv, pattern = "^manifest\\..+")
manifest <- do.call(rbind, mget(manifest.all))

# write manifest to file - DO NOT ALTER
utils::write.csv(manifest,
                 file.path(system.file("extdata", package = "scMultiome"), "manifest.csv"),
                 row.names = FALSE)


# collate metadata
metadata.all <- ls(envir = .GlobalEnv, pattern = "^metadata\\..+")
metadata <- do.call(rbind, mget(metadata.all))

# write metadata to file - DO NOT ALTER
fileLoc <- system.file("extdata", package = "scMultiome")
fileName <- "metadata.csv"
## add current package version to previous metadata file
if (file.exists(file.path(fileLoc, fileName))) {
    oldVersion <- paste0("_v", paste(strsplit(as.character(desc::desc_get("Version")), "\\.")[[1]][1:3], collapse = ""))
    oldFileName <- paste0("metadata", oldVersion, ".csv")
    file.rename(file.path(fileLoc, "metadata.csv"), file.path(fileLoc, oldFileName))
}
## write new metadata
utils::write.csv(metadata, file.path(fileLoc, fileName), row.names = FALSE)



# create data set list for package man page
makeDataSetList(metadata)



# clean up to prevent corruption on re-run
rm(list = ls(pattern = "^manifest.?"))
rm(list = ls(pattern = "^metadata.?"))
rm(files, fileLoc, fileName, oldFileName, oldVersion)

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.reprogramSeq <-
  data.frame(
    Title = "Reprogram-seq of LNCaP cells",
    Description = "scMultiome data of LNCaP infected with FOXA1, NKX2-1, GATA6",
    BiocVersion = "3.17",
    Genome = "hg38",
    SourceType = "HDF5", # refers to raw data
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/", # refers to raw data
    SourceVersion = "2022-10-06", # no commas!
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    TaxonomyId = "9606", # e.g. "9606"
    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
    RDataClass = "MultiAssayExperiment", # class that is returned from hub
    DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
    # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
    RDataPath = "scMultiome/reprogramSeq.h5"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.reprogramSeq <-
    data.frame(
        Call = "reprogramSeq()",
        Author = "Xie",
        Title = "Reprogram-seq of LNCaP cells",
        Species = "Homo sapiens", # e.g. "Homo sapiens"
        Lineage = "Prostate",
        CellNumber = "3903",
        Multiome = "paired",
        DiskSize = "430 MB",
        Version = "2022-10-04"
)

#'
#' TEAD inhibitor resistance
#'
#' Single cell multiomics on mesothelioma cell line H226 sensitive and resistant to the TEAD inhibitor GNE-7883
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the Hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix}: SingleCellExperiment with 6068436 rows and 4952 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 57765 rows and 4952 columns}
#'   \item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36451 rows and 4952 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 103723 rows and 4952 column}
#'   \item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 4952 columns}
#'   \item{\strong{TFPeaksDeviationsMatrix}: SingleCellExperiment with 1269 rows and 4952 columns}
#'   \item{\strong{TF_bindingMatrix}: SingleCellExperiment with 1504 rows and 4952 columns}
#' }
#'
#' @references
#' Manuscript under review...
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-TEADi_resistance.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' TEADi_resistance(metadata = TRUE)
#'
#' # download data
#'  \dontrun{
#' TEADi_resistance()
#' }
#'
#' @export
#'
TEADi_resistance <-
    function(metadata = FALSE,
             experiments = c("TileMatrix",
                             "GeneScoreMatrix",
                             "GeneExpressionMatrix",
                             "PeakMatrix",
                             "MotifMatrix",
                             "TFPeaksDeviationsMatrix",
                             "TF_bindingMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("TEADi_resistance", metadata, experiments, verbose = FALSE)
    }

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
# merged ChIP-seq
metadata.tfBinding.hg38 <- data.frame(
    Title = "TF Binding Info hg38 (ChIP-Atlas and ENCODE)",
    Description = "Combined transcription factor ChIP-seq data from ChIP-Atlas and ENCODE",
    BiocVersion = "3.17",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "GRangesList",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_atlas.rds"
)


metadata.tfBinding.hg19 <- data.frame(
    Title = "TF Binding Info hg19 (ChIP-Atlas and ENCODE)",
    Description = "Combined transcription factor ChIP-seq data from ChIP-Atlas and ENCODE",
    BiocVersion = "3.17",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "GRangesList",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_atlas.rds"
)

metadata.tfBinding.mm10 <- data.frame(
    Title = "TF Binding Info mm10 (ChIP-Atlas and ENCODE)",
    Description = "Combined transcription factor ChIP-seq data from ChIP-Atlas and ENCODE",
    BiocVersion = "3.17",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "GRangesList",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_atlas.rds"
)


# sample specific ChIP-Atlas
metadata.tfBinding.hg38.atlas.sample <- data.frame(
    Title = "TF Binding Info hg38 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample",
    BiocVersion = "3.19",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_atlas.sample.rds"
)

metadata.tfBinding.hg38.atlas.sample.v2 <- data.frame(
    Title = "TF Binding Info hg38 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_atlas.sample_v2.rds"
)

metadata.tfBinding.hg19.atlas.sample <- data.frame(
    Title = "TF Binding Info hg19 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample",
    BiocVersion = "3.19",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_atlas.sample.rds"
)

metadata.tfBinding.hg19.atlas.sample.v2 <- data.frame(
    Title = "TF Binding Info hg19 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_atlas.sample_v2.rds"
)


metadata.tfBinding.mm10.atlas.sample <- data.frame(
    Title = "TF Binding Info mm10 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample",
    BiocVersion = "3.19",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_atlas.sample.rds"
)

metadata.tfBinding.mm10.atlas.sample.v2 <- data.frame(
    Title = "TF Binding Info mm10 by sample (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_atlas.sample_v2.rds"
)

# sample  specific ENCODE
metadata.tfBinding.hg38.encode.sample <- data.frame(
    Title = "TF Binding Info hg38 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample",
    BiocVersion = "3.19",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_encode.sample.rds"
)

metadata.tfBinding.hg38.encode.sample.v2 <- data.frame(
    Title = "TF Binding Info hg38 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_encode.sample_v2.rds"
)

metadata.tfBinding.hg19.encode.sample <- data.frame(
    Title = "TF Binding Info hg19 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample",
    BiocVersion = "3.19",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_encode.sample.rds"
)

metadata.tfBinding.hg19.encode.sample.v2 <- data.frame(
    Title = "TF Binding Info hg19 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_encode.sample_v2.rds"
)


metadata.tfBinding.mm10.encode.sample <- data.frame(
    Title = "TF Binding Info mm10 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample",
    BiocVersion = "3.19",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_encode.sample.rds"
)

metadata.tfBinding.mm10.encode.sample.v2 <- data.frame(
    Title = "TF Binding Info mm10 by sample (ENCODE)",
    Description = "Transcription factor ChIP-seq data from ENCODE broken down by sample v2",
    BiocVersion = "3.21",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://www.encodeproject.org/",
    SourceVersion = "2021-09-21", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_encode.sample_v2.rds"
)

# tissue specific ChIP-Atlas
metadata.tfBinding.hg38.atlas.tissue <- data.frame(
    Title = "TF Binding Info hg38 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue",
    BiocVersion = "3.19",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_atlas.tissue.rds"
)

metadata.tfBinding.hg38.atlas.tissue.v2 <- data.frame(
    Title = "TF Binding Info hg38 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue v2",
    BiocVersion = "3.21",
    Genome = "hg38",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg38_atlas.tissue_v2.rds"
)


metadata.tfBinding.hg19.atlas.tissue <- data.frame(
    Title = "TF Binding Info hg19 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue",
    BiocVersion = "3.19",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_atlas.tissue.rds"
)

metadata.tfBinding.hg19.atlas.tissue.v2 <- data.frame(
    Title = "TF Binding Info hg19 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue v2",
    BiocVersion = "3.21",
    Genome = "hg19",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_hg19_atlas.tissue_v2.rds"
)

metadata.tfBinding.mm10.atlas.tissue <- data.frame(
    Title = "TF Binding Info mm10 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue",
    BiocVersion = "3.19",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_atlas.tissue.rds"
)

metadata.tfBinding.mm10.atlas.tissue.v2 <- data.frame(
    Title = "TF Binding Info mm10 by tissue (ChIP-Atlas)",
    Description = "Transcription factor ChIP-seq data from ChIP-Atlas broken down by tissue v2",
    BiocVersion = "3.21",
    Genome = "mm10",
    SourceType = "BED", # refers to raw data
    SourceUrl = "https://github.com/inutano/chip-atlas/",
    SourceVersion = "3.0", # no commas!
    Species = "Mus musculus",
    TaxonomyId = "10090",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech", # refers to raw data
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "List",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/tfBinding_mm10_atlas.tissue_v2.rds"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.tfBinding.hg38 <- data.frame(
    Call = "tfBinding(\"hg38\", \"atlas\")",
    Author = "ChipAtlas, ENCODE",
    Title = "TF Binding hg38 ChIPAtlas+ENCODE",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "280 MB",
    Version = "2022-09-20"
)


manifest.tfBinding.hg19 <- data.frame(
    Call = "tfBinding(\"hg19\", \"atlas\")",
    Author = "ChipAtlas, ENCODE",
    Title = "TF Binding hg19 ChIPAtlas+ENCODE",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "275 MB",
    Version = "2022-09-20"
)

manifest.tfBinding.mm10 <- data.frame(
    Call = "tfBinding(\"mm10\", \"atlas\")",
    Author = "ChipAtlas, ENCODE",
    Title = "TF Binding mm10 ChIPAtlas+ENCODE",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "160 MB",
    Version = "2022-09-20"
)


# sample specific ChIP-Atlas

manifest.tfBinding.hg38.atlas.sample <- data.frame(
    Call = "tfBinding(\"hg38\", \"atlas.sample\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding hg38 ChIPAtlas by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "660.2 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg38.atlas.sample.v2 <- data.frame(
    Call = "tfBinding(\"hg38\", \"atlas.sample\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding hg38 ChIPAtlas by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "585.2 MB",
    Version = "2024-12-08"
)

manifest.tfBinding.hg19.atlas.sample <- data.frame(
    Call = "tfBinding(\"hg19\", \"atlas.sample\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding hg19 ChIPAtlas by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "637.4 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg19.atlas.sample.v2 <- data.frame(
    Call = "tfBinding(\"hg19\", \"atlas.sample\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding hg19 ChIPAtlas by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "565.3 MB",
    Version = "2024-12-09"
)

manifest.tfBinding.mm10.atlas.sample <- data.frame(
    Call = "tfBinding(\"mm10\", \"atlas.sample\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding mm10 ChIPAtlas by sample",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "311.4 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.mm10.atlas.sample.v2 <- data.frame(
    Call = "tfBinding(\"mm10\", \"atlas.sample\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding mm10 ChIPAtlas by sample",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "253.4 MB",
    Version = "2024-12-08"
)



# sample specific ENCODE
manifest.tfBinding.hg38.encode.sample <- data.frame(
    Call = "tfBinding(\"hg38\", \"encode.tissue\", 1)",
    Author = "ENCODE",
    Title = "TF Binding hg38 ENCODE by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "167.2 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg38.encode.sample.v2 <- data.frame(
    Call = "tfBinding(\"hg38\", \"encode.tissue\", 2)",
    Author = "ENCODE",
    Title = "TF Binding hg38 ENCODE by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "140.2 MB",
    Version = "2024-12-08"
)

manifest.tfBinding.hg19.encode.sample <- data.frame(
    Call = "tfBinding(\"hg19\", \"encode.sample\", 1)",
    Author = "ENCODE",
    Title = "TF Binding hg19 ENCODE by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "168 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg19.encode.sample.v2 <- data.frame(
    Call = "tfBinding(\"hg19\", \"encode.sample\", 2)",
    Author = "ENCODE",
    Title = "TF Binding hg19 ENCODE by sample",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "140.9 MB",
    Version = "2024-12-12"
)

manifest.tfBinding.mm10.encode.sample <- data.frame(
    Call = "tfBinding(\"mm10\", \"encode.sample\", 1)",
    Author = "ENCODE",
    Title = "TF Binding mm10 ENCODE by sample",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "14.7 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.mm10.encode.sample.v2 <- data.frame(
    Call = "tfBinding(\"mm10\", \"encode.sample\", 2)",
    Author = "ENCODE",
    Title = "TF Binding mm10 ENCODE by sample",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "11.1 MB",
    Version = "2024-12-08"
)

# tissue specific ChIP-Atlas

manifest.tfBinding.hg38.atlas.tissue <- data.frame(
    Call = "tfBinding(\"hg38\", \"atlas.tissue\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding hg38 ChIPAtlas by tissue",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "466 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg38.atlas.tissue.v2 <- data.frame(
    Call = "tfBinding(\"hg38\", \"atlas.tissue\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding hg38 ChIPAtlas by tissue",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "426.9 MB",
    Version = "2024-12-08"
)

manifest.tfBinding.hg19.atlas.tissue <- data.frame(
    Call = "tfBinding(\"hg19\", \"atlas.tissue\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding hg19 ChIPAtlas by tissue",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "448.8 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.hg19.atlas.tissue.v2 <- data.frame(
    Call = "tfBinding(\"hg19\", \"atlas.tissue\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding hg19 ChIPAtlas by tissue",
    Species = "Homo sapiens",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "411.3 MB",
    Version = "2024-12-09"
)


manifest.tfBinding.mm10.atlas.tissue <- data.frame(
    Call = "tfBinding(\"mm10\", \"atlas.tissue\", 1)",
    Author = "ChipAtlas",
    Title = "TF Binding mm10 ChIPAtlas by tissue",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "231.6 MB",
    Version = "2024-09-26"
)

manifest.tfBinding.mm10.atlas.tissue <- data.frame(
    Call = "tfBinding(\"mm10\", \"atlas.tissue\", 2)",
    Author = "ChipAtlas",
    Title = "TF Binding mm10 ChIPAtlas by tissue",
    Species = "Mus musculus",
    Lineage = "All",
    CellNumber = "Bulk",
    Multiome = "n/a",
    DiskSize = "193 MB",
    Version = "2024-12-08"
)

#' @details
#' Single cell multiome data sets, paired and unpaired, were analyzed
#' with the \code{ArchR} package in order to obtain epiregulons.
#' \code{ArchR} projects were converted to \code{MultiAssayExperiment} objects.
#'
#' The creation of all datasets is described in detail in respective help files.
#' Run \code{listDatasets()} to view a list of available data sets or see \code{Datasets} below.
#' See \code{?<DATASET_NAME>} for details on particular data sets, e.g. \code{?prostateENZ}.
#'
#' @section Datasets:
#' ```{r child = system.file("scripts", "datasetList.Rmd", package = "scMultiome")}
#' ```
#'
# Bioconductor requires that AnnotationHub and ExperimentHub be fully imported
#' @import AnnotationHub
#' @import ExperimentHub
#'
#' @keywords internal
"_PACKAGE"

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.AR_drug <-
  data.frame(
    Title = "Response of prostate cancer cells to drug treatment",
    Description = "Gene expression and ATACseq data from 6 cell lines (LNCaP, VCaP, MDA, 22Rv1, DU145, H660) after treatment with AR-targeting drugs",
    BiocVersion = "3.20",
    Genome = "hg38",
    SourceType = "HDF5",
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE251977",
    SourceVersion = "Sep 09 2024",
    Species = "Homo sapiens",
    TaxonomyId = "9606",
    Coordinate_1_based = TRUE,
    DataProvider = "Genentech, Inc.",
    Maintainer = desc::desc_get_maintainer(),
    RDataClass = "MultiAssayExperiment",
    DispatchClass = "FilePath",
    RDataPath = "scMultiome/AR_drug.h5"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.AR_drug <-
  data.frame(
    Call = "AR_drug()",
    Author = "Xiaosai Yao",
    Title = "Response of prostate cancer cells to drug treatment",
    Species = "Homo sapiens",
    Lineage = "Prostate",
    CellNumber = "23118",
    Multiome = "paired",
    DiskSize = "3.3 GB",
    Version = "2024-09-09"
)

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.colonHealthy <- data.frame(
    Title = "Single-cell analysis of samples from healthy human colon",
    Description = "ATACseq and RNAseq data obtained by the colon tissues analysis. Samples were collected from adult human donors.",
    BiocVersion = "3.17",
    Genome = "hg38",
    SourceType = "tar.gz", # refers to raw data
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE165659&format=file", # refers to raw data; optional
    SourceVersion = "Feb 16 2021", # no commas!
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    TaxonomyId = "9606", # e.g. "9606"
    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE.
    DataProvider = "Stanford University", # refers to raw data
    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
    RDataClass = "MultiAssayExperiment", # class that is returned from hub
    DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
    # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
    RDataPath = "scMultiome/colonHealthy.h5"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.colonHealthy <- data.frame(
    Call = "colonHealthy()",
    Author = "Zhang",
    Title = "Healthy colon",
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    Lineage = "Colon",
    CellNumber = "59231",
    Multiome = "unpaired",
    DiskSize = "6.7 GB",
    Version = "2022-09-21"
)

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.tfMotifs.human <-
  data.frame(
    Title = "TF motifs human",
    Description = "Transcription factor motifs in the human genome from chromVARmotifs package",
    BiocVersion = "3.19",
    Genome = "",
    SourceType = "RDA", # refers to raw data
    SourceUrl = "https://github.com/GreenleafLab/chromVARmotifs/raw/master/data/human_pwms_v2.rda", # refers to raw data
    SourceVersion = "2", # no commas!
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    TaxonomyId = "9606", # e.g. "9606"
    Coordinate_1_based = NA, # can stay NA, defaults to TRUE
    DataProvider = "Stanford University School of Medicine", # refers to raw data
    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
    RDataClass = "PWMatrixList", # class that is returned from hub
    DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
    # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
    RDataPath = "scMultiome/human_pwms_v2.rds"
)

metadata.tfMotifs.mouse <-
    data.frame(
        Title = "TF motifs mouse",
        Description = "Transcription factor motifs in the mouse genome from chromVARmotifs package",
        BiocVersion = "3.19",
        Genome = "",
        SourceType = "RDA", # refers to raw data
        SourceUrl = "https://github.com/GreenleafLab/chromVARmotifs/raw/master/data/human_pwms_v2.rda", # refers to raw data
        SourceVersion = "2", # no commas!
        Species = "Mus musculus", # e.g. "Homo sapiens"
        TaxonomyId = "10090", # e.g. "9606"
        Coordinate_1_based = NA, # can stay NA, defaults to TRUE
        DataProvider = "Stanford University School of Medicine", # refers to raw data
        Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
        RDataClass = "PWMatrixList", # class that is returned from hub
        DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
        # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
        RDataPath = "scMultiome/mouse_pwms_v2.rds"
    )


# dataset manifest metadata
# see ?listDatasets for details
manifest.tfMotifs.human <-
  data.frame(
    Call = "tfMotifs('human')",
    Author = "Greenleaf Lab",
    Title = "TF motifs human",
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    Lineage = "n/a",
    CellNumber = "n/a",
    Multiome = "n/a",
    DiskSize = "256 KB",
    Version = "2"
)


manifest.tfMotifs.human <-
    data.frame(
        Call = "tfMotifs('mouse')",
        Author = "Greenleaf Lab",
        Title = "TF motifs human",
        Species = "Mus musculus", # e.g. "Homo sapiens"
        Lineage = "n/a",
        CellNumber = "n/a",
        Multiome = "n/a",
        DiskSize = "256 KB",
        Version = "2"
    )

#'
#' Single-cell analysis of samples from healthy human colon
#'
#' ATACseq and RNAseq data obtained by the colon tissues analysis. Samples were
#' collected from adult human donors.
#'
#' @inheritParams prostateENZ
#'
#' @inherit prostateENZ return
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project. Annotated with the Hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 59231 columns}
#'   \item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 19020 rows and 59231 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 59231 columns}
#'   \item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 59231 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 406946 rows and 59231 columns}
#' }
#'
#' @references
#' 1. Zhang K, Hocker JD, Miller M, Hou X, Chiou J, Poirion OB, Qiu Y, Li YE,
#' Gaulton KJ, Wang A, Preissl S, Ren B. A single-cell atlas of
#' chromatin accessibility in the human genome.
#' Cell. 2021 Nov 24;184(24):5985-6001.e19. doi: 10.1016/j.cell.2021.10.024.
#' Epub 2021 Nov 12. PMID: 34774128; PMCID: PMC8664161.
#'
#' 2. Becker, W.R., Nevins, S.A., Chen, D.C. et al. Single-cell analyses
#' define a continuum of cell state and composition changes in the malignant
#' transformation of polyps to colorectal cancer. Nat Genet 54, 985–995 (2022).
#'  https://doi.org/10.1038/s41588-022-01088-x
#'
#' @inheritSection prostateENZ Data storage and access
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-colonHealthy.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' colonHealthy(metadata = TRUE)
#' # download data
#' \dontrun{
#' colonHealthy()
#' }
#'
#' @export
#'
colonHealthy <-
    function(metadata = FALSE,
             experiments = c("TileMatrix",
                             "GeneIntegrationMatrix",
                             "GeneScoreMatrix",
                             "MotifMatrix",
                             "PeakMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("colonHealthy", metadata, experiments, verbose = FALSE)
    }

#'
#' Transcription factor motifs
#'
#' Transcription factor motifs sets from the `chromVARmotifs` R package
#'
#' This data set stores transcription factor motifs for human and
#' mouse genome, which can be used with the package epiregulon to compute scores of
#' transcription factor-regulatory element links.
#'
#' @inheritParams prostateENZ
#' @param species character string specifying the species of interest
#'
#' @return A list of position weight matrices, one for each transcription factor.
#'
#' @format
#' \code{PWMatrixList} object containing information on
#' transcription factor motifs.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{human_pwms_v2}: PWMatrixList object of length 1558}
#'   \item{\strong{mouse_pwms_v2}: PWMatrixList object of length 1558}
#' }
#'
#' @references
#' Schep, A., Wu, B., Buenrostro, J. &  Greenleaf, W. J. (2017) chromVAR:
#' inferring transcription-factor-associated accessibility from single-cell
#' epigenomic data.
#' \emph{Nature Methods} 14, 975–978.
#' \href{http://dx.doi.org/0.1038/nmeth.4401}{doi:0.1038/nmeth.4401}
#'
#' @section Data storage and access:
#' The transcription factor motifs are stored separately for each species in .rds files encoding \code{PWMatrixList}.
#' Data for both species are be accessed with the same function \code{tfMotifs}.
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-tfMotifs.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' tfMotifs("mouse", metadata = TRUE)
#' # download data
#' \dontrun{
#' tfMotifs("human")
#' }
#'
#' @export
#'
tfMotifs <- function(species = c("human", "mouse"),
                      metadata = FALSE) {
    checkmate::assertFlag(metadata)
    species <- match.arg(species, several.ok = FALSE)
    species <- c(human="Homo sapiens", mouse="Mus musculus")[species]
    eh <- AnnotationHub::query(ExperimentHub::ExperimentHub(),
                               pattern = c("scMultiome", "TF motifs", species))
    eh_ID <- eh$ah_id
    ans <-
        if (metadata) {
            eh[eh_ID]
        } else {
            readRDS(eh[[eh_ID]])
        }

    return(ans)
}

#' documentation templates
#'
#' Create templates for data set documentation.
#'
#' Functions to facilitate documenting a new data set.
#' Each function creates a file and attempts to open it in RStudio for editing.
#'
#' \code{makeMakeData} creates an Rmarkdown report called \code{inst/scripts/make-data-<dataset>.Rmd}.
#' \code{makeMakeMetadata} creates an R script called \code{inst/scripts/make-metadata-<dataset>.R}.
#' \code{makeR} creates an R file called \code{R/<dataset>.Rmd}.
#'
#' @param dataset name of data set as character string
#'
#' @return
#' All functions return TRUE invisibly.
#'
#' @name templates
#'
#' @rdname templates
#' @author Aleksander Chlebowski
#'
makeMakeData <- function(dataset) {
    checkmate::assertString(dataset)

    fileName <- file.path(system.file("scripts", package = "scMultiome"),
                          paste0("make-data-", dataset, ".Rmd"))
    file.create(fileName)
    writeLines(con = fileName, text = c(
        "---",
        "title: \"Your Data Set Title\"",
        "author: \"Your Name\"",
        paste("date:", Sys.Date()),
        "output:",
        "  BiocStyle::html_document:",
        "    titlecaps: false",
        "    toc_float: true",
        "---",
        "",
        "```{r setup, include = FALSE}",
        "knitr::opts_chunk$set(echo = TRUE, eval = FALSE)",
        "```",
        ""))

    if ("tools:rstudio" %in% search() && requireNamespace("rstudioapi", quietly = TRUE)) {
        rstudioapi::navigateToFile(fileName)
    } else {
        message(fileName, " created")
    }

    return(invisible(TRUE))
}



#' @rdname templates
#'
makeMakeMetadata <- function(dataset) {
    checkmate::assertString(dataset)

    fileName <- file.path(system.file("scripts", package = "scMultiome"),
                          paste0("make-metadata-", dataset, ".R"))
    file.create(fileName)

    writeLines(con = fileName, text = c(
        "",
        "# ExperimentHub metadata",
        "# see ?ExperimentHubData::makeExperimentHubMetadata for details",
        paste0("metadata.", dataset, " <-"),
        "  data.frame(",
        "    Title = \"<YOUR DATA TITLE>\",",
        "    Description = \"<YOUR DATA TITLE>\",",
        paste0("    BiocVersion = \"", as.character(utils::packageVersion("BiocVersion")[,seq_len(2)]), "\","),
        "    Genome = \"<GENOME VERSION USED IN ANALYSIS>\",",
        "    SourceType = \"<SOURCE FILE TYPE>\", # refers to raw data",
        "    SourceUrl = \"<SOURCE FILE LOCATION>\", # refers to raw data",
        "    SourceVersion = \"<VERSION NUMEBR OR DATE>\", # no commas!",
        "    Species = \"<SPECIES NAME>\", # e.g. \"Homo sapiens\"",
        "    TaxonomyId = \"<SPECIES ID>\", # e.g. \"9606\"",
        "    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE",
        "    DataProvider = \"<SOURCE DATA PROVIDER (INSTITUTION)>\", # refers to raw data",
        "    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer",
        "    RDataClass = \"MultiAssayExperiment\", # class that is returned from hub",
        "    DispatchClass = \"FilePath\", # format saved on disk; FilePath only returns file location",
        "    # Location_Prefix = \"\", # SKIP if data stored in the Bioconductor AWS S3",
        paste0("    RDataPath = \"scMultiome/", dataset, "\""),
        ")",
        "",
        "# dataset manifest metadata",
        "# see ?listDatasets for details",
        paste0("manifest.", dataset, " <-"),
        "  data.frame(",
        paste0("    Call = \"", dataset, "()\","),
        "    Author = \"<DATA SET AUTHOR>\",",
        "    Title = \"<YOUR DATA TITLE>\",",
        "    Species = \"<SPECIES NAME>\", # e.g. \"Homo sapiens\"",
        "    Lineage = \"<TISSUE OR ORGAN>\",",
        "    CellNumber = \"<NUMBER OF CELLS>\",",
        "    Multiome = \"<PAIRED OR UNPAIRED>\",",
        "    DiskSize = \"<H5 FILE SIZE ON DISK>\",",
        paste0("    Version = \"", Sys.Date(), "\""),
        ")"
    ))

    if ("tools:rstudio" %in% search() && requireNamespace("rstudioapi", quietly = TRUE)) {
        rstudioapi::navigateToFile(fileName)
    } else {
        message(fileName, " created")
    }

    return(invisible(TRUE))
}



#' @rdname templates
#'
makeR <- function(dataset) {
    checkmate::assertString(dataset)

    fileName <- file.path(system.file("R", package = "scMultiome"),
                          paste0(dataset, ".R"))
    file.create(fileName)

    writeLines(con = fileName, text = c(
        "#'",
        "#' <YOUR DATA SET TITLE>",
        "#'",
        "#' <YOUR DATA SET DESCRIPTION",
        "#'",
        "#' @inheritParams prostateENZ",
        "#'",
        "#' @inherit prostateENZ return",
        "#'",
        "#' @format",
        "#' <YOUR DATA FORMAT>",
        "#' \\code{MultiAssayExperiment} obtained from an \\code{ArchR} project.",
        "#' Annotated with the <YOUR GENOME> genome build.",
        "#' Contains the following experiments:",
        "#' \\itemize{",
        "#'   \\item{}",
        "#' }",
        "#'",
        "#' @references",
        "#' <YOUR DATA REFERENCE>",
        "#'",
        "#' @inheritSection prostateENZ Data storage and access",
        "#'",
        "#' @section Data preparation:",
        paste0("#' ```{r child = system.file(\"scripts\", \"make-data-", dataset, ".Rmd\", package = \"scMultiome\")}"),
        "#' ```",
        "#'",
        "#' @examples",
        paste0("#' ", dataset, "()"),
        "#'",
        "#' @export",
        "#'",
        paste(dataset, "<-"),
        "    function(metadata = FALSE,",
        "             experiments = c(\"TileMatrix500\",",
        "                             \"GeneScoreMatrix\",",
        "                             \"GeneIntegrationMatrix\",",
        "                             \"PeakMatrix\",",
        "                             \"MotifMatrix\")) {",
        "        checkmate::assertFlag(metadata)",
        "        experiments <- match.arg(experiments, several.ok = TRUE)",
        "",
        paste0("        retrieve(\"", dataset, "\", metadata, experiments, verbose = FALSE)"),
        "    }"
    ))

    if ("tools:rstudio" %in% search() && requireNamespace("rstudioapi", quietly = TRUE)) {
        rstudioapi::navigateToFile(fileName)
    } else {
        message(fileName, " created")
    }

    return(invisible(TRUE))
}

datasetName <- "fictitiousDataset"

ansMD <- makeMakeData(datasetName)
ansMM <- makeMakeMetadata(datasetName)
ansMR <- makeR(datasetName)

path.Rmd <- file.path(system.file("scripts", package = "scMultiome"), paste0("make-data-", datasetName, ".Rmd"))
path.metaR <- file.path(system.file("scripts", package = "scMultiome"), paste0("make-metadata-", datasetName, ".R"))
path.R <- file.path(system.file("R", package = "scMultiome"), paste0(datasetName, ".R"))

test_that("file ceration", {
    expect_true(ansMD)
    expect_true(ansMM)
    expect_true(ansMR)
})

test_that("files exist", {
    expect_true(file.exists(path.Rmd))
    expect_true(file.exists(path.metaR))
    expect_true(file.exists(path.R))
})


unlink(c(path.Rmd, path.metaR, path.R))

#'
#' LNCaP Cells Treated with Enzalutamide
#'
#' Single-cell ATAC sequencing of parental LNCaP cells (DMSO treated, the control),
#' LNCaP cells treated with 10µM enzalutamide for 48 hours,
#' and LNCaP-derived enzalutamide-resistant RES-A and RES-B cells.
#'
#' @param experiments character vector of matrices to return; see \code{Format}
#' @param metadata logical flag specifying whether to return data or metadata only
#'
#' @return
#' \code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
#' with assays stored as \code{DelayedMatrix} objects.
#' If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
#'
#' @format
#' \code{MultiAssayExperiment} obtained from an \code{ArchR} project.
#' Annotated with the Hg38 genome build.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 15522 columns}
#'   \item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 15522 columns}
#'   \item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 23525 rows and 15522 columns}
#'   \item{\strong{PeakMatrix}: SingleCellExperiment with 80210 rows and 15522 columns}
#'   \item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 15522 columns}
#' }
#'
#' @references
#' Single-cell ATAC and RNA sequencing reveal pre-existing and persistent cells
#' associated with prostate cancer relapse. 
#' Taavitsainen \emph{et al.}, \emph{Nature Communications} 2021 Sep 6;12(1):5307 
#' \href{https://pubmed.ncbi.nlm.nih.gov/34489465/}{doi: 10.1038/s41467-021-25624-1}
#'
#' @section Data storage and access:
#' The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
#' objects and they in turn are split into components, all of which are stored in a
#' single hdf5 file. Data and can be accessed with a special function that extracts
#' elements of the requested experiment(s), reassembles them, and builds an MAE.
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-prostateENZ.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' prostateENZ(metadata = TRUE)
#' # download data
#' \dontrun{
#' prostateENZ()
#' }
#'
#'
#' @export
#'
prostateENZ <-
    function(metadata = FALSE,
             experiments = c("TileMatrix",
                             "GeneScoreMatrix",
                             "GeneIntegrationMatrix",
                             "PeakMatrix",
                             "MotifMatrix")) {
        checkmate::assertFlag(metadata)
        experiments <- match.arg(experiments, several.ok = TRUE)

        retrieve("prostateENZ", metadata, experiments, verbose = FALSE)
    }

#'
#' TF Binding Info
#'
#' Combined transcription factor ChIP-seq data from ChIP-Atlas and ENCODE
#'
#' This is a special data set that stores transcription factor binding sites for human and
#' mouse genomic builds, which can be used with the package epiregulon to compute regulons.
#'
#' @inheritParams prostateENZ
#' @param genome character string specifying the genomic build
#' @param source character string specifying the ChIP-seq data source
#' @param version numeric indicating data version number (see Details)
#' @param peak_number numeric indicating threshold to be applied number of peaks
#' per transcription factor in the combined version of GenomicRanges
#' (from all samples and tissues).
#'
#' @details
#' In version 2 chipatlas sample and tissue GenomicRanges were build using experiments
#' with at least 2E7 unique reads and 1E3 peaks. In sample specific encode
#' chip-seq data a quality metrics was number of peaks only.
#'
#' @return A list of TF binding sites as a \code{GrangesList} object.
#'
#' @format
#' \code{GRangesList} object containing binding site information
#' of transcription factor ChIP-seq.
#' Contains the following experiments:
#' \itemize{
#'   \item{\strong{hg38_atlas}: GRangesList object of length 1558}
#'   \item{\strong{hg19_atlas}: GRangesList object of length 1558}
#'   \item{\strong{mm10_atlas}: GRangesList object of length 768}
#'   \item{\strong{hg38_encode.sample}: List object of length 171}
#'   \item{\strong{hg19_encode.sample}: List object of length 171}
#'   \item{\strong{mm10_encode.sample}: List object of length 31}
#'   \item{\strong{hg38_atlas.sample}: List object of length 1112}
#'   \item{\strong{hg19_atlas.sample}: List object of length 1112}
#'   \item{\strong{mm10_atlas.sample}: List object of length 517}
#'   \item{\strong{hg38_atlas.tissue}: List object of length 22}
#'   \item{\strong{hg19_atlas.tissue}: List object of length 22}
#'   \item{\strong{mm10_atlas.tissue}: List object of length 23}
#' }
#'
#' @references
#' ChIP-Atlas 2021 update: a data-mining suite for exploring epigenomic landscapes by
#' fully integrating ChIP-seq, ATAC-seq and Bisulfite-seq data.
#' Zou Z, Ohta T, Miura F, Oki S.
#' \emph{Nucleic Acids Research. Oxford University Press (OUP);} 2022.
#' \href{http://dx.doi.org/10.1093/nar/gkac199}{doi:10.1093/nar/gkac199}
#'
#' ChIP‐Atlas: a data‐mining suite powered by full integration of public ChIP‐seq data.
#' Oki S, Ohta T, Shioi G, Hatanaka H, Ogasawara O, Okuda Y, Kawaji H, Nakaki R, Sese J, Meno C.
#' \emph{EMBO}; Vol. 19, EMBO reports. 2018.
#' \href{http://dx.doi.org/10.15252/embr.201846255}{doi:10.15252/embr.201846255}
#'
#' ENCODE: {https://www.encodeproject.org/}
#'
#' @section Data storage and access:
#' Each genomic build is a separate \code{GRangesList} object, stored in a separate RDS file.
#' All genomic builds can be accessed with the same function \code{tfBinding}.
#'
#' @section Data preparation:
#' ```{r child = system.file("scripts", "make-data-tfBinding.Rmd", package = "scMultiome")}
#' ```
#'
#' @examples
#' # check metada of dataset
#' tfBinding("mm10", metadata = TRUE)
#' # download data
#' \dontrun{
#' tfBinding("mm10", "atlas")
#' }
#'
#' @export
#'
tfBinding <- function(genome = c("hg38", "hg19", "mm10"),
                      source = c("atlas", "encode.sample", "atlas.sample","atlas.tissue"),
                      metadata = FALSE,
                      version=1,
                      peak_number = 1000) {
    checkmate::assertFlag(metadata)
    genome <- match.arg(genome, several.ok = FALSE)
    source <- match.arg(source, several.ok = FALSE)
    checkmate::assert_choice(version, c(1,2))
    key <- paste0(c(genome, source), collapse=".")
    if(version==1){
        message("Retrieving chip-seq data, version 1")
        to_file_dict <- c(hg38.atlas="tfBinding_hg38_atlas.rds",
                          hg19.atlas = "tfBinding_hg19_atlas.rds",
                          mm10.atlas = "tfBinding_mm10_atlas.rds",
                          hg38.atlas.sample="tfBinding_hg38_atlas.sample.rds",
                          hg19.atlas.sample = "tfBinding_hg19_atlas.sample.rds",
                          mm10.atlas.sample = "tfBinding_mm10_atlas.sample.rds",
                          hg38.encode.sample = "tfBinding_hg38_encode.sample.rds",
                          hg19.encode.sample = "tfBinding_hg19_encode.sample.rds",
                          mm10.encode.sample = "tfBinding_mm10_encode.sample.rds",
                          hg38.atlas.tissue = "tfBinding_hg38_atlas.tissue.rds",
                          hg19.atlas.tissue = "tfBinding_hg19_atlas.tissue.rds",
                          mm10.atlas.tissue = "tfBinding_mm10_atlas.tissue.rds")
    }
    else{
        message("Version 2 of the chip-seq data is being retrieved. For reproducibility with the scMultiome version < 1.7.1 please set version = 1.")
        to_file_dict <- c(hg38.atlas="tfBinding_hg38_atlas.rds",
                          hg19.atlas = "tfBinding_hg19_atlas.rds",
                          mm10.atlas = "tfBinding_mm10_atlas.rds",
                          hg38.atlas.sample="tfBinding_hg38_atlas.sample_v2.rds",
                          hg19.atlas.sample = "tfBinding_hg19_atlas.sample_v2.rds",
                          mm10.atlas.sample = "tfBinding_mm10_atlas.sample_v2.rds",
                          hg38.encode.sample = "tfBinding_hg38_encode.sample_v2.rds",
                          hg19.encode.sample = "tfBinding_hg19_encode.sample_v2.rds",
                          mm10.encode.sample = "tfBinding_mm10_encode.sample_v2.rds",
                          hg38.atlas.tissue = "tfBinding_hg38_atlas.tissue_v2.rds",
                          hg19.atlas.tissue = "tfBinding_hg19_atlas.tissue_v2.rds",
                          mm10.atlas.tissue = "tfBinding_mm10_atlas.tissue_v2.rds")

    }

    eh <- AnnotationHub::query(ExperimentHub::ExperimentHub(),
                               pattern = c("scMultiome", "tfBinding", to_file_dict[key]))

    if (source %in% c("atlas")) {
        eh_ID <- sort(eh$ah_id)[1]
    } else {
        eh_ID <- eh$ah_id
    }


    ans <-
        if (metadata) {
            eh[eh_ID]
        } else {
            readRDS(eh[[eh_ID]])
        }

    if(version==2 && !grepl("(sample|tissue)", to_file_dict[key])){
        ans <- ans[unlist(lapply(ans,length)) >= peak_number]
    }

    return(ans)
}

#' manipulate GeneRanges
#'
#' Prepare and recover \code{GRanges} objects for and after storing.
#'
#' \code{GRanges} objects, which can be encountered in \code{rowRanges} slots of
#' \code{SingleCellExperiment}s, are stored as data frames (of type compound).
#'
#' \code{storeGR} converts \code{GRanges} to a data frames converts factors to characters.
#' \code{restoreGR} resets data types in the basic columns and re-instantiates \code{GRanges}.
#'
#' @name RGtools
#'
#' @param x object of class \code{GRanges}
#' @param df a \code{data.frame}
#'
#' @return
#' \code{storeGR} returns a data frame, \code{restoreGR} returns a \code{GRanges} object.
#'

#' @rdname GRtools
#'
#' @keywords internal
#'
storeGR <- function(x) {
    checkmate::assertClass(x, "GRanges")
    ans <- as.data.frame(x)
    rownames(ans) <- names(x)
    ans[vapply(ans, is.factor, logical(1L))] <- lapply(Filter(is.factor, ans), as.character)

    return(ans)
}

#' @rdname GRtools
#'
#' @keywords internal
#'
restoreGR <- function(df) {
    checkmate::assertDataFrame(df)
    basic <- c("seqnames", "start", "end", "width", "strand")
    df[basic] <- lapply(df[basic], function(x) methods::as(x, typeof(x)))
    ans <- GenomicRanges::makeGRangesFromDataFrame(df, keep.extra.columns=TRUE)

    return(ans)
}

# # NOTE:
# In the wild the reconverted GRanges object may not be identical to the original
# as factor levels will be arranged alphabetically.

gr0 <- GenomicRanges::GRanges(
    "seqnames" = S4Vectors::Rle(c("chr1", "chr1", "chr2", "chr3"), c(1, 3, 2, 4)),
    "ranges" = IRanges::IRanges(1:10, width = 10:1, names = head(letters, 10)),
    "strand" = S4Vectors::Rle(BiocGenerics::strand(c("-", "+", "*", "+", "-")), c(1, 2, 2, 3, 2)),
    "score" = 1:10,
    "GC" = seq(1, 0, length = 10))


# prepare for storage
gr0df <- storeGR(gr0)

# restore from storage
GR0 <- restoreGR(gr0df)

test_that("conversion of GR to data.frame", {
    expect_s3_class(gr0df, "data.frame")
    expect_false(any(vapply(gr0df, is.factor, logical(1L))))
})

test_that("reconversion of data.frame to GR", {
    expect_s4_class(GR0, "GRanges")
    expect_identical(GR0, gr0)
})

#' save and load data sets
#'
#' Functions to disassemble and save, and load and reassemble MultiAssayExperiment data sets.
#'
#' These are utilities for developers to add new data sets to the package.
#' Most will usually be called internally.
#'
#' \code{saveMAE} is used to save a \code{MultiAssayExperiment} to a hdf5 file.
#' It creates the \code{file} and passes individual \code{experiments} to \code{saveExp}.
#'
#' \code{saveExp} is called by \code{saveMAE} to disassemble experiment \code{exp}
#' and save its elements in \code{file}. A group hierarchy is created with
#' the top level group called \code{expName}, e.g. "GeneScoreMatrix",
#' and lower level groups to store specific elements:
#' \itemize{
#'   \item{\code{experiment} class is saved in group "class"}
#'   \item{assays are saved as sparse matrices in subgroup "assays", using \code{writeSparseMatrix}}
#'   \item{\code{colData} and \code{colnames} are saved in subgroup "properties"}
#'   \item{if \code{experiment} has \code{rownames}, the are saved in "properties"}
#'   \item{\code{rowData} is saved in "properties", unless it is an empty \code{DataFrame}}
#'   \item{if \code{experiment} has a \code{rowRanges} component,
#'         it is converted to a data frame and saved in "properties"}
#'   \item{if \code{experiment} has a \code{metadata} component,
#'         it is deparsed to a string and saved in "properties"}
#'   \item{if \code{experiment} has a \code{reducedDims} component,
#'         they are saved in subgroup "reducedDims"}
#'   \item{if \code{experiment} has a \code{altExps} component,
#'         they are saved in subgroup "altExps" by recursively calling \code{saveExp}}
#' }
#' \code{DataFrames} (e.g. \code{colData}, \code{rowData}, embeddings)
#' are converted to \code{data.frames} and saved as compound type.
#'
#' \code{loadMAE} is called by accessor functions to retrieve data. It locates the hdf5 file
#' in which the data set is stored and uses \code{loadExp} to extract the specified \code{experiments}.
#'
#' \code{loadExp} first checks which property elements are stored for the experiment in question,
#' loads all elements of the experiment and builds a \code{SummarizedExperiment} object.
#' If the experiment was originally a \code{SingleCellExperiment} or a subclass thereof,
#' that class as well as possible additional slots are restored.
#'
#' \code{testFile} can be used to test whether a data set loads correctly from a local file.
#' It calls \code{loadMAE} and extracts all experiment verbosely.
#'
#' \code{uploadFile} will upload a single file to Bioconductor's staging directory.
#'
#' @name fileOperations
#'
#' @param mae object of class \code{MultiAssayExperiment}
#' @param file path to a hdf5 file
#' @param experiments character string specifying which experiments to save/load
#' @param verbose logical flag specifying operation verbosity
#' @param overwrite logical flag specifying whether to allow overwriting the hdf5 file
#' @param exp an experiment object that inherits from class \code{SummarizedExperiment},
#'            usually a \code{SingleCellExperiment}
#' @param expName name of the experiment, i.e. name of the \code{ArchR} Matrix
#' @param sasToken access token to \code{endpoint}
#' @param endpoint Bioconductor's data bucket endpoint url
#'
#' @return
#' \code{saveExp} returns \code{TRUE} invisibly if the save was successful.
#' \code{saveMAE} returns a named list of \code{TRUE} values.
#' \code{loadExp} returns a \code{SingleCellExperiment} or an object of a subclass.
#' \code{loadMAE} returns a \code{MultiAssayExperiment}.
#' \code{testFile} returns the \code{MultiAssayExperiment} stored in \code{file} in its entirety.
#' \code{uploadFile} returns TRUE invisibly.
#'
#' @seealso
#' Vignette "rhdf5 - HDF5 interface for R" (\code{vignette} or \code{browseVignettes})
#' for details of hdf5 file construction.
#' \code{writeSparseMatrix} for details of saving sparse matrices.
#'
#' @examples
#'
#' # create dummy MultiAssayExperiment
#' mae <- scMultiome:::dummyMAE()
#'
#' fileName <- tempfile(fileext = ".h5")
#' saveMAE(mae, fileName)                                # save MAE
#' remae <- loadMAE(fileName, c("EXP1", "EXP2"), TRUE)   # load MAE (internal)
#' remae_exp1 <- loadMAE(fileName, "EXP1", TRUE)         # load MAE with one experiment (internal)
#'
#' # create dummy SingleCellExperiment
#' sce <- scMultiome:::dummySCE()
#'
#' saveExp(sce, "EXP3", fileName, TRUE)                  # save one experiment (internal)
#' resce <- loadExp(fileName, "EXP3", TRUE)              # load one experiment (internal)
#'
#' testFile(fileName)                                    # load whole MAE



#' @import MultiAssayExperiment
#'
#' @rdname fileOperations
#'
#' @export
#' @author Aleksander Chlebowski and Xiaosai
#'
saveMAE <- function(mae, file, experiments = NULL, verbose = TRUE, overwrite = FALSE) {
    # TODO:
    #     consider saving colData only once in root rather once for each experiment
    #     would save storage but interfere with listing available experiments
    checkmate::assertClass(mae, "MultiAssayExperiment")
    lapply(experiments(mae), checkmate::assertClass, classes = "SummarizedExperiment")
    checkmate::assertCharacter(experiments, null.ok = TRUE)
    checkmate::assertChoice(experiments, names(mae), null.ok = TRUE)
    checkmate::assertDirectoryExists(dirname(file), access = "w")
    checkmate::assertFlag(verbose)
    checkmate::assertFlag(overwrite)

    if (file.exists(file)) {
        if (overwrite) {
            if (verbose) message("file ", file, " exists and will be overwritten")
            unlink(file, force = TRUE)
        } else {
            stop("file ", file, " already exists")
        }
    }

    if (is.null(experiments)) experiments <- names(mae)

    # create file
    if(verbose) message("creating h5 file ")
    rhdf5::h5createFile(file)

    # save and track progress
    ans <- lapply(experiments, function(x) {
        if(verbose) message("saving experiment ", x)
        ans <- saveExp(mae[[x]], x, file, verbose)
        if (verbose) message(" ... done", "\n")
        return(ans)
    })
    names(ans) <- experiments

    return(ans)
}



#' @import MultiAssayExperiment
#'
#' @rdname fileOperations
#'
#' @export
#'
loadMAE <- function(file, experiments, verbose) {
    checkmate::assertFileExists(file, access = "r")
    assertHDF5(file)
    checkmate::assertCharacter(experiments)
    checkmate::assertFlag(verbose)

    if (verbose) {
        dataset <- dynGet("dataset",
                          ifnotfound = paste("stored in file",
                                             dynGet("file",
                                                    ifnotfound = stop("resource missing"))))
        message("loading dataset ", dataset)
    }

    # list experiments in file
    fileContents <- rhdf5::h5ls(file)
    expStored <- fileContents[fileContents[["group"]] == "/", "name"]
    lapply(experiments, checkmate::assertChoice, choices = expStored)

    # load files
    if (verbose) message("loading experiments")
    expList <- lapply(experiments, loadExp, file = file, verbose = verbose)
    names(expList) <- experiments

    # build mae
    if (verbose) message("building MultiAssayExperiment")
    mae <- MultiAssayExperiment::MultiAssayExperiment(experiments = expList)

    return(mae)
}



#' @rdname fileOperations
#'
#' @export
#'
methods::setGeneric(
    name = "saveExp",
    def = function(exp, expName, file, verbose) standardGeneric("saveExp"))



#' @import SummarizedExperiment
#' @import SingleCellExperiment
#'
#' @rdname fileOperations
#'
#' @export
#'
methods::setMethod(
    f = "saveExp",
    signature = "SummarizedExperiment",
    definition = function(exp, expName, file, verbose) {
        checkmate::assertString(expName)
        checkmate::assertFileExists(file, access = "w")
        assertHDF5(file)
        checkmate::assertFlag(verbose)

        # PART ONE: disassemble experiment
        if (verbose) message("\t dissassembling experiment")
        assays <- assays(exp)
        colData <- SummarizedExperiment::colData(exp)
        rowData <- SummarizedExperiment::rowData(exp)
        colnames <- colnames(exp)
        rownames <- rownames(exp)
        rowRanges <- SummarizedExperiment::rowRanges(exp)
        metadata <- S4Vectors::metadata(exp)

        # PART TWO: save experiment elements
        # create master group for this experiment
        rhdf5::h5createGroup(file, expName)

        # write object class
        if (verbose) message("\t writing class")
        rhdf5::h5write(
            obj = class(exp),
            file = file,
            name = sprintf("%s/class", expName))

        # write assays
        if (verbose) message("\t writing assays")
        rhdf5::h5createGroup(file, sprintf("%s/assays", expName))
        for (ass in names(assays)) {
            if (verbose) message("\t ... ", ass)
            alabaster.matrix::writeSparseMatrix(
                x = methods::as(assays[[ass]],"dgCMatrix"),
                file = file,
                name = sprintf("%s/assays/%s", expName, ass))
        }

        # write properties
        if (verbose) message("\t writing properties")
        rhdf5::h5createGroup(file, sprintf("%s/properties", expName))

        # colData is only saved if it has columns
        if (ncol(colData) != 0L) {
            if (verbose) message("\t ...colData")
            rhdf5::h5write(
                obj = as.data.frame(colData),
                file = file,
                name = sprintf("%s/properties/colData", expName))
        }

        # write colnames
        if (verbose) message("\t ...colnames")
        rhdf5::h5write(
            obj = colnames,
            file = file,
            name = sprintf("%s/properties/colnames", expName))

        # rowData is only saved if it has columns
        # column-less rowData will be built from rownames
        if (ncol(rowData) != 0L) {
            if (verbose) message("\t ...rowData")
            rhdf5::h5write(
                obj = as.data.frame(rowData),
                file = file,
                name = sprintf("%s/properties/rowData", expName))
        }

        # rownames are only saved if there are any
        if (!is.null(rownames)) {
            if (verbose) message("\t ...rownames")
            rhdf5::h5write(
                obj = rownames,
                file = file,
                name = sprintf("%s/properties/rownames", expName))
        }

        # write rowRanges
        if (inherits(rowRanges, "GRanges")) {
            if (verbose) message("\t ... rowRanges")
            rhdf5::h5write(
                obj = storeGR(rowRanges),
                file = file,
                name = sprintf("%s/properties/rowRanges", expName)
            )
        } else if (inherits(rowRanges, "DataFrame")) {
            if (verbose) message("\t ... rowRanges")
            rhdf5::h5write(
                obj = as.data.frame(rowRanges),
                file = file,
                name = sprintf("%s/properties/rowRanges", expName)
            )
         }

        # write metadata, if any
        if (!is.null(metadata)) {
            if (verbose) message("\t ... metadata")
            rhdf5::h5write(
                obj = deparse(metadata),
                file = file,
                name = sprintf("%s/properties/metadata", expName)
            )
        }

        return(invisible(TRUE))
    })



#' @import SingleCellExperiment
#'
#' @rdname fileOperations
#'
#' @export
#'
methods::setMethod(
    f = "saveExp",
    signature = "SingleCellExperiment",
    definition = function(exp, expName, file, verbose) {
        checkmate::assertString(expName)
        checkmate::assertFileExists(file, access = "w")
        assertHDF5(file)
        checkmate::assertFlag(verbose)

        methods::callNextMethod()

        # write reduced dimensions, if any
        reducedDims <- SingleCellExperiment::reducedDims(exp)

        if (length(reducedDims) > 0L) {
            if (verbose) message("\t writing reducedDims")
            rhdf5::h5createGroup(file, sprintf("%s/reducedDims", expName))
            for (red in names(reducedDims)) {
                if (verbose) message("\t ... ", red)
                rhdf5::h5write(
                    obj = reducedDims[[red]],
                    file = file,
                    name = sprintf("%s/reducedDims/%s", expName, red))
            }
        }

        # write alternative experiments, if any
        altExps <- SingleCellExperiment::altExps(exp)

        if (length(altExps) > 0L) {
            if (verbose) message("\t writing altExps")
            rhdf5::h5createGroup(file, sprintf("%s/altExps", expName))
            for (alt in names(altExps)) {
                if (verbose) message("\t ... ", alt)
                saveExp(exp = altExps[[alt]],
                        expName = sprintf("%s/altExps/%s", expName, alt),
                        file = file,
                        verbose = verbose)
            }
        }
        return(invisible(TRUE))
    })



#' @import SingleCellExperiment
#'
#' @rdname fileOperations
#'
#' @export
#'
loadExp <- function(file, expName, verbose) {
    checkmate::assertFileExists(file, access = "r")
    assertHDF5(file)
    checkmate::assertString(expName)
    checkmate::assertFlag(verbose)

    # load class
    expClass <- rhdf5::h5read(file, sprintf("%s/class", expName))

    if (verbose) message("loading ", expName)

    # list file contents
    fileContents <- rhdf5::h5ls(file)
    colnames.present <- is.element("colnames",
                                   fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    colData.present <- is.element("colData",
                                  fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    rownames.present <- is.element("rownames",
                                   fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    rowData.present <- is.element("rowData",
                                  fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    rowRanges.present <- is.element("rowRanges",
                                    fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    metadata.present <- is.element("metadata",
                                   fileContents[fileContents[["group"]] == sprintf("/%s/properties", expName), "name"])
    reducedDims.present <- is.element("reducedDims",
                                      fileContents[fileContents[["group"]] == sprintf("/%s", expName), "name"])
    altExps.present <- is.element("altExps",
                                  fileContents[fileContents[["group"]] == sprintf("/%s", expName), "name"])


    # load everything in group specified by experiment

    # load properties
    if (verbose) message("\t loading properties")

    # cell information
    if (colnames.present) {
        if (verbose) message("\t ... colnames")
        colnames <- rhdf5::h5read(file, sprintf("%s/properties/colnames", expName))
    }
    if (colData.present) {
        if (verbose) message("\t ... colData")
        colData <- rhdf5::h5read(file, sprintf("%s/properties/colData", expName))
    }

    # feature information
    if (rownames.present) {
        if (verbose) message("\t ... rownames")
        rownames <- rhdf5::h5read(file, sprintf("%s/properties/rownames", expName))
    }
    if (rowData.present) {
        if (verbose) message("\t ... rowData")
        rowData <- rhdf5::h5read(file, sprintf("%s/properties/rowData", expName))
    }

    # load row ranges if saved
    if (rowRanges.present) {
        if (verbose) message("\t ... rowRanges")
        rowRanges <- rhdf5::h5read(file = file, name = sprintf("%s/properties/rowRanges", expName))
        rowRanges <- restoreGR(rowRanges)
    }

    # load metadata if saved
    if (metadata.present) {
        if (verbose) message("\t ... metadata")
        metadata <- rhdf5::h5read(file = file, name = sprintf("%s/properties/metadata", expName))
        metadata <- eval(parse(text = metadata))
    }

    # load assays
    if (verbose) message("\t loading assays")
    assaysStored <- fileContents[fileContents[["group"]] == sprintf("/%s/assays", expName), "name"]
    assays <- lapply(assaysStored, function(ass) {
        if (verbose) message("\t ... ", ass)
        HDF5Array::H5SparseMatrix(filepath = file, group = sprintf("/%s/assays/%s", expName, ass))
    })
    names(assays) <- assaysStored

    # load reduced dimensions if saved
    if (reducedDims.present) {
        if (verbose) message("\t loading reducedDims")
        reducedDimsStored <- fileContents[fileContents[["group"]] == sprintf("/%s/reducedDims", expName), "name"]
        reducedDims <- lapply(reducedDimsStored, function(red) {
            if (verbose) message("\t ... ", red)
            rhdf5::h5read(file = file, name = sprintf("%s/reducedDims/%s", expName, red))
        })
        names(reducedDims) <- reducedDimsStored
    }

    # load alternative experiment
    if (altExps.present) {
        if (verbose) message("\t loading altExps")
        altExpsStored <- fileContents[fileContents[["group"]] == sprintf("/%s/altExps", expName), "name"]
        altExps <- lapply(altExpsStored, function(alt) {
            if (verbose) message("\t ... ", alt)
            loadExp(file = file,
                    expName = sprintf("%s/altExps/%s", expName, alt),
                    verbose = verbose)
        })
        names(altExps) <- altExpsStored
    }


    # rebuild experiment
    if (verbose) message("\t rebuilding experiment")
    ans <- SummarizedExperiment::SummarizedExperiment(assays = assays)
    if (colData.present) {
        if (verbose) message("\t ...colData")
        SummarizedExperiment::colData(ans) <- S4Vectors::DataFrame(colData)
    }
    if (colnames.present) {
        if (verbose) message("\t ...colnames")
        colnames(ans) <- colnames
    }
    if (rowData.present) {
        if (verbose) message("\t ...rowData")
        SummarizedExperiment::rowData(ans) <- S4Vectors::DataFrame(rowData)
    }
    if (rowRanges.present) {
        if (verbose) message("\t ...rowRanges")
        SummarizedExperiment::rowRanges(ans) <- rowRanges
    }
    if (rownames.present) {
        if (verbose) message("\t ...rownames")
        rownames(ans) <- rownames
    }
    if (metadata.present) {
        if (verbose) message("\t ...metadata")
        S4Vectors::metadata(ans) <- metadata
    }

    # if inherits from SCE, convert and add more slots
    if (methods::extends(expClass, "SingleCellExperiment")) {
        # direct conversion from (Ranged)SummarizedExperiment fails
        # due to different requirements for the int_elementMetadata slot
        ans <- methods::as(ans, "SingleCellExperiment")
        if (!methods::is(ans, expClass)) {
            ans <- methods::as(ans, expClass)
        }

        if (reducedDims.present) {
            if (verbose) message("\t ...reducedDims")
            SingleCellExperiment::reducedDims(ans, withDimnames = FALSE) <- reducedDims
        }
        if (altExps.present) {
            if (verbose) message("\t ...altExps")
            SingleCellExperiment::altExps(ans, withDimnames = FALSE) <- altExps
        }
    }

    return(ans)
}



#' @rdname fileOperations
#'
#' @export
#'
testFile <- function(file) {
    checkmate::assertFileExists(file, access = "r")
    assertHDF5(file)

    # list file contents
    fc <- rhdf5::h5ls(file)
    # list experiments (groups in root)
    experiments <- fc[fc$group == "/", "name"]

    message("TESTING loading MAE from file:\t", file)
    message("LOADING ALL EXPERIMENTS:\t", paste(experiments, collapse = ", "), "\n")
    ans <- loadMAE(file, experiments, verbose = TRUE)
    return(ans)
}



#' @rdname fileOperations
#'
#' @export
#'
uploadFile <- function(file, sasToken, endpoint = "https://bioconductorhubs.blob.core.windows.net") {
    checkmate::assertFileExists(file, access = "r")
    checkmate::assertString(sasToken)
    checkmate::assertString(endpoint)

    if (!requireNamespace("AzureStor")) {
        stop("uploading files requires package AzureStor, which is not insatlled")
    }

    message("establishing connection")
    # create endpoint object
    ep <- AzureStor::storage_endpoint(endpoint, sas = sasToken)
    # create container
    container <- AzureStor::storage_container(ep, "staginghub")

    message("files present in staging directory")
    # list files
    show(AzureStor::list_storage_files(container))

    message("commencing upload")
    # upload file
    AzureStor::storage_upload(
        container,
        src = normalizePath(file),
        dest = file.path("scMultiome", basename(file)))

    message("upload complete")
    # list files
    show(AzureStor::list_storage_files(container))

    return(invisible(TRUE))
}

data(iris)

# sce0 <- dummySCE("none")
# sce1 <- dummySCE("rowData")
# sce2 <- dummySCE("rowRanges")
# sce3 <- dummySCE("reducedDims")
# sce4 <- dummySCE("altExps")
# sce5 <- dummySCE()

mae <- MultiAssayExperiment(experiments = list(
    "EXP0" = dummySCE("none"),
    "EXP1" = dummySCE("rowData"),
    "EXP2" = dummySCE("rowRanges"),
    "EXP3" = dummySCE("reducedDims"),
    "EXP4" = dummySCE("altExps"),
    "EXP5" = dummySCE()
))


fileName <- tempfile(fileext = ".h5")
fileName2 <- tempfile(fileext = ".csv")
write.csv(head(iris), fileName2)

# saving
suppressMessages({
    ansSave <- saveMAE(mae, fileName, verbose = TRUE)
    ansSave2 <- saveMAE(mae, fileName, verbose = FALSE, overwrite = TRUE)
})
test_that("saving mae", {
    expect_true(is.list(ansSave))
    expect_true(all(vapply(ansSave, isTRUE, logical(1L))))
    expect_true(is.list(ansSave2))
    expect_true(all(vapply(ansSave2, isTRUE, logical(1L))))
    expect_true(file.exists(fileName))
})


# loading
ansLoad <- loadMAE(fileName, experiments = sprintf("EXP%i", 0:5), verbose = FALSE)

test_that("loading mae", {
    expect_s4_class(ansLoad, "MultiAssayExperiment")
    expect_identical(length(experiments(ansLoad)), 6L)
})
test_that("assay identity pre- and post", {
    expect_identical(assay(mae[[1]], 1), methods::as(assay(ansLoad[[1]], 1), "dgCMatrix"))
    expect_identical(assay(mae[[1]], 2), methods::as(assay(ansLoad[[1]], 2), "dgCMatrix"))
    expect_identical(assay(mae[[2]], 1), methods::as(assay(ansLoad[[2]], 1), "dgCMatrix"))
    expect_identical(assay(mae[[2]], 2), methods::as(assay(ansLoad[[2]], 2), "dgCMatrix"))
})


# loading with wrapper
suppressMessages(
    ansTest <- testFile(fileName)
)

test_that("loading mae with wrapper", {
    expect_identical(ansLoad, ansTest)
})



# failures
test_that("bad arguments", {
    expect_error(loadMAE(fileName2))
})

test_that("missing arguments", {
    expect_error(saveMAE(mae))
    expect_error(saveMAE(file = fileName))
    expect_error(loadMAE(fileName))
})

test_that("conflicts", {
    expect_error(saveMAE(mae, fileName, overwrite = FALSE))
})

unlink(fileName, force = TRUE)
unlink(fileName2, force = TRUE)

test_that("missing arguments, continued", {
    expect_error(loadMAE(fileName))
})

#' check if a file is a valid HDF5 file
#'
#' Check if a file path argument points to an non-corrupt HDF5 file.
#'
#' Compares the first 8 bytes of a file to those of the standard HDF5 file header.
#'
#' @param path path to file to test
#'
#' @return
#' Returns invisible \code{path} if check is successful, otherwise signals an error.
#'
#' @examples
#' fileName1 <- tempfile(fileext = ".h5")
#' rhdf5::h5createFile(fileName1)
#' rhdf5::h5write(mtcars, fileName1, "mtcars")
#' rhdf5::h5closeAll()
#' fileName2 <-  tempfile(fileext = ".csv")
#' write.csv(mtcars, fileName2)
#'
#' assertHDF5(fileName1)   # passes
#' \dontrun{
#' assertHDF5(fileName2)   # fails
#' }
#'
#' @section Further development:
#' The HDF5 file header contains 8 bytes, which hold specific meanings.
#' Currently the function only tests that the header of the file
#' specified by \code{path} is identical to a healthy HDF5 file and signals
#' a general error if that is not the case.
#' Reporting specific types of corruption can be implemented.
#'
#' @references
#' <http://web.ics.purdue.edu/~aai/HDF5/html/H5.format.html#BootBlock>
#'
#' @export
#' @author Aleksander Chlebowski
assertHDF5 <- function(path) {

  fileHead <- readBin(path, "raw", n = 8L)
  hdf5Head <- as.raw(c(137L, 72L, 68L, 70L, 13L, 10L, 26L, 10L))

  ident <- identical(fileHead, hdf5Head)

  if (ident) {
    return(invisible(path))
  } else {
    stop("Assertion on \"path\" failed: Must be a hdf5 file.", call. = FALSE)
  }
}

data(mtcars)

# create hdf5 file
fileName1 <- tempfile(fileext = ".h5")
rhdf5::h5createFile(fileName1)
rhdf5::h5write(mtcars, fileName1, "mtcars")
rhdf5::h5closeAll()

# create a different file
fileName2 <-  tempfile(fileext = ".csv")
write.csv(mtcars, fileName2)


test_that("passing assertion", {
    expect_invisible(assertHDF5(fileName1))
    expect_type(assertHDF5(fileName1), "character")
})
test_that("failing assertion", {
    expect_error(assertHDF5(fileName2))
})

#' create dummy data sets
#'
#' Create dummy SCE and MAE objects.
#'
#' @param features character string specifying which (optional) features to create in the SCE
#' @param experiments named list of character vectors
#'                    specifying experiments to create and their features
#'
#' @return
#' \code{dummySCE} returns a \code{SingleCellExperiment}.
#' \code{dummyMAE} returns a \code{MultiAssayExperiment}.
#'
#' @name dummies
#'
#' @examples
#' scMultiome:::dummySCE()
#' scMultiome:::dummySCE("rowData")
#' scMultiome:::dummySCE("rowRanges")
#' scMultiome:::dummySCE("reducedDims")
#' scMultiome:::dummySCE("altExps")
#'
#' scMultiome:::dummyMAE(list("dummyExperiment" = NULL))
#'



#' @import SingleCellExperiment
#'
#' @keywords internal
#' @rdname dummies
#' @export
#'
dummySCE <- function(features = c("rowData", "rowRanges", "reducedDims", "altExps", "none")) {
    features <- match.arg(features, several.ok = TRUE)

    ncells <- 10
    nfeats <- 20

    u <- matrix(stats::rpois(nfeats * ncells, 5), ncol = ncells)
    v <- log2(u + 1)

    u <- methods::as(u, "dgCMatrix")
    v <- methods::as(v, "dgCMatrix")

    pca <- matrix(stats::runif(ncells * 5), ncells)
    tsne <- matrix(stats::rnorm(ncells * 2), ncells)

    ans <- SingleCellExperiment(assays = list(counts = u, logcounts = v))

    rownames(ans) <- paste("feat", seq_len(nfeats), sep = "_")
    colnames(ans) <- paste("cell", seq_len(ncells), sep = "_")
    colData(ans) <- S4Vectors::DataFrame(
        "Cell" = colnames(ans),
        "one" = sample(letters, ncells),
        "two" = stats::rnorm(ncells),
        row.names = colnames(ans)
    )
    if (is.element("rowRanges", features)) {
        rowRanges(ans) <- GenomicRanges::GRanges(
            data.frame(
                "seqnames" = paste("chr", seq_along(rownames(ans))),
                "start" = seq_along(rownames(ans)),
                "end" = seq_along(rownames(ans)) * 5,
                "width" = seq_along(rownames(ans)) * 5,
                "strand" = rep_len(c("+", "-"), length(rownames(ans))),
                row.names = rownames(ans)
            )
        )
    }
    if (is.element("rowData", features)) {
        rowData(ans) <- S4Vectors::DataFrame(
            "Feat" = paste("chr", seq_along(rownames(ans))),
            "idx" = seq_along(rownames(ans)),
            row.names = rownames(ans)
        )
    }
    if (is.element("reducedDims", features)) {
        reducedDims(ans) <- S4Vectors::SimpleList(PCA = pca, tSNE = tsne)
    }
    if (is.element("altExps", features)) {
        alt.mat.1 <- matrix(stats::rpois(nfeats * ncells, 5), ncol = ncells)
        colnames(alt.mat.1) <- colnames(ans)

        alt.mat.2 <- matrix(stats::rpois(nfeats * ncells, 5), ncol = ncells)
        colnames(alt.mat.2) <- colnames(ans)

        altExp(ans, "Hash") <- SingleCellExperiment(list(counts = alt.mat.1))
        altExp(ans, "Protein") <- SingleCellExperiment(list(counts = alt.mat.2))
    }

    return(ans)
}



#' @import MultiAssayExperiment
#'
#' @keywords internal
#' @rdname dummies
#' @export
#'
dummyMAE <- function(experiments = list("EXP1" = NULL, "EXP2" = NULL)) {
    checkmate::assertList(experiments)
    lapply(experiments, checkmate::assertCharacter, null.ok = TRUE)

    experiments <- lapply(experiments, function(x) {
        if (is.null(x)) c("rowData", "rowRanges", "reducedDims", "altExps") else x
    })

    expList <- lapply(experiments, dummySCE)
    mae <- MultiAssayExperiment(experiments = expList)
    return(mae)
}

# helper function for comparing SCE slots
f <- function(sce) {
    ans <- logical(4L)
    names(ans) <- c("rowData", "rowRanges", "reducedDims", "altExps")
    ans["rowData"] <- prod(dim(SummarizedExperiment::rowData(sce))) > 0L
    ans["rowRanges"] <- inherits(SummarizedExperiment::rowRanges(sce), "GRanges")
    ans["reducedDims"] <- length(SingleCellExperiment::reducedDims(sce)) > 0L
    ans["altExps"] <- length(SingleCellExperiment::altExps(sce)) > 0L
    return(names(ans[ans]))
}


test_that("dummy SCE is created with all features", {
    sce1 <- dummySCE()
    sce2 <- dummySCE(features = c("rowData", "rowRanges", "reducedDims", "altExps"))
    expect_s4_class(sce1, "SingleCellExperiment")
    expect_s4_class(sce2, "SingleCellExperiment")
    expect_identical(f(sce1), f(sce2))
})

test_that("dummy SCE can be created with some features", {
    sce1 <- dummySCE(features = c("rowData"))
    sce2 <- dummySCE(features = c("rowData", "rowRanges"))
    sce3 <- dummySCE(features = c("rowData", "rowRanges", "reducedDims"))
    sce4 <- dummySCE(features = c("rowRanges", "altExps"))
    expect_identical(f(sce1), c("rowData"))
    expect_identical(f(sce2), c("rowData", "rowRanges"))
    expect_identical(f(sce3), c("rowData", "rowRanges", "reducedDims"))
    expect_identical(f(sce4), c("rowRanges", "altExps"))
})




test_that("dummy MAE is created", {
  mae <- dummyMAE()
  expect_s4_class(mae, "MultiAssayExperiment")
  expect_equal(length(mae), 2L)
})

test_that("dummy MAE is created containing SCEs with features", {
  mae <- dummyMAE()
  expect_s4_class(mae, "MultiAssayExperiment")
  expect_length(mae, 2L)

  mae <- dummyMAE(experiments = list("E1" = c("rowRanges", "reducedDims"),
                                     "E2" = c("reducedDims", "altExps"),
                                     "E3" = c("rowData", "altExps", "rowRanges")))
  features <- lapply(MultiAssayExperiment::experiments(mae), f)
  expect_identical(vapply(features, length, integer(1L), USE.NAMES = FALSE), c(2L, 2L, 3L))
  expect_identical(names(mae), c("E1", "E2", "E3"))
})




test_that("bad arguments", {
    expect_error(
        dummySCE("all")
    )
    expect_error(
        dummyMAE(experiments = c("E1", "E2", "E3"))
    )
})

# ExperimentHub metadata
# see ?ExperimentHubData::makeExperimentHubMetadata for details
metadata.hematopoiesis <-
  data.frame(
    Title = "scATAC-seq and unpaired scRNA-seq of hematopoetic cells",
    Description = "Example scATAC-seq data of hematopoietic cells included in ArchR package was integrated with scRNAseq. ScATAC-seq data was obtained from GSE139369 and scRNA-seq obtained from https://jeffgranja.s3.amazonaws.com/ArchR/TestData/scRNA-Hematopoiesis-Granja-2019.rds",
    BiocVersion = "3.17",
    Genome = "hg19",
    SourceType = "FASTQ", # refers to raw data
    SourceUrl = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE139369", # refers to raw data
    SourceVersion = "2019-10-29", # no commas!
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    TaxonomyId = "9606", # e.g. "9606"
    Coordinate_1_based = TRUE, # can stay NA, defaults to TRUE
    DataProvider = "Stanford", # refers to raw data
    Maintainer = desc::desc_get_maintainer(), # refers to package maintainer
    RDataClass = "MultiAssayExperiment", # class that is returned from hub
    DispatchClass = "FilePath", # format saved on disk; FilePath only returns file location
    # Location_Prefix = "", # SKIP if data stored in the Bioconductor AWS S3
    RDataPath = "scMultiome/hematopoiesis"
)

# dataset manifest metadata
# see ?listDatasets for details
manifest.hematopoiesis <-
  data.frame(
    Call = "hematopoiesis()",
    Author = "Granje",
    Title = "scATAC-seq and unpaired scRNA-seq of hematopoetic cells",
    Species = "Homo sapiens", # e.g. "Homo sapiens"
    Lineage = "blood",
    CellNumber = "10250",
    Multiome = "unpaired",
    DiskSize = "1.0 GB",
    Version = "2019-10-29"
)

fileName <- system.file("extdata", "manifest.csv", package = "scMultiome")

test_that("locating manifest file", {
    expect_true(file.exists(fileName))
})

test_that("DataFrame is returned", {
    ans <- listDatasets()
    expect_s4_class(ans, "DataFrame")
})

---
title: "AR Drug Data Set"
author: "Tomasz Włodarczyk"
date: 9 September 2024
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```


Data can be downloaded Gene Expression Omnibus
(acc. no. [GSE251977](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE251977))

Downstream analysis was performed with the \code{ArchR} package:

#### 1. Initiate `ArchR` project
```{r}
# attach ArchR package
library(ArchR)

# configure ArchR
addArchRGenome("hg38")

# create arrow file from fragment files
## list fragment files
fragments <- <FRAGMENT_FILES> # available in Gene Expression Omnibus (GSE251977)
## assign sample names
names(fragments) <- <SAMPLE_IDs>
## create arrows
createArrowFiles(inputFiles = fragments, sampleNames = names(fragments),
                     minTSS = 4, minFrags = 1000)

# specify output directory
outDir <- <OUTPUT_DIRECTORY>

# locate arrow files
arrows <- <ARROW_FILES>
    
doublet.score <- addDoubletScores(
    input = arrows,
    k = 10, #Refers to how many cells near a 'pseudo-doublet' to count.
    knnMethod = 'UMAP', #Refers to the embedding to use for nearest neighbor search.
    LSIMethod = 1
)

# create ArchR project
project <- ArchRProject(arrows, outDir)

```

#### 2. HTO demultiplexing
```{r}
library(ggplot2)
library(DropletUtils)
library(SingleCellExperiment)
library(Matrix)
library(ArchR)
library(zellkonverter)
library(scater)
library(gridExtra)
library(BiocParallel)


temp_dir <- tempdir()

destfiles_HTO <- c(file.path(temp_dir, "LIB5458339_SAM24418230.hashing.csv"),
                   file.path(temp_dir, "LIB5458340_SAM24418231.hashing.csv"),
                   file.path(temp_dir, "LIB5463784_SAM24425416.hashing.csv"),
                   file.path(temp_dir, "LIB5467656_SAM24427130.hashing.csv")
                   )

download.file(c("https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE251977&format=file&file=GSE251977%5FLIB5458339%5FSAM24418230%2Ehashing%2Ecsv%2Egz",
                "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE251977&format=file&file=GSE251977%5FLIB5458340%5FSAM24418231%2Ehashing%2Ecsv%2Egz",
                "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE251977&format=file&file=GSE251977%5FLIB5463784%5FSAM24425416%2Ehashing%2Ecsv%2Egz",
                "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE251977&format=file&file=GSE251977%5FLIB5467656%5FSAM24427130%2Ehashing%2Ecsv%2Egz"),
              "",
              method = "libcurl"
              )

# download GEX data

destfiles_GEX <- c(file.path(temp_dir, "LIB5458339_SAM24418230_raw_feature_bc_matrix.h5"),
                   file.path(temp_dir, "LIB5458340_SAM24418231_raw_feature_bc_matrix.h5"),
                   file.path(temp_dir, "LIB5463784_SAM24425416_raw_feature_bc_matrix.h5"),
                   file.path(temp_dir, "LIB5467656_SAM24427130_raw_feature_bc_matrix.h5")
                   )

download.file(c(),destfiles_GEX,method = "libcurl")

HTO_uri <- data.frame(SAMID = c("SAM24418230", "SAM24418231", "SAM24425416", "SAM24427130"), HTO_uri = destfiles_HTO)

arcseq_info <-data.frame(SAMID = c("SAM24418230", "SAM24418231", "SAM24425416", "SAM24427130"), uri = destfiles_GEX)

# merge arcseq and HTO paths
merge_file_info <- merge(HTO_uri, arcseq_info, by.x = "SAMID" , by.y = "sampleName")
merge_file_info <- merge_file_info[order(merge_file_info$SAMID), ]



######## load matrices
# load RNA matrix

hashing_qc <- list()
umapplot <- list()
seRNA_final <- list()
for (i in seq_len(nrow(merge_file_info))) {
    message(merge_file_info$SAMID[i])
    # import gex
    seRNA <- ArchR::import10xFeatureMatrix(
        input = file.path( merge_file_info$uri[i], "raw_feature_bc_matrix.h5"),
        names = merge_file_info$SAMID[i])
    names(assays(seRNA)) <- "counts"

    # import HTO and convert to sce
    HTO <- data.table::fread(merge_file_info$HTO_uri[i])
    rownames_HTO <- HTO$Antibody
    HTO <- HTO[,-1]
    colnames(HTO) <- paste0(merge_file_info$SAMID[i], "#", colnames(HTO),"-1")
    HTO <- as(as.matrix(HTO), "dgCMatrix")
    rownames(HTO) <- rownames_HTO
    HTO <- SingleCellExperiment(assays = list(counts=HTO))

    # merge GEX and HTO into a SCE
    common_cells <- intersect(colnames(HTO), colnames(seRNA))
    HTO <- HTO[, common_cells]
    seRNA <- seRNA[, common_cells]
    seRNA <- as(seRNA, "SingleCellExperiment")
    altExp(seRNA, "HTO") <- HTO

    # call empty droplets to define ambient droplets
    set.seed(10010)
    e.out.gene <- emptyDrops(counts(seRNA), by.rank = 30000 )
    is.cell <- e.out.gene$FDR <= 0.001
    summary(is.cell)

    # plot empty droplet assignments
    par(mfrow=c(1,2))
    r <- rank(-e.out.gene$Total)
    plot(r, e.out.gene$Total, log="xy", xlab="Rank", ylab="Total gene count", main="")
    abline(h=metadata(e.out.gene)$retain, col="darkgrey", lty=2, lwd=2)
    hist(log10(e.out.gene$Total[is.cell]), xlab="Log[10] gene count", main="")

    # Estimate HTO ambient proportions using empty droplets
    hto.mat <- assay(altExp(seRNA),"counts")[,which(is.cell)]
    ambient <- proportions(rowSums(assay(altExp(seRNA), "counts")[,is.na(e.out.gene$FDR)]))
    # plot ambient proportions
    barplot(ambient,las=2, main="ambient proportion")
    hash.stats <- hashedDrops(hto.mat, ambient=ambient)
    table(hash.stats$Best[hash.stats$Confident])

    # examine hashing
    colors <- rep("grey", nrow(hash.stats))
    colors[hash.stats$Doublet] <- "red"
    colors[hash.stats$Confident] <- "black"

    hashing_qc[[merge_file_info$SAMID[i]]] <- plot(hash.stats$LogFC, hash.stats$LogFC2,
                                                   xlab="Log fold-change from best to second HTO",
                                                   ylab="Log fold-change of second HTO over ambient",
                                                   col=colors,
                                                   main=merge_file_info$SAMID[i])


    # keep only non-empty cells
    seRNA <- seRNA[, which(is.cell)]
    colData(seRNA) <- cbind(colData(seRNA), hash.stats)
    colData(seRNA)$library <- sapply(strsplit(colnames(seRNA), split = "#"), "[",1)

    assay(altExp(seRNA), "logcounts") <- log10(assay(altExp(seRNA), "counts")+1)
    assay(altExp(seRNA), "clr") <- sweep(assay(altExp(seRNA), "logcounts"), 2,
                                         colMeans(assay(altExp(seRNA), "logcounts")), "-")
    seRNA <- runUMAP(seRNA, altexp = "HTO", name="UMAP_HTO", assay.type = "clr", exprs_values = "clr")
    seRNA$hash_assignment <- rownames_HTO[seRNA$Best]
    umapplot[[merge_file_info$SAMID[i]]] <- plotReducedDim(seRNA[, which(seRNA$Doublet == FALSE & seRNA$Confident == TRUE)],
                                                           dimred = "UMAP_HTO",
                                                           color_by = "hash_assignment",
                                                           point_size=0.5, rasterise=TRUE) +
        ggtitle(merge_file_info$SAMID[i])

    # save seRNA
    seRNA_final[[merge_file_info$SAMID[i]]] <- seRNA
}

seRNA_final <- do.call(cbind, seRNA_final)


library(ArchR)
seRNA_final$hash_assignment2 <- paste0(seRNA_final$library, seRNA_final$hash_assignment)


# copy ArchR project from /gstore/data/genomics/congee_rest_runs/6679b364f79a145e53521a61/ArchR_output/croo_output/ArchRProject
proj <- ArchR::loadArchRProject("OUTPUT/ArchRProject/")
common <- intersect(proj$cellNames, colnames(seRNA_final))
proj <- proj[common,]

# add HTO information

for (row_data in colnames(colData(seRNA_final))){
    proj <- addCellColData(
        ArchRProj = proj,
        data = colData(seRNA_final)[common, row_data],
        cells = common,
        name = row_data,
        force = TRUE
    )
}

#filter out doublets and non-confident calls
proj <- proj[which(proj$Confident == TRUE & proj$Doublet == FALSE), ]
```

#### 3. Clustering
```{r}

# filter doublets
archr.proj <- filterDoublets(
    ArchRProj = archr.proj,
    cutEnrich = 1,
    cutScore = -Inf,
    filterRatio = 1
)

# filter cells that do not contain rna
archr.proj <- archr.proj[!is.na(archr.proj$Gex_nUMI)]

# add reduced dims
archr.proj <- addIterativeLSI(
    ArchRProj = archr.proj,
    useMatrix = 'TileMatrix',
    name = 'IterativeLSI_TileMatrix',
    threads = 4,
    seed = 2,
    force = TRUE
)

archr.proj <- addIterativeLSI(
    ArchRProj = archr.proj,
    useMatrix = 'GeneExpressionMatrix',
    name = 'IterativeLSI_GeneExpressionMatrix',
    firstSelection = "variable",
    depthCol = "Gex_nUMI",
    varFeatures = 2500,
    binarize = FALSE,
    threads = 4,
    seed = 2,
    force = TRUE
)

archr.proj <- addCombinedDims(
    archr.proj,
    reducedDims = c('IterativeLSI_TileMatrix', 'IterativeLSI_GeneExpressionMatrix'),
    name =  'IterativeLSI_Combined'
)


# add clusters
archr.proj <- addClusters(
input = archr.proj,
reducedDims = 'IterativeLSI_Combined',
name = 'Clusters_Combined',
seed = 2,
prefix = 'Combined_C',
force = TRUE,
method = "scran"
)
cM <- confusionMatrix(getCellColData(archr.proj)[,'hash_assignment2'], archr.proj$Sample)
cM <- cM / Matrix::rowSums(cM)
pheatmap::pheatmap(
    mat = as.matrix(cM),
    color = paletteContinuous('whiteBlue'),
    border_color = 'black',
    filename = paste0(getOutputDirectory(archr.proj),'/Plots/GPSA-Sample-hash_assignment2-pheatmap.pdf')
)


# add embeddings

archr.proj <- addUMAP(
    ArchRProj = archr.proj,
    reducedDims = 'IterativeLSI_Combined',
    name = 'UMAP_Combined',
    seed = 2,
    threads = 1,
    force = TRUE
)

archr.proj <- addUMAP(
    ArchRProj = archr.proj,
    reducedDims = 'IterativeLSI_TileMatrix',
    name = 'UMAP_ATAC',
    seed = 2,
    threads = 1,
    force = TRUE
)


archr.proj <- addUMAP(
    ArchRProj = archr.proj,
    reducedDims = 'IterativeLSI_GeneExpressionMatrix',
    name = 'UMAP_RNA',
    seed = 2,
    threads = 1
)



# tSNE
archr.proj <- addTSNE(
    ArchRProj = archr.proj,
    reducedDims = 'IterativeLSI_Combined',
    name = 'TSNE_Combined',
    perplexity = 30,
    seed = 2,
    force = TRUE,
    threads = max(floor(4/2), 1)
)

```

#### 4. Clustering and motif annotation
```{r}
# Peak calling

library(BSgenome.Hsapiens.Genentech.GRCh38)

archr.proj <- addGroupCoverages(
    ArchRProj = archr.proj,
    groupBy = 'hash_assignment2',
    threads = 4
)


archr.proj <- ArchR.helper::addReproduciblePeakSet(
    ArchRProj = archr.proj,
    groupBy = 'hash_assignment2',
    peakMethod = "MACSr",
    excludeChr = c('chrMT','chrY'),
    genomeSize = 2.7e9,
    threads = 4,
    force = TRUE
)

archr.proj <- addPeakMatrix(
    ArchRProj = archr.proj,
    binarize = FALSE,
    threads = 1,
    force = TRUE
)

# TF annotation
peaks.anno <- scMultiome::tfBinding()
archr.proj <- addPeakAnnotations(
    ArchRProj = archr.proj,
    regions = peaks.anno,
    name = 'TF_peaks',
    force = TRUE
)

archr.proj <- addDeviationsMatrix(
    ArchRProj = archr.proj,
    peakAnnotation = 'TF_peaks',
    matrixName = 'TFPeaksDeviationsMatrix',
    threads = 1,
    force = TRUE
)


# motif annotation
archr.proj  <- addMotifAnnotations(ArchRProj = archr.proj, motifSet = 'cisbp', name = 'Motif', species='Homo sapiens')

archr.proj <- addDeviationsMatrix(
    ArchRProj = archr.proj,
    peakAnnotation = 'Motif',
    threads = 1,
    force = TRUE
)



#add BigWigs

getGroupBW(
    ArchRProj = archr.proj,
    groupBy = "hash_assignment2",
    normMethod = "ReadsInTSS",
    threads = 1,
)


getGroupBW(
    ArchRProj = archr.proj,
    groupBy = 'hash_assignment2',
    normMethod = 'ReadsInTSS',
    threads = 1
)

#add extra  cell information
sample_info <- read.csv("HTO_SAMID.csv") # to be dowloaded from Gene Expression Omnibus, GSE251977
sample_info$SAMID_HTO <- paste0(sample_info$SAMID, sample_info$HTO)

archr.proj$TREATMENT <- sample_info$TREATMENT[match(archr.proj$hash_assignment2, sample_info$SAMID_HTO)]
archr.proj$Cell <- unlist(lapply(strsplit(archr.proj$TREATMENT, split = "-"),"[",1))
archr.proj$TEST_ARTICLE <- unlist(lapply(strsplit(archr.proj$TREATMENT, split = "-"),"[",2))

archr.proj$Cell[archr.proj$Cell == "22RV1"] <- "22Rv1"

```

#### 5. Save results
```{r}
# convert project to MultiAssayExperiment object
MAE <- maw.archr::create.mae.with.multiple.sces.from.archr(outDir)

# save object
saveMAE("inst/extdata/prostateENZ.h5")
```

---
title: "LNCaP Cells Data Set"
author: "Aleksander Chlebowski"
date: 6 September 2022
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```


scATAC data was downloaded from Gene Expression Omnibus
(acc. no. [GSE168667](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE168667))
and analyzed with SingleCell ATAC - 10X pipeline v2.0.0. 
scRNAseq data was downloaded from  Gene Expression Omnibus
(acc.no. [GSE168668](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE168668))
and analyzed with SingleCell Gene Expression Analysis - 10X pipeline v6.0.1. 

Downstream analysis was performed with the \code{ArchR} package:

#### 1. Initiate `ArchR` project
```{r}
# attach ArchR package
library(ArchR)

# configure ArchR
addArchRThreads(16L)
addArchRGenome("hg38")

# create arrow file from fragment files
## list fragment files
fragments <- <FRAGMENT_FILES>
## assign sample names
names(fragments) <- <SAMPLE_IDs>
## create arrows
createArrowFiles(inputFiles = fragments, sampleNames = names(fragments))

# specify output directory
outDir <- <OUTPUT_DIRECTORY>

# locate arrow files
arrows <- <ARROW_FILES>

# create ArchR project
project <- ArchRProject(arrows, outDir)

# add sample annotation
sampleNames <- c("SRR13927735", "SRR13927736", "SRR13927737", "SRR13927738")
sampleCells <- c("LNCaP", "LNCaP", "LNCaP RES-A", "LNCaP RES-B")
sampleTreatment <- c("0.1% DMSO 48h", "enzalutamide 48h", "enzalutamide", "enzalutamide")
sampleEnzalutamide <- c("sensitive", "sensitive", "resistant", "resistant")
names(sampleCells) <- names(sampleTreatment) <- names(sampleEnzalutamide) <- sampleNames
project$Cells <- sampleCells[project$Sample]
project$Treatment <- sampleTreatment[project$Sample]
project$Enzalutamide <- sampleEnzalutamide[project$Sample]
project$sampleLabels <- sampleLabels[project$Sample]
```

#### 2. Prepare RNA-seq data
```{r}
# gene expression data is analyzed with `scran.chan` package
# the result is a SingleCellExperiment object
SCE <- <scran.chan ANALYSIS>
    
# adjust for integration
rownames(SCE) <- rowData(SCE)$Symbol
assay(SCE, "counts") <- as(assay(SCE, "counts"), "dgCMatrix")
# drop duplicates
SCE <- SCE[!duplicated(rowData(SCE)$Symbol), ]
```

#### 3. Commence `ArchR` analysis
```{r}
# reduce dimensionality by iterative LSI
project <- addIterativeLSI(project, useMatrix = "TileMatrix", name = "iLSI_ATAC")

# integrate ATAC and RNAseq
## prepare grouping for constrained integration
groupMapping <- SimpleList(
    sens_NT = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927735"],
        RNA = grep("SRR13927739", colnames(SCE), value = TRUE)
    ),
    sens_Enz = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927736"],
        RNA = grep("SRR13927740", colnames(SCE), value = TRUE)
    ),
    RES_A = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927737"],
        RNA = grep("SRR13927741", colnames(SCE), value = TRUE)
    ),
    RES_B = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927738"],
        RNA = grep("SRR13927742", colnames(SCE), value = TRUE)
    )
)
## execute
project <- addGeneIntegrationMatrix(project, useMatrix = "GeneScoreMatrix",
                                    matrixName = "GeneIntegrationMatrix",
                                    reducedDims = "iLSI_ATAC", seRNA = SCE,
                                    groupATAC = "Sample", groupRNA = "Sample", groupList = groupMapping,
                                    nameCell = "predictedCell",
                                    nameGroup = "predictedGroup",
                                    nameScore = "predictedScore",
                                    addToArrow = TRUE, force = TRUE)

# add LSI for RNAseq
project <- addIterativeLSI(project, useMatrix = "GeneIntegrationMatrix", name = "iLSI_RNAseq")

# combine dim-reduced ATAC and RNAseq
project <- addCombinedDims(project, name = "iLSI_Combined", reducedDims = c("iLSI_ATAC", "iLSI_RNAseq"))

# add UMAP embedding on combined reduced dimensionality
project <- addUMAP(project, reducedDims = "iLSI_Combined", name = "UMAP_Combined", verbose = FALSE)

# impute weights (for smoother visualizations)
project <- addImputeWeights(project, reducedDims = "iLSI_Combined")

# add group coverages
## inspect available cell numbers
table(project$Sample)
project <- addGroupCoverages(project, groupBy = "Sample", minCells = 30, maxCells = 250)

# add pseudo-bulk replicates
## requires MACS2 installation
project <- addReproduciblePeakSet(project, groupBy = "Sample", pathToMacs2 = "<PATH_TO_MACS2_INSTALLATION>")

# add peak matrix
project <- addPeakMatrix(project)
getAvailableMatrices(project)

# add motif annotation
project <- addMotifAnnotations(project, motifset = "cisbp", name = "Motif")

# add background peaks
project <- addBgdPeaks(project, method = "chromVAR")

# add deviation matrix
project <- addDeviationsMatrix(project, peakAnnotation = "Motif", force = TRUE)
getAvailableMatrices(project)

# save ArchR project
saveArchRProject(project)
```

#### 4. Save results
```{r}
# convert project to MultiAssayExperiment object
MAE <- maw.archr::create.mae.with.multiple.sces.from.archr(outDir)

# inspect object
MAE

# remove unpublished class
ind <- which(names(MAE) == "TileMatrix500")
MAElim <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = c(
    TileMatrix500 = as(experiments(MAE)[[ind]], "SingleCellExperiment"),
    as.list(experiments(MAE)[-ind])
))

# save object
saveMAE("inst/extdata/prostateENZ.h5")
```

---
title: "TF Motifs Info"
author: "Tomasz Włodarczyk"
date: 2024-01-31
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

#### 1. Data download
Data sets were downloaded from https://github.com/GreenleafLab/chromVARmotifs/raw/master/data/ and format was changed to rds.

#### 2. Data preparation
The motifs were curated from the cisBP database. Position frequency matrices were converted to PWMs by taking the log of the frequencies (after adding a pseudocount of 0.008) divided by 0.25.

#### 3. Session information

```{r}
sessionInfo()
```

---
title: "Reprogram-seq of LNCaP cells"
author: "Xiaosai"
date: 2022-10-06
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

scMultiome data was processed by ArchR.

---
title: "scMultiome tutorial"
author: Aleksander Chlebowski and Xiaosai Yao
output:
  BiocStyle::html_document:
    toc: true
    number_section: true
    self_contained: true
    titlecaps: true
vignette: >
  %\VignetteIndexEntry{scMultiome tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = TRUE
)
```


# Introduction

Single cell data is gaining sophistication - Cells can be measured in multiple modalities including gene expression, chromatin accessibility, cell surface markers and protein expression. These orthogonal measures of the same or matched cells enable a holistic construction of the cell state. However it has been challenging to share multiomic data, especially in an integrated format that consolidates the multiple layers of measurement. The `MultiAssayExperiment` container (implemented in the `r Biocpkg("MultiAssayExperiment")` package) provides a framework to package the various modalities into a single object on a per cell basis.

The `r Biocpkg("scMultiome")` package is a collection of public single cell multiome data sets pre-processed and packaged into `MultiAssayExperiment` objects for downstream analysis. It also provides basic functions to save the `MultiAssayExperiment` as `.hdf5` files so that users need to only load the desired modalities into memory. 

The 'scMultiome' package is similar to `r Biocpkg("SingleCellMultiModal")` in terms of providing multimodal data as an ExperimentHub package. One key difference is that we included additional functionalities to load only the desired modalities, and allow users to save their `MAE` objects as `.hdf5`s. The selective loading of experiments is desirable because multimodal data can be large: A typical 10x scMultiome consists of 100-200K atac-seq peaks on top of the typical 30-50K genes from rna-seq. Finally, `r Biocpkg("scMultiome")` provides a list of transcription factor and chromatin co-activator binding sites compiled from ENCODE and ChIP-Atlas ChIP-seq data as `GRangesList` objects. ChIP-seq data complements transcription factor motif information by potentially distinguishing related family members and providing occupancy information of chromatin factors that do not directly bind to DNA.

   
# Installation
``` {r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")

BiocManager::install("scMultiome")

```


# Accessing datasets

## Available Data Sets

View currently available data sets and the names of their accessor functions. Help pages for particular accessors contain more information on their data sets.

```{r, results = FALSE, message=FALSE}
library(scMultiome)
listDatasets()
```
```{r, results = 'asis', echo = FALSE}
lds <- listDatasets()
knitr::kable(lds, caption = "Available Data Sets")

```

Access a data set by calling its accessor function:
```{r}
prostateENZ(metadata = TRUE)

```


See the help files for accessor functions for possible extra options, e.g. `?prostateENZ`.


## Transcription Factor Binding Sites

In addition to single cell multiomic data, the package also provides binding sites of transcription factors, obtained from bulk ChIP-seq studies (merging of ChIP-Atlas and ENCODE).

If multiple ChIP-seq files are available for the same transcription factor, the peaks are merged to create a union set of peaks. Currently three genomic builds genomes are provided: hg38, hg19, and mm10. 

The ChIP-seq data is packaged into individual RDS files and they are accessed with a common accessor function, `tfBinding`, specifying the genome and source.

```{r}
tfBinding(genome = "hg38", source ="atlas", metadata = TRUE)
```

<br><br><br>





# Adding Datasets

If you want to contribute your publicly available multiome data set, please read this section and contact the package maintainer, `r desc::desc_get_maintainer()`.

Once the data has been preprocessed and converted into `MultiAssayExperiment`, it can be saved in a hdf5 file using `saveMAE`. You also need to provide metadata, documentation, and the accessor function that will retrieve the data from `ExperimentHub`.


## Developer Mode

Adding data involves updating the package and as such, it must be done in "developer mode". The developer mode allows access to additional tools such as documentation templates.

To work in developer mode, you must first clone the package repository with `git`. Create a branch from `master` and work on that.

Start an R session in the package directory (e.g. by opening the RStudio project in RStudio) and load all the functions. This is necessary for the R engine to temporarily identify your working directory as the package installation directory, and to expose the internal functions that you will be using.

```{r devtoolsAvailable, eval = FALSE, include = FALSE}
devtoolsAvailable <- requireNamespace("devtools", quietly = TRUE)

if (devtoolsAvailable) {
    # attach development version of the package
    devtools::load_all()
}

```


## Saving Your Data Set

`MultiAssayExperiment` is saved in the hdf5 format as it splits the MAEs into individual experiments so that you can choose to load selected experiments. Upon loading, selected experiments are reassembled and wrapped into an MAE object. Assays are represented by `DelayedMatrix` objects to save memory.and saved as a hdf5 file.

Currently only `MultiAssayExperiment` objects are supported. Experiments must be objects that inherit from `SummarizedExperiment` and will usually be `SingleCellExperiment`, hence full support is provided for the latter and their slots (`reducedDims` and `altExps`).

```{r saving, class.output = "scroll250", class.echo = "scroll250"}
# construct a dummy data set
mae <- dummyMAE()
mae

# name the file to save to
fileName <- tempfile(fileext = ".h5")

# save data set
saveMAE(mae, fileName)

```

You can use `testFile` to validate that your data set can be reconstructed.

```{r test reloading}
testFile(fileName)

```

For a detailed explanation of the process see `?saveMAE`.


## Creating Metadata

To add metadata, we first create a R script to store your data set's metadata. The metadata script provides key information such as data resource, species, genomic build and version number. This R script will be named `inst/scripts/make-metadata-<DATASET_NAME>.R`. 

```{r making metadata, eval = FALSE}
makeMakeMetadata(<DATASET_NAME>)
```


Metadata must be a 1-row data frame with specific columns and values must be character strings (some fields allow character vectors). See `inst/scripts/make-metadata.R` for more information and `inst/scripts/make-metadata-prostateENZ.R` for an example.

The file also stores metadata that will be returned by `listDatasets`. Likewise, this must be a 1-row data frame and values must be character strings.

Once your `make-metadata` file is ready, build the metadata 

```{r build metadata, eval = FALSE}
source(system.file("scripts", "make-metadata.R", package = "scMultiome"))
```

This must run without errors. If successful,
all the datasets will be captured in `inst/extdata/manifest.csv` and `inst/extdata/metadata.csv`

Subsequently, validate metadata 

```{r validation, eval = FALSE, class.output = "scroll250"}
ExperimentHubData::makeExperimentHubMetadata(dirname(system.file(package = "scMultiome")))
```

This call must also run without errors. It will return an `ExperimentHub` object that will display your metadata in the form that the end users will see it.



## Documenting Your Data Set

Every data set needs an additional documentation to describe how the data was obtained. This doesn't have to be a working script, just a report. Pseudocode is acceptable. Note that code evaluation has been disabled so that you can copy your actual code and the lengthy analysis does not run again.

Create an Rmarkdown file called `inst/scripts/make-data-<DATASET_NAME>.Rmd`. 

```{r making, eval = FALSE}
makeMakeData(<DATASET_NAME>)

```



## Creating accessor function

Every data set is accessed by its own accessor function, which provides access and help for your data set. 
For every data set, we create a `R` file that defines the function to retrieve the data set and provide important documentation. The package framework is constructed such that accessor functions are extremely simple and you can basically copy the original one (`prostateENZ`) and most of its documentation.

Create an R file called `<DATASET_NAME>.R`

```{r making2, eval = FALSE}
makeR("dataset")

```

This file will quote the accompanying Rmd file created above. This way the R file itself is more concise and easier to edit. Adjust the file accordingly:

1. Give the file the same title was used in the `make-metadata-<>.R` file.
2. Add a Description section.
3. Document any arguments other than `experiments` and `metadata`.
4. Describe the format of your `MultiAssayExperiment`.
5. Cite appropriate references.
6. Make sure the default value of the `experiments` argument reflects the experiment names in your data set.
7. If you want to restore custom classes to your experiments, add converting functions here. They do not require documentation or exporting.


## Building documentation

Once the files are ready, build the documentation, and `?scMultiome` and `?<DATASET_NAME>` to review it.
```{r documentation, eval = FALSE}
# build documentation
devtools::document()

# view package man page
?scMultiome

# view your data set man page
help("dataset")

```


## Updating Package Files

Edit the `DESCRIPTION` file to update package metadata: add yourself as package author and package version.

```{r DESCRIPTION, eval = FALSE}
desc::desc_add_author("<GIVEN_NAME>", "<FAMILY_NAME>", "<EMAIL>", role = "aut")
desc::desc_bump_version("minor")

```

Push your changes to git and create a merge request. Once the request is approved and merged to the `master` branch, the package maintainer will take over and move on to the next stage.


## Uploading Your Data Set

The final step of the process is to place the data set in Bioconductor's data bucket. This can only be done with Bioconductor's knowledge and blessing.

Bioconductor will be notified that a `scMultiome` update is coming by email at `hubs@bioconductor.org`. They will issue a temporary SAS token for the Bioconductor data bucket. The data set will be placed in the Bioconductor staging directory with `uploadFile` and Bioconductor will be notified again that the upload is ready. They will receive a link to the package repository, update metadata in `ExperimentHub` and finalize the process.

```{r upload, eval = FALSE}
uploadFile(file = fileName, sasToken = "<SAS_TOKEN>")

```

<br>

Consult [this vignette](https://www.bioconductor.org/packages/release/bioc/vignettes/HubPub/inst/doc/CreateAHubPackage.html#additional-resources-to-existing-hub-package) for current Bioconductor requirements.


## Congratulations

If you found any of this vignette or the process confusing, we would welcome feedback and gladly add more clarifications. Please email the package maintainer, `r desc::desc_get_maintainer()`.


# Session Info

```{r}
sessionInfo()
```
<br><br><br>

---
title: "TF Binding Info"
author: "Xiaosai Yao, Tomasz Włodarczyk"
date: 2024-03-21
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

#### 1. Data download
We download public ChIP-seq peak calls from ChIP-Atlas and ENCODE

#### 1.1. ChIP-Atlas

ChIP-seq binding sites were downloaded from [ChIP-Atlas](https://github.com/inutano/chip-atlas/wiki)

```{r}
# metatdata 
# download fileList.tab from https://dbarchive.biosciencedbc.jp/kyushu-u/metadata/fileList.tab

dir <- "chipAtlas/"
fileLIST <- read.delim(file.path(dir, "metadata/fileList.tab"), header = FALSE)


for (genome in c("hg38", "mm10")){
    
    TFLIST <- fileLIST[which(fileLIST[,3] == "TFs and others" &
                                 fileLIST[,2] == genome &
                                 fileLIST[,4] != "-" &
                                 fileLIST[,5] == "All cell types" &
                                 fileLIST[,7] == "05"),]
    
    download.files <- paste0("wget http://dbarchive.biosciencedbc.jp/kyushu-u/", 
                             genome,"/assembled/", TFLIST$V1, ".bed")
    write.table(x = download.files,
                file = file.path(dir, genome, ".sh"),
                quote = FALSE,
                col.names = FALSE,
                row.names = FALSE)
    
    write.table(TFLIST,
                file = file.path(dir, genome,".metadata.txt"),
                quote = FALSE,
                col.names = FALSE,
                row.names = FALSE,
                sep = "\t")
    
}

```
Download bed files by sample

```{r}
for (genome in c("mm10","hg38")){
   metadata <- read.delim("experimentList.tab", header=FALSE)
   metadata <- metadata[metadata$V2 == genome,]
   metadata <- metadata[metadata$V3 == "TFs and others",]


# write files for download
fileConn <- file(paste0("data/download.tissue.", genome,".txt"))
writeLines(paste0("wget -nc https://chip-atlas.dbcls.jp/data/", genome,"/eachData/bed05/", metadata$V1,".05.bed"), fileConn)
close(fileConn)
}
```
#### 1.2. ENCODE

Transcription factor ChIP-seq peaks were downloaded from [ENCODE data portal](https://www.encodeproject.org/)


#### 2. Merge peaks

Merge peaks of the same TFs into the same bed files.

#### 2.1. ChIP-Atlas

```{r ChIP-Atlas}

library(GenomicRanges)
library(rtracklayer)

##################### chipatlas_bedfiles_merge #####################
## Takes in a list of bed files and an accompanying legend that shows 
## which BED files correspond to  a specific TF
## Outputs a directory of merged bed files

dir <- 'chipAtlas/'
outdir <- 'chipatlas/data/chipatlas'



genomes <- c("mm10","hg38")
chr_order <- list()
chr_order[["mm10"]] <- c(paste0("chr",1:19),"chrX","chrY","chrM")
chr_order[["hg38"]] <- c(paste0("chr",1:22),"chrX","chrY","chrM")

for (genome in genomes){
  # Get directories of all bed files and make a list of the path of all bed files
  list_beds <- list.files(file.path(dir, paste0(genome, '_1e5')),
                          pattern = "*.bed")

  # Read the metatdata file

  metadata <- read.delim(file.path(dir, genome, ".metadata.txt"), header=FALSE)
  metadata$filename <- file.path(dir, genome, "_1e5", metadata$V1,".bed")
  colnames(metadata)[4] <- "TF"

  ## specify sorting order for chromosomes
  chr_order_genome <- chr_order[[genome]]

  ### get list of TFs represented in BED files
  TF.list <- unique(metadata$TF)

  for (i in seq_along(TF.list)) {

    print(TF.list[i])

    # get bed files associated with TF
    TF.files <- metadata[which(metadata$TF == TF.list[i]), "filename"]

    # merge bed files and sort by chromosome and starting coordinate

    merged_bed <- as.data.frame(rtracklayer::import.bed(file_name))
    merged_bed$seqnames <- factor(merged_bed$seqnames, levels = chr_order_genome)
    merged_bed <- na.omit(merged_bed)
    merged_bed <- merged_bed[order(merged_bed$seqnames, merged_bed$start),]

    ## convert to granges object and merge overlapping ranges
    gr <- makeGRangesFromDataFrame(merged_bed,
                                   seqnames.field = "seqnames",
                                   start.field = "start",
                                   end.field = "end")
    gr <- reduce(gr, drop.empty.ranges = TRUE)

    # write new bed file to directory
    export.bed(gr, con = file.path(outdir, genome, TF.list[i],".bed"))
  }

}
```

#### 2.2. ENCODE

Filter and merge ENCODE peaks

```{r ENCODE}
############### filter peaks ####################
## Takes in bed files and filters the peaks 
## in each file based on the enrichment score.
## Low Enrichment peaks are filtered out.

dir = 'encode/'
outdir = 'chipatlas/data/encode/'
for (genome in c("mm10", "hg38")){

    # Make a list of all the bed files
    list_beds <- list.files(file.path(dir, genome, "raw"), pattern = "*.bed.gz")

    #filter each bed file to have p value score > 4
    for (i in seq_along(list_beds)) {

        print(list_beds[i])

        curr_bed <- read.table(file.path(dir, genome, "raw", list_beds[i]))

        post_QC_bed <- curr_bed[curr_bed$V7 >=5,] #Q values

        if (nrow(post_QC_bed) >100){
            write.table(post_QC_bed,
                        file.path('/gstore/scratch/u/yaox19/encode/', 
                               genome, "filtered", list_beds[i]),
                        row.names = F, col.names = F, quote = F, sep="\t")
        }

    }
}




##################### ENCODE_bedfiles_merge #####################
## Takes in a list of bed files and an accompanying legend that shows 
## which BED files correspond to  a specific TF
## Outputs a directory of merged bed files



chr_order <- list()
chr_order[["mm10"]] <- c(paste0("chr",1:19),"chrX","chrY","chrM")
chr_order[["hg38"]] <- c(paste0("chr",1:22),"chrX","chrY","chrM")

replacement <- list(mm10 = "-mouse", hg38 = "-human")

for (genome in c("hg38","mm10")){

    # read metadata
    list_beds <- list.files(file.path(dir, genome, "raw"), pattern = "*.bed.gz")
    metadata <- read.delim(file.path(dir, genome, "raw", "metadata.tsv"))
    if(genome == "hg38") metadata <- metadata[metadata$"File.assembly"=="GRCh38",]
    metadata <- metadata[, c("File.accession", "Experiment.target")]
    metadata$"Experiment.target" <-  gsub(replacement[[genome]],"", metadata$"Experiment.target")

    # capitalize the first alphabet for mouse genes
    if (genome == "mm10"){
        metadata$"Experiment.target" <- stringr::str_to_title(metadata$"Experiment.target")
    }

    metadata$"File.accession" <- trimws( metadata$"File.accession")
    metadata$"File.accession" <- file.path(dir, genome, "filtered",
                                        metadata$"File.accession",".bed.gz")
    colnames(metadata) <- c("filename","TF")

    # remove files that do not exist
    metadata <- metadata[which(file.exists(metadata$filename)),]

    # specify sorting order for chromosomes
    chr_order_genome <- chr_order[[genome]]

    # get list of TFs represented in BED files
    TF.list <- unique(metadata$TF)

    for (i in seq_along(TF.list)) {

        print(TF.list[i])

        # get bed files associated with TF
        TF.files <- metadata[which(metadata$TF == TF.list[i]), "filename"]

        # merge bed files and sort by chromosome and starting coordinate
          extraCols_narrowPeak <- c(singnalValue = "numeric", pValue = "numeric",
                            qValue = "numeric", peak = "integer")
        merged_bed <- lapply(TF.files, function(file) as.data.frame(rtracklayer::import.bed(file, extraCols = extraCols_narrowPeak)))
        merged_bed <- do.call('rbind', merged_bed)
        merged_bed$seqnames <- factor(merged_bed$seqnames, levels = chr_order[[genome]])
        merged_bed <- merged_bed[!is.na(merged_bed$seqnames),]
        merged_bed <- merged_bed[order(merged_bed$seqnames, merged_bed$start),]

        # convert to granges object and merge overlapping ranges
        gr <- makeGRangesFromDataFrame(merged_bed,
                                       seqnames.field = "V1",
                                       start.field = "V2",
                                       end.field = "V3")
        gr <- reduce(gr, drop.empty.ranges = TRUE)

        # write new bed file to directory
        export.bed(gr, con=file.path(outdir, genome, TF.list[i],".bed"))
    }




}
```

#### 2.3. Merge both ENCODE and ChIP-Atlas

```{r ENCODE and ChIP-seq}

dir <- "chipatlas/data/"

chr_order <- list()
chr_order[["mm10"]] <- c(paste0("chr",1:19),"chrX","chrY","chrM")
chr_order[["hg38"]] <- c(paste0("chr",1:22),"chrX","chrY","chrM")


for (genome in c("mm10", "hg38")){

    # specify sorting order for chromosomes
    chr_order_genome <- chr_order[[genome]]

    ########### merge with chipatlas bed files
    chipatlas.files <- list.files(file.path(dir, "chipatlas", genome),pattern = "*.bed")
    encode.files <- list.files(file.path(dir, "encode", genome),pattern = "*.bed")
    shared.TFs <- intersect(chipatlas.files, encode.files)
    chipatlas.TFs <- setdiff(chipatlas.files, encode.files)
    encode.TFs <- setdiff(encode.files, chipatlas.files)
    combined_grl <- GRangesList()
    #### shared.TFs
    for (i in seq_along(shared.TFs)) {

        print(shared.TFs[i])

        chipatlas <- as.data.frame(import.bed(file.path(dir, "chipatlas", genome, shared.TFs[i])))
        encode <- as.data.frame(import.bed(file.path(dir, "encode", genome, shared.TFs[i])))

        merged_bed <- rbind(chipatlas[,1:3], encode[,1:3])
        merged_bed$seqnames <- factor(merged_bed$seqnames, levels = chr_order_genome)
        merged_bed <- merged_bed[!is.na(merged_bed$seqnames),]
        merged_bed <- merged_bed[order(merged_bed$seqnames, merged_bed$start),]

        gr <- makeGRangesFromDataFrame(merged_bed,seqnames.field = "seqnames",
                                   start.field = "start",end.field = "end")
        gr <- reduce(gr_merged, drop.empty.ranges = TRUE)
        combined_grl <- c(combined_grl, GRangesList(gr))
        export.bed(gr, con=file.path(dir, "chipatlas_encode_merged", genome, shared.TFs[i]))

    }

    ##### chipatlas
    for (i in seq_along(chipatlas.TFs)) {

        print(chipatlas.TFs[i])

        chipatlas <- as.data.frame(import.bed(file.path(dir, "chipatlas", genome, chipatlas.TFs[i])))

        merged_bed <- chipatlas[,1:3]
        merged_bed$seqnames <- factor(merged_bed$seqnames, levels = chr_order_genome)
        merged_bed <- merged_bed[!is.na(merged_bed$seqnames),]
        merged_bed <- merged_bed[order(merged_bed$seqnames, merged_bed$start),]
    
        gr <- makeGRangesFromDataFrame(merged_bed,seqnames.field = "seqnames",
                                       start.field = "start",end.field = "end")
        gr <- reduce(gr, drop.empty.ranges = TRUE)
        combined_grl <- c(combined_grl, GRangesList(gr))
        export.bed(gr, con=file.path(dir, "chipatlas_encode_merged", genome, chipatlas.TFs[i]))

    }

    ##### encode
    for (i in seq_along(length(encode.TFs))) {

        print(encode.TFs[i])

        chipatlas <- as.data.frame(import.bed(file.path(dir, "encode", genome, encode.TFs[i])))

        merged_bed <- chipatlas[,1:3]
        merged_bed$seqnames <- factor(merged_bed$seqnames, levels = chr_order_genome)
        merged_bed <- merged_bed[!is.na(merged_bed$seqnames),]
        merged_bed <- merged_bed[order(merged_bed$seqnames, merged_bed$start),]
    
        gr <- makeGRangesFromDataFrame(merged_bed,seqnames.field = "seqnames",
                                       start.field = "start",end.field = "end")
        gr <- reduce(gr, drop.empty.ranges = TRUE)
        combined_grl <- c(combined_grl, GRangesList(gr))
        export.bed(gr, con=file.path(dir, "chipatlas_encode_merged", genome, encode.TFs[i]))
        saveRDS(grl, paste0(output_dir, "tfBinding_", genome,"_atlas.rds"))
    }
}
```

#### 2.4 Merge ChIP-Atlas peaks by sample

```{r ChIP-Atlas by sample}

library(rtracklayer)
#import metadata

for (genome in c("hg38","mm10")){
    metadata <- read.delim(paste0(data_dir, "chipAtlas/experimentList.tab"), header=FALSE)
    metadata <- metadata[metadata$V2 == genome,]
    metadata <- metadata[metadata$V3 == "TFs and others",]

    # add QC metrics - new in version 2
    metadata_qc <- do.call(rbind, strsplit(metadata$V8, split = ","))
    metadata_qc <- data.frame(apply(metadata_qc, 2, as.numeric))
    colnames(metadata_qc) <- c("Number.of.reads", "Percentage.mapped", "Percentage.duplicates", "Number.of.peaks")

    metadata <- cbind(metadata, metadata_qc)
    metadata$unique.reads <- round(metadata[,"Number.of.reads"]*
                                       metadata[,"Percentage.mapped"]*
                                       (100-metadata[,"Percentage.duplicates"])/10000)

    metadata <- metadata[which(metadata$unique.reads >= 20000000 &
                                   #metadata_qc$Percentage.mapped >= 70 &
                                   metadata$Number.of.peaks >= 1000), ]

    # import bed
    extraCols_narrowPeak <- c(singnalValue = "numeric", pValue = "numeric",
                              qValue = "numeric", peak = "integer")

    grl <- list()

    celllines <- unique(metadata$V6)
    celllines <- as.vector(na.omit(celllines))
    for (cellline in celllines){
        metadata.cellline <- metadata[which(metadata$V6 == cellline),]
        unique.tf <- unique(metadata.cellline$V4)

        for (tf in unique.tf){

            message("cell line = ", cellline, " ", "tf = ", tf)
            gr <- list()
            beds <- metadata.cellline$V1[metadata.cellline$V4 ==tf]

            if (length(beds) >0) {
                counter <- 1
                for (bed in beds) {
                    file <- paste0(data_dir, "chipAtlas/", genome, "_1e5_individual/", bed,".05.bed")

                    if (file.exists(file)) {
                        # each gr list is per TF per tissue
                        gr[[counter]] <- try(rtracklayer::import(file, extraCols = extraCols_narrowPeak))
                        counter <- counter + 1
                    }
                }

                if (length(gr) > 0) {
                    grl[[cellline]][[tf]] <- reduce(do.call(c, gr))
                }
            }
        }

        if (length(grl[[cellline]])>0){
            grl[[cellline]] <- GRangesList(grl[[cellline]])

        }
    }
    saveRDS(grl, paste0(output_dir, "tfBinding_", genome,"_atlas.sample_v2.rds"))
}

```
#### 2.5 Merge ChIP-Atlas peaks by tissue
```{r ChIP-Atlas by tissue}
library(rtracklayer)
#import metadata

for (genome in c("hg38","mm10")){
    metadata <- read.delim(data_dir, "chipAtlas/experimentList.tab", header=FALSE)
    metadata <- metadata[metadata$V2 == genome,]
    metadata <- metadata[metadata$V3 == "TFs and others",]

    # add QC metrics - new in version 2
    metadata_qc <- do.call(rbind, strsplit(metadata$V8, split = ","))
    metadata_qc <- data.frame(apply(metadata_qc, 2, as.numeric))
    colnames(metadata_qc) <- c("Number.of.reads", "Percentage.mapped", "Percentage.duplicates", "Number.of.peaks")

    metadata <- cbind(metadata, metadata_qc)
    metadata$unique.reads <- round(metadata[,"Number.of.reads"]*
                                       metadata[,"Percentage.mapped"]*
                                       (100-metadata[,"Percentage.duplicates"])/10000)

    metadata <- metadata[which(metadata$unique.reads >= 20000000 &
                                   metadata$Number.of.peaks >= 1000), ]
    # import bed
    extraCols_narrowPeak <- c(singnalValue = "numeric", pValue = "numeric",
                              qValue = "numeric", peak = "integer")

    grl <- list()
=
    tissues <- unique(metadata$V5)
    tissues <- as.vector(na.omit(tissues))

    for (tissue in tissues){
        metadata.tissue <- metadata[which(metadata $V5 == tissue),]
        unique.tf <- unique(metadata.tissue$V4)
        #
        for (tf in unique.tf){

            message("tissue = ", tissue," ", "tf = ", tf)
            gr <- list()
            beds <- metadata.tissue$V1[metadata.tissue$V4 ==tf]

            if (length(beds) >0) {
                counter <- 1
                for (bed in beds) {
                    file <- paste0(data_dir, "chip_peaks/", genome, "_1e5_individual/", bed,".05.bed")

                    if (file.exists(file)) {
                    # each gr list is per TF per tissue
                        gr[[counter]] <- try(rtracklayer::import(file, extraCols = extraCols_narrowPeak))
                        counter <- counter + 1
                    }

                }

                if (length(gr) > 0) {
                    grl[[tissue]][[tf]] <- GenomicRanges::reduce(do.call(c, gr))
                }

            }
        }

        if (length(grl[[tissue]]) > 0) {
            grl[[tissue]] <- GRangesList(grl[[tissue]])
        }
    }

    saveRDS(grl, paste0(output_dir, "tfBinding_", genome,"_atlas.tissue_v2.rds"))
}

```
#### 2.6 Merge ENCODE peaks by sample
```{r ENCODE by sample}
library(rtracklayer)

metadata.path <- c(GRCh38=paste0(data_dir,"encode/human"),
                   mm10=paste0(data_dir, "encode/mm10/encode"))

for (genome in c("GRCh38", "mm10")){

    metadata <- read.delim(file.path(metadata.path[genome], "raw","metadata.tsv"), header=TRUE)
    metadata <- metadata[metadata$File.assembly == genome,]
    metadata <- metadata[metadata$Assay == "TF ChIP-seq",]
    metadata <- metadata[metadata$Audit.NOT_COMPLIANT == "",]
    metadata <- metadata[metadata$Audit.ERROR == "",]
    metadata$Peaks <- NA
    rownames(metadata) <- metadata$File.accession

    # import bed
    extraCols_narrowPeak <- c(singnalValue = "numeric", pValue = "numeric",
                              qValue = "numeric", peak = "integer")

    grl <- list()

    celllines <- unique(metadata$Biosample.term.name)
    celllines <- as.vector(na.omit(celllines))
    for (cellline in celllines){
        metadata.cellline <- metadata[which(metadata $Biosample.term.name == cellline),]
        unique.tf <- unique(metadata.cellline$Experiment.target)

        for (tf in unique.tf){
            message("tf = ", tf, " ", "cell line = ", cellline)
            tf.name <- sapply(strsplit(tf, "-"),"[",1)
            beds <- metadata.cellline$File.accession[metadata.cellline$Experiment.target ==tf]

            gr <- list()

            if (length(beds) >0) {

                for (bed in beds) {
                    file <- file.path(metadata.path[genome], "raw", paste0(bed,".bed.gz"))

                    if (file.exists(file)) {
                        temp_bed <- try(rtracklayer::import(file, extraCols = extraCols_narrowPeak))
                        peak_no <- length(temp_bed)
                        metadata[bed, "Peaks"] <- peak_no
                        if (peak_no > 1000) {              # new in version 2
                            gr[[bed]] <- temp_bed
                        }
                        rm(temp_bed)
                    }
                }
            }

            if (length(gr) > 0){
                grl[[cellline]][[tf.name]] <- GenomicRanges::reduce(do.call(c, unname(gr)))
            }
        }

        if (length(grl[[cellline]])>0){
            grl[[cellline]] <- GRangesList(grl[[cellline]])

        }
    }
    saveRDS(grl, paste0(output_dir,"tfBinding_", genome,"_encode.sample_v2.rds"))
    write.table(metadata, paste0(output_dir, "encode.metadata.", genome, ".txt"), quote = FALSE, row.names = FALSE, sep = "\t")
}


```

#### 3. Liftover


Perform liftover from hg38 to hg19 for the ChIP-seq binding sites

```{r liftover, eval = FALSE}
# chain downloaded from https://hgdownload.soe.ucsc.edu/goldenPath/hg38/liftOver/hg38ToHg19.over.chain.gz
# need to reformat chain file from space to tab
# sed -r 's/^([0-9]+) ([0-9]+) ([0-9]+)$/\1\t\2\t\3/' hg38ToHg19.over.chain > hg38_to_hg19_tabs.chain

ch <- rtracklayer::import.chain(con = "/chipatlas/data/hg38_to_hg19_tabs.chain")

library(liftOver)

# convert binding sites from ENCODE
grl_hg38 <- readRDS(paste0(output_dir,"tfBinding_", genome,"_encode.sample_v2.rds"))
grl_hg19 <- lapply(grl_hg38, liftOver, ch)
saveRDS(grl_hg19, file = paste0(output_dir,"tfBinding_hg19_encode.sample_v2.rds"))

# convert binding sites from ChIP Atlas

# convert grl by cell line
grl_hg38 <- readRDS(paste0(output_dir, "tfBinding_hg38_atlas.sample_v2.rds"))
grl_hg19 <- lapply(grl_hg38, liftOver, ch)
saveRDS(grl_hg19, file = paste0(output_dir, "tfBinding_hg19_atlas.sample_v2.rds")))

#convert grl by tissue
grl_hg38 <- readRDS(paste0(output_dir, "tfBinding_hg38_atlas.tissue_v2.rds"))
grl_hg19 <- lapply(grl_hg38, liftOver, ch)
saveRDS(grl_hg19, file = paste0(output_dir, "tfBinding_hg19_atlas.tissue_v2.rds"))

#convert combined grl
grl_hg38 <- readRDS(paste0(output_dir, "tfBinding_hg38_atlas.rds")
grl_hg19 <- lapply(grl_hg38, liftOver, ch)
saveRDS(grl_hg19, file = paste0(output_dir, "tfBinding_hg19_atlas.rds"))
```

---
title: "Data Set List"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

+ **AR_drug**: Response of prostate cancer cells to drug treatment
+ **colonHealthy**: Single-cell analysis of samples from healthy human colon
+ **hematopoiesis**: scATAC-seq and unpaired scRNA-seq of hematopoetic cells
+ **PBMC_10x**: PBMC Data Set
+ **prostateENZ**: LNCaP Cells Treated with Enzalutamide
+ **reprogramSeq**: Reprogram-seq of LNCaP cells
+ **TEADi_resistance**: Resistance of TEAD inhibitor to drug
+ **tfBinding_hg19_atlas**: TF Binding Info hg19 (ChIP-Atlas and ENCODE)
+ **tfBinding_hg19_atlas.sample**: TF Binding Info hg19 by sample (ChIP-Atlas)
+ **tfBinding_hg19_atlas.sample_v2**: TF Binding Info hg19 by sample (ChIP-Atlas)
+ **tfBinding_hg19_atlas.tissue**: TF Binding Info hg19 by tissue (ChIP-Atlas)
+ **tfBinding_hg19_atlas.tissue_v2**: TF Binding Info hg19 by tissue (ChIP-Atlas)
+ **tfBinding_hg19_encode.sample**: TF Binding Info hg19 by sample (ENCODE)
+ **tfBinding_hg19_encode.sample_v2**: TF Binding Info hg19 by sample (ENCODE)
+ **tfBinding_hg38_atlas**: TF Binding Info hg38 (ChIP-Atlas and ENCODE)
+ **tfBinding_hg38_atlas.sample**: TF Binding Info hg38 by sample (ChIP-Atlas)
+ **tfBinding_hg38_atlas.sample_v2**: TF Binding Info hg38 by sample (ChIP-Atlas)
+ **tfBinding_hg38_atlas.tissue**: TF Binding Info hg38 by tissue (ChIP-Atlas)
+ **tfBinding_hg38_atlas.tissue_v2**: TF Binding Info hg38 by tissue (ChIP-Atlas)
+ **tfBinding_hg38_encode.sample**: TF Binding Info hg38 by sample (ENCODE)
+ **tfBinding_hg38_encode.sample_v2**: TF Binding Info hg38 by sample (ENCODE)
+ **tfBinding_mm10_atlas**: TF Binding Info mm10 (ChIP-Atlas and ENCODE)
+ **tfBinding_mm10_atlas.sample**: TF Binding Info mm10 by sample (ChIP-Atlas)
+ **tfBinding_mm10_atlas.sample_v2**: TF Binding Info mm10 by sample (ChIP-Atlas)
+ **tfBinding_mm10_atlas.tissue**: TF Binding Info mm10 by tissue (ChIP-Atlas)
+ **tfBinding_mm10_atlas.tissue_v2**: TF Binding Info mm10 by tissue (ChIP-Atlas)
+ **tfBinding_mm10_encode.sample**: TF Binding Info mm10 by sample (ENCODE)
+ **tfBinding_mm10_encode.sample_v2**: TF Binding Info mm10 by sample (ENCODE)
+ **human_pwms_v2**: TF motifs human
+ **mouse_pwms_v2**: TF motifs mouse

---
  title: "Single-cell analysis of samples from healthy human colon"
author: "Tomasz Włodarczyk"
date: 2022-09-21
output:
  BiocStyle::html_document:
  titlecaps: false
toc_float: true
---
  
```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```
scATAC data was downloaded from Gene Expression Omnibus
(acc. no. [GSE165659](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE165659))
and analyzed with SingleCell ATAC - 10X pipeline v2.0.0 
scRNAseq data in form of Seurat objects was downloaded from  https://drive.google.com/drive/folders/12j9ufV1L0uWbUlab-VoXRznDLKDO7PQ. In case
of future change in the data storage location, it will be updated in the readme file in project's Github repository (https://github.com/winstonbecker/scCRC_continuum) 

Downstream analysis was performed with the \code{ArchR} package v. 1.0.2:


```{r}
library(ArchR)
library(parallel)

catlas_files <- <FRAGMENT_FILES>
outputDir <- <OUTPUT_DIRECTORY>
arrow_files <- createArrowFiles(inputFiles = catlas_files, sampleNames = <SAMPLE_NAMES>)
doubScores <- addDoubletScores(input = arrow_files)

# cerate ArchR project
project <- ArchRProject(arrow_files, outputDirectory = outputDir)

# filtering out doublet cells
project <- filterDoublets(project)

# add Iterative Latent Semantic Indexing reduced-dimensionality space
project <- addIterativeLSI(ArchRProj = project, useMatrix = "TileMatrix", 
                           name = "IterativeLSI", clusterParams = list(resolution = c(0.2), 
                                                    sampleCells = 10000, n.start = 10))

# batch correction
project  <- addHarmony( ArchRProj = project, reducedDims = "IterativeLSI",
    name = "Harmony", groupBy = "Sample")

project <- addClusters( input = project, reducedDims = "IterativeLSI",
    method = "Seurat", name = "Clusters", resolution = 0.8)

# add clusters after Harmony batch correction
project <- addClusters(input = project, reducedDims = "Harmony",
    method = "Seurat", name = "Clusters_Harmony", resolution = 0.8)

# add UMAP embedding 
project <- addUMAP(ArchRProj = project, reducedDims = "IterativeLSI",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI")

project <- addUMAP(ArchRProj = project, reducedDims = "IterativeLSI",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_Harmony")

# add column with log base 10 of the fragment numbers
project <- addCellColData(ArchRProj = project, data = log10(project$nFrags),
                                  name = "log10_nFrags", cells = project$cellNames)




# upload gene expression data
# use files downloaded from 
# https://drive.google.com/drive/folders/12j9ufV1L0uWbUlab-VoXRznDLKDO7PQ_?usp=sharing

data_files <- c("Final_scHTAN_colon_normal_epithelial_220213.rds",
                "Final_scHTAN_colon_immune_220213.rds",
                "Final_scHTAN_colon_stromal_220213.rds")

# define object names
RNAseq_se_names <- gsub(".*scHTAN_|_220213.rds", "", data_files)

# create objects as instances of SingleCellExperiment class
for (i in seq_along(RNAseq_se_names)) assign(RNAseq_se_names[i], Seurat::as.SingleCellExperiment(readRDS(data_files[i])))

# add column with cell types and disease state
for (obj in RNAseq_se_names){
    eval(parse(text = paste0("colData(", obj, ")$CellType <-", obj,"@colData@listData$CellType")))
    eval(parse(text = paste0("colData(", obj, ")$DiseaseState <-", obj,"@colData@listData$DiseaseState")))
}

# uniformize colData columns before merging
shared_cols <- purrr::map(list(colon_immune, colon_stromal, colon_normal_epithelial), colData) %>%
    purrr::map(colnames) %>%
    purrr::reduce(intersect)

# remove reducedDims since their column names differ across objects
for (obj in RNAseq_se_names){
    eval(parse(text = paste0(obj, "@colData <- ", obj, "@colData[,colnames(", obj, "@colData) %in% shared_cols]")))
    eval(parse(text = paste0("SingleCellExperiment::reducedDim(", obj, ") <- NULL")))
    eval(parse(text = paste0(obj, "@int_colData@listData <- list()")))
}

# merge RNAseq data objects

colon_RNAseq <- cbind(colon_immune, colon_normal_epithelial, colon_stromal)


RNA_se <- SummarizedExperiment(assay = list(counts = as(assay(colon_RNAseq, "counts"), "dgCMatrix")),
                               colData = colData(colon_RNAseq), rowData = rowData(colon_RNAseq))


# select samples from healthy donors (no cancer)
RNA_se <- RNA_se[,colData(RNA_se)$DiseaseState == "Normal"]

# RNA integration
project <- addGeneIntegrationMatrix(
    ArchRProj = project,
    useMatrix = "GeneScoreMatrix",
    reducedDims = "IterativeLSI",
    seRNA = RNA_se,
    addToArrow = TRUE,
    groupRNA = "CellType",
    nameCell = "predicted_cell_un",
    nameGroup = "predicted_group_un",
    nameScore = "predicted_score_un")


project <- addGroupCoverages(ArchRProj = project, groupBy = "predicted_group_un")

# add pseudo-bulk replicates
## requires MACS2 installation

project <- addReproduciblePeakSet( ArchRProj = project,
    groupBy = "predicted_group_un", pathToMacs2 = <PATH_TO_MACS2>)


# LSI reduced dimensionality based on the GeneIntegrationMatrix

project <- addIterativeLSI(ArchRProj = project, clusterParams = list(resolution = 0.2,
        sampleCells = 1000, n.start = 10), saveIterations = FALSE,
    useMatrix = "GeneIntegrationMatrix", varFeatures = 2500,
    firstSelection = "variable", binarize = FALSE, name = "LSI_RNA")

# add clusters based on the new reduced-dimensionality space
project <- addClusters( input = project, reducedDims = "LSI_RNA",
    method = "Seurat", name = "Clusters_RNA", resolution = 0.8)

# add UMAP embedding
project <- addUMAP(ArchRProj = project, reducedDims = "LSI_RNA",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI_RNA", metric = "cosine",
    force = TRUE)

# batch correction 
project  <- addHarmony( ArchRProj = project, reducedDims = "LSI_RNA",
    name = "Harmony_RNA", groupBy = "Sample")

# UMAP embedding after batch correction
project <- addUMAP(ArchRProj = project, reducedDims = "Harmony_RNA",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI_RNA_Harmony", metric = "cosine",
    force = TRUE
)

# find clutsters after batch correction
project <- addClusters( input = project, reducedDims = "Harmony_RNA",
    method = "Seurat", name = "Clusters_RNA_Harmony", resolution = 0.8)


# combine reduced-dimensionality spaces produced from ATACseq and RNAseq data
project <- addCombinedDims(project, reducedDims = c("IterativeLSI", "LSI_RNA"),
                            name =  "LSI_Combined")

# add UMAP embedding
project <- addUMAP(ArchRProj = project, name = "UMAP_combined", reducedDims = "LSI_Combined",
        nNeighbors = 30, minDist = 0.5, metric = "cosine")


# find clusters in combined reduced space
project <- addClusters(input = project, reducedDims = "LSI_Combined",
                        method = "Seurat", name = "Clusters_combined",
                        resolution = 0.4)

# add information about sequence motifs recognized by known transcriptions factors
project <- addMotifAnnotations(ArchRProj = project,
                                       motifSet = "cisbp", name = "Motif")


# add background peaks to be compared against during peak variation assessement
project <- addBgdPeaks(project)

# calculate per-cell devations of motif annotations
project <- addDeviationsMatrix(project, peakAnnotation = "Motif")

# save project
saveArchRProject(project, outputDir)

# convert project into MultiAssayExperiment object 
MAE <- maw.archr::create.mae.with.multiple.sces.from.archr(outputDir, tile.sizes = 500)

saveRDS(MAE, <OUTPUT_PATH>)

```

---
title: "scATAC-seq and unpaired scRNA-seq of hematopoetic cells"
author: "Xiaosai Yao"
date: 2022-10-11
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE, eval = FALSE)
```

Example scATAC-seq data of hematopoietic cells included in ArchR package was integrated with scRNA-seq. ScATAC-seq data was obtained from GSE139369 and scRNA-seq data was obtained from https://jeffgranja.s3.amazonaws.com/ArchR/TestData/scRNA-Hematopoiesis-Granja-2019.rds"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fileOperations.R
\name{fileOperations}
\alias{fileOperations}
\alias{saveMAE}
\alias{loadMAE}
\alias{saveExp}
\alias{saveExp,SummarizedExperiment-method}
\alias{saveExp,SingleCellExperiment-method}
\alias{loadExp}
\alias{testFile}
\alias{uploadFile}
\title{save and load data sets}
\usage{
saveMAE(mae, file, experiments = NULL, verbose = TRUE, overwrite = FALSE)

loadMAE(file, experiments, verbose)

saveExp(exp, expName, file, verbose)

\S4method{saveExp}{SummarizedExperiment}(exp, expName, file, verbose)

\S4method{saveExp}{SingleCellExperiment}(exp, expName, file, verbose)

loadExp(file, expName, verbose)

testFile(file)

uploadFile(
  file,
  sasToken,
  endpoint = "https://bioconductorhubs.blob.core.windows.net"
)
}
\arguments{
\item{mae}{object of class \code{MultiAssayExperiment}}

\item{file}{path to a hdf5 file}

\item{experiments}{character string specifying which experiments to save/load}

\item{verbose}{logical flag specifying operation verbosity}

\item{overwrite}{logical flag specifying whether to allow overwriting the hdf5 file}

\item{exp}{an experiment object that inherits from class \code{SummarizedExperiment},
usually a \code{SingleCellExperiment}}

\item{expName}{name of the experiment, i.e. name of the \code{ArchR} Matrix}

\item{sasToken}{access token to \code{endpoint}}

\item{endpoint}{Bioconductor's data bucket endpoint url}
}
\value{
\code{saveExp} returns \code{TRUE} invisibly if the save was successful.
\code{saveMAE} returns a named list of \code{TRUE} values.
\code{loadExp} returns a \code{SingleCellExperiment} or an object of a subclass.
\code{loadMAE} returns a \code{MultiAssayExperiment}.
\code{testFile} returns the \code{MultiAssayExperiment} stored in \code{file} in its entirety.
\code{uploadFile} returns TRUE invisibly.
}
\description{
Functions to disassemble and save, and load and reassemble MultiAssayExperiment data sets.
}
\details{
These are utilities for developers to add new data sets to the package.
Most will usually be called internally.

\code{saveMAE} is used to save a \code{MultiAssayExperiment} to a hdf5 file.
It creates the \code{file} and passes individual \code{experiments} to \code{saveExp}.

\code{saveExp} is called by \code{saveMAE} to disassemble experiment \code{exp}
and save its elements in \code{file}. A group hierarchy is created with
the top level group called \code{expName}, e.g. "GeneScoreMatrix",
and lower level groups to store specific elements:
\itemize{
\item{\code{experiment} class is saved in group "class"}
\item{assays are saved as sparse matrices in subgroup "assays", using \code{writeSparseMatrix}}
\item{\code{colData} and \code{colnames} are saved in subgroup "properties"}
\item{if \code{experiment} has \code{rownames}, the are saved in "properties"}
\item{\code{rowData} is saved in "properties", unless it is an empty \code{DataFrame}}
\item{if \code{experiment} has a \code{rowRanges} component,
it is converted to a data frame and saved in "properties"}
\item{if \code{experiment} has a \code{metadata} component,
it is deparsed to a string and saved in "properties"}
\item{if \code{experiment} has a \code{reducedDims} component,
they are saved in subgroup "reducedDims"}
\item{if \code{experiment} has a \code{altExps} component,
they are saved in subgroup "altExps" by recursively calling \code{saveExp}}
}
\code{DataFrames} (e.g. \code{colData}, \code{rowData}, embeddings)
are converted to \code{data.frames} and saved as compound type.

\code{loadMAE} is called by accessor functions to retrieve data. It locates the hdf5 file
in which the data set is stored and uses \code{loadExp} to extract the specified \code{experiments}.

\code{loadExp} first checks which property elements are stored for the experiment in question,
loads all elements of the experiment and builds a \code{SummarizedExperiment} object.
If the experiment was originally a \code{SingleCellExperiment} or a subclass thereof,
that class as well as possible additional slots are restored.

\code{testFile} can be used to test whether a data set loads correctly from a local file.
It calls \code{loadMAE} and extracts all experiment verbosely.

\code{uploadFile} will upload a single file to Bioconductor's staging directory.
}
\examples{

# create dummy MultiAssayExperiment
mae <- scMultiome:::dummyMAE()

fileName <- tempfile(fileext = ".h5")
saveMAE(mae, fileName)                                # save MAE
remae <- loadMAE(fileName, c("EXP1", "EXP2"), TRUE)   # load MAE (internal)
remae_exp1 <- loadMAE(fileName, "EXP1", TRUE)         # load MAE with one experiment (internal)

# create dummy SingleCellExperiment
sce <- scMultiome:::dummySCE()

saveExp(sce, "EXP3", fileName, TRUE)                  # save one experiment (internal)
resce <- loadExp(fileName, "EXP3", TRUE)              # load one experiment (internal)

testFile(fileName)                                    # load whole MAE
}
\seealso{
Vignette "rhdf5 - HDF5 interface for R" (\code{vignette} or \code{browseVignettes})
for details of hdf5 file construction.
\code{writeSparseMatrix} for details of saving sparse matrices.
}
\author{
Aleksander Chlebowski and Xiaosai
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertHDF5.R
\name{assertHDF5}
\alias{assertHDF5}
\title{check if a file is a valid HDF5 file}
\usage{
assertHDF5(path)
}
\arguments{
\item{path}{path to file to test}
}
\value{
Returns invisible \code{path} if check is successful, otherwise signals an error.
}
\description{
Check if a file path argument points to an non-corrupt HDF5 file.
}
\details{
Compares the first 8 bytes of a file to those of the standard HDF5 file header.
}
\section{Further development}{

The HDF5 file header contains 8 bytes, which hold specific meanings.
Currently the function only tests that the header of the file
specified by \code{path} is identical to a healthy HDF5 file and signals
a general error if that is not the case.
Reporting specific types of corruption can be implemented.
}

\examples{
fileName1 <- tempfile(fileext = ".h5")
rhdf5::h5createFile(fileName1)
rhdf5::h5write(mtcars, fileName1, "mtcars")
rhdf5::h5closeAll()
fileName2 <-  tempfile(fileext = ".csv")
write.csv(mtcars, fileName2)

assertHDF5(fileName1)   # passes
\dontrun{
assertHDF5(fileName2)   # fails
}

}
\references{
\url{http://web.ics.purdue.edu/~aai/HDF5/html/H5.format.html#BootBlock}
}
\author{
Aleksander Chlebowski
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reprogramSeq.R
\name{reprogramSeq}
\alias{reprogramSeq}
\title{reprogramSeq}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project.
Annotated with the hg38 genome build.
Contains the following experiments:
\itemize{
\item{\strong{TileMatrix500}: SingleCellAccessibilityExperiment with 6062095 rows and 3903 columns}
\item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36438 rows and 3903 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 3903 columns}
\item{\strong{NEPCMatrix}: SingleCellExperiment with 2 rows and 3903 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 126602 rows and 3903 columns}
\item{\strong{TF_bindingMatrix}: SingleCellExperiment with 1274 rows and 3903 columns}
}
}
\usage{
reprogramSeq(
  metadata = FALSE,
  experiments = c("TileMatrix500", "GeneExpressionMatrix", "GeneScoreMatrix",
    "NEPCMatrix", "PeakMatrix", "TF_bindingMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
scMultiome data of LNCaP infected with FOXA1, NKX2-1, GATA6
}
\section{Data preparation}{


scMultiome data was processed by ArchR.
}

\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\examples{
# check metada of dataset
reprogramSeq(metadata = TRUE)
# download data
\dontrun{
reprogramSeq()
}

}
\references{
Genentech dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dummies.R
\name{dummies}
\alias{dummies}
\alias{dummySCE}
\alias{dummyMAE}
\title{create dummy data sets}
\usage{
dummySCE(
  features = c("rowData", "rowRanges", "reducedDims", "altExps", "none")
)

dummyMAE(experiments = list(EXP1 = NULL, EXP2 = NULL))
}
\arguments{
\item{features}{character string specifying which (optional) features to create in the SCE}

\item{experiments}{named list of character vectors
specifying experiments to create and their features}
}
\value{
\code{dummySCE} returns a \code{SingleCellExperiment}.
\code{dummyMAE} returns a \code{MultiAssayExperiment}.
}
\description{
Create dummy SCE and MAE objects.
}
\examples{
scMultiome:::dummySCE()
scMultiome:::dummySCE("rowData")
scMultiome:::dummySCE("rowRanges")
scMultiome:::dummySCE("reducedDims")
scMultiome:::dummySCE("altExps")

scMultiome:::dummyMAE(list("dummyExperiment" = NULL))

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfMotifs.R
\name{tfMotifs}
\alias{tfMotifs}
\title{Transcription factor motifs}
\format{
\code{PWMatrixList} object containing information on
transcription factor motifs.
Contains the following experiments:
\itemize{
\item{\strong{human_pwms_v2}: PWMatrixList object of length 1558}
\item{\strong{mouse_pwms_v2}: PWMatrixList object of length 1558}
}
}
\usage{
tfMotifs(species = c("human", "mouse"), metadata = FALSE)
}
\arguments{
\item{species}{character string specifying the species of interest}

\item{metadata}{logical flag specifying whether to return data or metadata only}
}
\value{
A list of position weight matrices, one for each transcription factor.
}
\description{
Transcription factor motifs sets from the \code{chromVARmotifs} R package
}
\details{
This data set stores transcription factor motifs for human and
mouse genome, which can be used with the package epiregulon to compute scores of
transcription factor-regulatory element links.
}
\section{Data storage and access}{

The transcription factor motifs are stored separately for each species in .rds files encoding \code{PWMatrixList}.
Data for both species are be accessed with the same function \code{tfMotifs}.
}

\section{Data preparation}{

\subsection{1. Data download}{

Data sets were downloaded from https://github.com/GreenleafLab/chromVARmotifs/raw/master/data/ and format was changed to rds.
}

\subsection{2. Data preparation}{

The motifs were curated from the cisBP database. Position frequency matrices were converted to PWMs by taking the log of the frequencies (after adding a pseudocount of 0.008) divided by 0.25.
}

\subsection{3. Session information}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{sessionInfo()
}\if{html}{\out{</div>}}
}
}

\examples{
# check metada of dataset
tfMotifs("mouse", metadata = TRUE)
# download data
\dontrun{
tfMotifs("human")
}

}
\references{
Schep, A., Wu, B., Buenrostro, J. &  Greenleaf, W. J. (2017) chromVAR:
inferring transcription-factor-associated accessibility from single-cell
epigenomic data.
\emph{Nature Methods} 14, 975–978.
\href{http://dx.doi.org/0.1038/nmeth.4401}{doi:0.1038/nmeth.4401}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classDefinitions.R
\docType{class}
\name{customClasses}
\alias{customClasses}
\title{custom classes}
\description{
Additional class definitions.
}
\section{Classes}{

\itemize{
\item{\code{SingleCellAccessibilityExperiment}, contains \code{SingleCellExperiment}}
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeDataSetList.R
\name{makeDataSetList}
\alias{makeDataSetList}
\title{create data set list}
\usage{
makeDataSetList(metadata)
}
\arguments{
\item{metadata}{a \code{data.frame} containing data set metadata}
}
\value{
Invisible TRUE.
}
\description{
Automatically creates the data set list of the package.
}
\details{
This is an internal helper function for developers and will not be called directly.
It creates the file inst/scripts/datasetList.Rmd, which is incorporated
into the package help page to automatically list the current data sets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PBMC_10x.R
\name{PBMC_10x}
\alias{PBMC_10x}
\title{10k PBMC data}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project.
Annotated with the Hg38 genome build.
Contains the following experiments:
\itemize{
\item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 9702 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 9702 columns}
\item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36438 rows and 9702 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 159290 rows and 9702 columns}
\item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 9702 columns}
}
}
\usage{
PBMC_10x(
  metadata = FALSE,
  experiments = c("TileMatrix", "GeneScoreMatrix", "GeneExpressionMatrix", "PeakMatrix",
    "MotifMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
PBMC from a Healthy Donor downloaded from 10x Genomics. Granulocytes were removed by cell sorting.
Paired ATAC and Gene Expression libraries were generated from the isolated nuclei.
Targeted nuclei recovery was 10,000.
Data source: https://www.10xgenomics.com/datasets/pbmc-from-a-healthy-donor-granulocytes-removed-through-cell-sorting-10-k-1-standard-2-0-0
}
\section{Data preparation}{

\subsection{1. Download data}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# specify output directory
outDir <- <OUTPUT_DIRECTORY>

# dowbnload data from 10XGenomics server    
download.file(c("https://cf.10xgenomics.com/samples/cell-arc/1.0.0/pbmc_granulocyte_sorted_10k/pbmc_granulocyte_sorted_10k_filtered_feature_bc_matrix.h5", "https://cf.10xgenomics.com/samples/cell-arc/1.0.0/pbmc_granulocyte_sorted_10k/pbmc_granulocyte_sorted_10k_atac_fragments.tsv.gz", 
                "https://cf.10xgenomics.com/samples/cell-arc/1.0.0/pbmc_granulocyte_sorted_10k/pbmc_granulocyte_sorted_10k_atac_fragments.tsv.gz.tbi"), c(file.path(outDir,"pbmc_granulocyte_sorted_10k_filtered_feature_bc_matrix.h5"), file.path(outDir,"pbmc_granulocyte_sorted_10k_atac_fragments.tsv.gz"), file.path(outDir,"pbmc_granulocyte_sorted_10k_atac_fragments.tsv.gz.tbi")),
              method="libcurl")
                                                                                                                
}\if{html}{\out{</div>}}
}

\subsection{2. Initiate \code{ArchR} project}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(ArchR)

# configure ArchR
addArchRGenome("hg38")

# create arrow file from fragment files
## list fragment files
fragments <- <FRAGMENT_FILES>
## assign sample names
names(fragments) <- <SAMPLE_IDs>
## create arrows
createArrowFiles(inputFiles = fragments, sampleNames = names(fragments))

# locate arrow files
arrows <- <ARROW_FILES>

# create ArchR project
project <- ArchRProject(arrows, outDir)

}\if{html}{\out{</div>}}
}

\subsection{3. Add gene expression data}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{seRNA <- import10xFeatureMatrix(
  input = c(file.path(outDir, "pbmc_granulocyte_sorted_10k_filtered_feature_bc_matrix.h5")),
  names = c("PBMC_10k")
)

# filter out genes which are expressed in less than 3 cells
seRNA <- seRNA[colSums(assay(seRNA))>2,]

proj <- addGeneExpressionMatrix(input = proj, seRNA = seRNA, force = TRUE)
}\if{html}{\out{</div>}}
}

\subsection{4. Quality control}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#Filter Cells
proj <- proj[proj$TSSEnrichment > 6 & proj$nFrags > 2500 & !is.na(proj$Gex_nUMI)]

#Doublet Filtration
proj <- addDoubletScores(proj)
proj <- filterDoublets(proj)
}\if{html}{\out{</div>}}
}

\subsection{5. Clustering and dimensionality reduction}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#LSI-ATAC
proj <- addIterativeLSI(
  ArchRProj = proj,
  clusterParams = list(
    resolution = 0.2,
    sampleCells = 10000,
    n.start = 10
  ),
  saveIterations = FALSE,
  useMatrix = "TileMatrix",
  depthCol = "nFrags",
  name = "LSI_ATAC"
)

#LSI-RNA
proj <- addIterativeLSI(
  ArchRProj = proj,
  clusterParams = list(
    resolution = 0.2,
    sampleCells = 10000,
    n.start = 10
  ),
  saveIterations = FALSE,
  useMatrix = "GeneExpressionMatrix",
  depthCol = "Gex_nUMI",
  varFeatures = 2500,
  firstSelection = "variable",
  binarize = FALSE,
  name = "LSI_RNA"
)

#Combined Dims
proj <- addCombinedDims(proj, reducedDims = c("LSI_ATAC", "LSI_RNA"), name =  "LSI_Combined")

#UMAPs
proj <- addUMAP(proj, reducedDims = "LSI_ATAC", name = "UMAP_ATAC", minDist = 0.8, force = TRUE)

proj <- addUMAP(proj, reducedDims = "LSI_RNA", name = "UMAP_RNA", minDist = 0.8, force = TRUE)

proj <- addUMAP(proj, reducedDims = "LSI_Combined", name = "UMAP_Combined", minDist = 0.8, force = TRUE)

#Add Clusters
proj <- addClusters(proj, reducedDims = "LSI_Combined", name = "Clusters", resolution = 0.4, force = TRUE)
#proj <- addClusters(proj, reducedDims = "LSI_RNA", name = "Clusters_genes", resolution = 0.4, force = TRUE)
}\if{html}{\out{</div>}}
}

\subsection{6. Peak calling}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(BSgenome.Hsapiens.UCSC.hg38)

proj<- addGroupCoverages(ArchRProj = proj, groupBy = "Clusters")

proj <- addReproduciblePeakSet(
  ArchRProj = proj,
  groupBy = "Clusters",
  pathToMacs2 = pathToMacs2,
  force = TRUE,
  threads = 10
)

# add information about sequence motifs recognized by known transcriptions factors
proj <- addMotifAnnotations(ArchRProj = proj,
                               motifSet = "cisbp", name = "Motif")

proj <- addPeakMatrix(proj)

# add background peaks to be compared against during peak variation assessement
proj <- addBgdPeaks(proj)

# calculate per-cell devations of motif annotations
proj <- addDeviationsMatrix(proj, peakAnnotation = "Motif", force = TRUE)
}\if{html}{\out{</div>}}
}

\subsection{7. Cell type annotation}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{GeneExpressionMatrix <- getMatrixFromProject(proj, useMatrix = "GeneExpressionMatrix")

# Loading reference data with Ensembl annotations.
library(celldex)
#ref.data <- HumanPrimaryCellAtlasData(ensembl=FALSE)
bpe_data <- BlueprintEncodeData()


expr_assay <- assays(GeneExpressionMatrix)[[1]]
rownames(expr_assay) <- rowData(GeneExpressionMatrix)$name

# Performing predictions.
library(SingleR)
predictions <- SingleR(test=expr_assay, assay.type.test=1,
                       ref=bpe_data, labels=bpe_data$label.fine) # could be switched to label.main

proj$cell_type_SingleR <- predictions$labels



library(ArchR)
library(scater)
library(SingleCellExperiment)
library(epiregulon.archr)

GeneExpressionMatrix <- getMatrixFromProject(proj, "GeneExpressionMatrix")
GeneExpressionMatrix <- ArchRMatrix2SCE(GeneExpressionMatrix, rename="normalizedCounts")
reducedDim(GeneExpressionMatrix, "UMAP_Combined") <- getEmbedding(proj, embedding = "UMAP_Combined")
GeneExpressionMatrix$cell_type[is.na(GeneExpressionMatrix$cell_type)] <- "unknown"
plotReducedDim(GeneExpressionMatrix, dimred="UMAP_Combined", colour_by = "Clusters", text_by = "Clusters")
plotReducedDim(GeneExpressionMatrix, dimred="UMAP_Combined", colour_by = "cell_type_SingleR", text_by = "cell_type_SingleR")


# identify cell types in cluster 10
GeneExpressionMatrix$C10 <- GeneExpressionMatrix$cell_type_SingleR
GeneExpressionMatrix$C10[GeneExpressionMatrix$Clusters!="C10"] <- NA
plotReducedDim(GeneExpressionMatrix, dimred="UMAP_Combined", colour_by = "C10", text_by = "C10")

# identify cell types in cluster 11
GeneExpressionMatrix$C11 <- GeneExpressionMatrix$cell_type_SingleR
GeneExpressionMatrix$C11[GeneExpressionMatrix$Clusters!="C11"] <- NA
plotReducedDim(GeneExpressionMatrix, dimred="UMAP_Combined", colour_by = "C11", text_by = "C11")
table(GeneExpressionMatrix$C11)

# identify cell types in cluster 1
GeneExpressionMatrix$C1 <- GeneExpressionMatrix$cell_type_SingleR
GeneExpressionMatrix$C1[GeneExpressionMatrix$Clusters!="C1"] <- NA
table(GeneExpressionMatrix$C1)

clusters <- proj$Clusters
manual_annotation <- rep(NA, length(clusters))
manual_annotation[clusters \%in\% "C6"] <- "Naive CD4+ T"
manual_annotation[clusters \%in\% c("C13")] <- "CD14+ Mono"
manual_annotation[clusters \%in\% "C12"] <- "Monocytes"
manual_annotation[clusters \%in\% c("C2", "C3")] <- "B"
manual_annotation[clusters \%in\% "C4"] <- "Memory CD8+ T"
manual_annotation[clusters \%in\% "C14"] <- "FCGR3A+ Mono"
manual_annotation[clusters \%in\% "C5"] <- "NK"
manual_annotation[clusters \%in\% c("C9", "C10")] <- "Memory CD4+ T"
manual_annotation[clusters \%in\% c("C7", "C8")] <- "Naive CD8+ T"
manual_annotation[clusters \%in\% c("C1")] <- "DC"
proj$cell_type <- manual_annotation
proj <- proj[!is.na(proj$cell_type)]
}\if{html}{\out{</div>}}
}
}

\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\examples{
# check metada of dataset
PBMC_10x(metadata = TRUE)
# download data
 \dontrun{
PBMC_10x()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scMultiome-package.R
\docType{package}
\name{scMultiome-package}
\alias{scMultiome}
\alias{scMultiome-package}
\title{scMultiome: Collection of Public Single-Cell Multiome (scATAC + scRNAseq) Datasets}
\description{
Single cell multiome data, containing chromatin accessibility (scATAC-seq) and gene expression (scRNA-seq) information analyzed with the ArchR package and presented as MultiAssayExperiment objects.
}
\details{
Single cell multiome data sets, paired and unpaired, were analyzed
with the \code{ArchR} package in order to obtain epiregulons.
\code{ArchR} projects were converted to \code{MultiAssayExperiment} objects.

The creation of all datasets is described in detail in respective help files.
Run \code{listDatasets()} to view a list of available data sets or see \code{Datasets} below.
See \code{?<DATASET_NAME>} for details on particular data sets, e.g. \code{?prostateENZ}.
}
\section{Datasets}{

\itemize{
\item \strong{AR_drug}: Response of prostate cancer cells to drug treatment
\item \strong{colonHealthy}: Single-cell analysis of samples from healthy human colon
\item \strong{hematopoiesis}: scATAC-seq and unpaired scRNA-seq of hematopoetic cells
\item \strong{PBMC_10x}: PBMC Data Set
\item \strong{prostateENZ}: LNCaP Cells Treated with Enzalutamide
\item \strong{reprogramSeq}: Reprogram-seq of LNCaP cells
\item \strong{TEADi_resistance}: Resistance of TEAD inhibitor to drug
\item \strong{tfBinding_hg19_atlas}: TF Binding Info hg19 (ChIP-Atlas and ENCODE)
\item \strong{tfBinding_hg19_atlas.sample}: TF Binding Info hg19 by sample (ChIP-Atlas)
\item \strong{tfBinding_hg19_atlas.sample_v2}: TF Binding Info hg19 by sample (ChIP-Atlas)
\item \strong{tfBinding_hg19_atlas.tissue}: TF Binding Info hg19 by tissue (ChIP-Atlas)
\item \strong{tfBinding_hg19_atlas.tissue_v2}: TF Binding Info hg19 by tissue (ChIP-Atlas)
\item \strong{tfBinding_hg19_encode.sample}: TF Binding Info hg19 by sample (ENCODE)
\item \strong{tfBinding_hg19_encode.sample_v2}: TF Binding Info hg19 by sample (ENCODE)
\item \strong{tfBinding_hg38_atlas}: TF Binding Info hg38 (ChIP-Atlas and ENCODE)
\item \strong{tfBinding_hg38_atlas.sample}: TF Binding Info hg38 by sample (ChIP-Atlas)
\item \strong{tfBinding_hg38_atlas.sample_v2}: TF Binding Info hg38 by sample (ChIP-Atlas)
\item \strong{tfBinding_hg38_atlas.tissue}: TF Binding Info hg38 by tissue (ChIP-Atlas)
\item \strong{tfBinding_hg38_atlas.tissue_v2}: TF Binding Info hg38 by tissue (ChIP-Atlas)
\item \strong{tfBinding_hg38_encode.sample}: TF Binding Info hg38 by sample (ENCODE)
\item \strong{tfBinding_hg38_encode.sample_v2}: TF Binding Info hg38 by sample (ENCODE)
\item \strong{tfBinding_mm10_atlas}: TF Binding Info mm10 (ChIP-Atlas and ENCODE)
\item \strong{tfBinding_mm10_atlas.sample}: TF Binding Info mm10 by sample (ChIP-Atlas)
\item \strong{tfBinding_mm10_atlas.sample_v2}: TF Binding Info mm10 by sample (ChIP-Atlas)
\item \strong{tfBinding_mm10_atlas.tissue}: TF Binding Info mm10 by tissue (ChIP-Atlas)
\item \strong{tfBinding_mm10_atlas.tissue_v2}: TF Binding Info mm10 by tissue (ChIP-Atlas)
\item \strong{tfBinding_mm10_encode.sample}: TF Binding Info mm10 by sample (ENCODE)
\item \strong{tfBinding_mm10_encode.sample_v2}: TF Binding Info mm10 by sample (ENCODE)
\item \strong{human_pwms_v2}: TF motifs human
\item \strong{mouse_pwms_v2}: TF motifs mouse
}
}

\author{
\strong{Maintainer}: Xiaosai Yao \email{xiaosai.yao@gmail.com} (\href{https://orcid.org/0000-0001-9729-0726}{ORCID})

Authors:
\itemize{
  \item Aleksander Chlebowski \email{aleksander.chlebowski@contractors.roche.com}
  \item Aaron Lun \email{lun.aaron@gene.com}
  \item Tomasz Włodarczyk \email{tomasz.wlodarczyk@contractors.roche.com}
}

Other contributors:
\itemize{
  \item Shiqi Xie \email{xie.shiqi@gene.com} [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRtools.R
\name{RGtools}
\alias{RGtools}
\alias{storeGR}
\alias{restoreGR}
\title{manipulate GeneRanges}
\usage{
storeGR(x)

restoreGR(df)
}
\arguments{
\item{x}{object of class \code{GRanges}}

\item{df}{a \code{data.frame}}
}
\value{
\code{storeGR} returns a data frame, \code{restoreGR} returns a \code{GRanges} object.
}
\description{
Prepare and recover \code{GRanges} objects for and after storing.
}
\details{
\code{GRanges} objects, which can be encountered in \code{rowRanges} slots of
\code{SingleCellExperiment}s, are stored as data frames (of type compound).

\code{storeGR} converts \code{GRanges} to a data frames converts factors to characters.
\code{restoreGR} resets data types in the basic columns and re-instantiates \code{GRanges}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prostateENZ.R
\name{prostateENZ}
\alias{prostateENZ}
\title{LNCaP Cells Treated with Enzalutamide}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project.
Annotated with the Hg38 genome build.
Contains the following experiments:
\itemize{
\item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 15522 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 15522 columns}
\item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 23525 rows and 15522 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 80210 rows and 15522 columns}
\item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 15522 columns}
}
}
\usage{
prostateENZ(
  metadata = FALSE,
  experiments = c("TileMatrix", "GeneScoreMatrix", "GeneIntegrationMatrix", "PeakMatrix",
    "MotifMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
Single-cell ATAC sequencing of parental LNCaP cells (DMSO treated, the control),
LNCaP cells treated with 10µM enzalutamide for 48 hours,
and LNCaP-derived enzalutamide-resistant RES-A and RES-B cells.
}
\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\section{Data preparation}{


scATAC data was downloaded from Gene Expression Omnibus
(acc. no. \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE168667}{GSE168667})
and analyzed with SingleCell ATAC - 10X pipeline v2.0.0.
scRNAseq data was downloaded from  Gene Expression Omnibus
(acc.no. \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE168668}{GSE168668})
and analyzed with SingleCell Gene Expression Analysis - 10X pipeline v6.0.1.

Downstream analysis was performed with the \code{ArchR} package:
\subsection{1. Initiate \code{ArchR} project}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# attach ArchR package
library(ArchR)

# configure ArchR
addArchRThreads(16L)
addArchRGenome("hg38")

# create arrow file from fragment files
## list fragment files
fragments <- <FRAGMENT_FILES>
## assign sample names
names(fragments) <- <SAMPLE_IDs>
## create arrows
createArrowFiles(inputFiles = fragments, sampleNames = names(fragments))

# specify output directory
outDir <- <OUTPUT_DIRECTORY>

# locate arrow files
arrows <- <ARROW_FILES>

# create ArchR project
project <- ArchRProject(arrows, outDir)

# add sample annotation
sampleNames <- c("SRR13927735", "SRR13927736", "SRR13927737", "SRR13927738")
sampleCells <- c("LNCaP", "LNCaP", "LNCaP RES-A", "LNCaP RES-B")
sampleTreatment <- c("0.1\% DMSO 48h", "enzalutamide 48h", "enzalutamide", "enzalutamide")
sampleEnzalutamide <- c("sensitive", "sensitive", "resistant", "resistant")
names(sampleCells) <- names(sampleTreatment) <- names(sampleEnzalutamide) <- sampleNames
project$Cells <- sampleCells[project$Sample]
project$Treatment <- sampleTreatment[project$Sample]
project$Enzalutamide <- sampleEnzalutamide[project$Sample]
project$sampleLabels <- sampleLabels[project$Sample]
}\if{html}{\out{</div>}}
}

\subsection{2. Prepare RNA-seq data}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# gene expression data is analyzed with `scran.chan` package
# the result is a SingleCellExperiment object
SCE <- <scran.chan ANALYSIS>
    
# adjust for integration
rownames(SCE) <- rowData(SCE)$Symbol
assay(SCE, "counts") <- as(assay(SCE, "counts"), "dgCMatrix")
# drop duplicates
SCE <- SCE[!duplicated(rowData(SCE)$Symbol), ]
}\if{html}{\out{</div>}}
}

\subsection{3. Commence \code{ArchR} analysis}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# reduce dimensionality by iterative LSI
project <- addIterativeLSI(project, useMatrix = "TileMatrix", name = "iLSI_ATAC")

# integrate ATAC and RNAseq
## prepare grouping for constrained integration
groupMapping <- SimpleList(
    sens_NT = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927735"],
        RNA = grep("SRR13927739", colnames(SCE), value = TRUE)
    ),
    sens_Enz = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927736"],
        RNA = grep("SRR13927740", colnames(SCE), value = TRUE)
    ),
    RES_A = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927737"],
        RNA = grep("SRR13927741", colnames(SCE), value = TRUE)
    ),
    RES_B = SimpleList(
        ATAC = project$cellNames[project$Sample == "SRR13927738"],
        RNA = grep("SRR13927742", colnames(SCE), value = TRUE)
    )
)
## execute
project <- addGeneIntegrationMatrix(project, useMatrix = "GeneScoreMatrix",
                                    matrixName = "GeneIntegrationMatrix",
                                    reducedDims = "iLSI_ATAC", seRNA = SCE,
                                    groupATAC = "Sample", groupRNA = "Sample", groupList = groupMapping,
                                    nameCell = "predictedCell",
                                    nameGroup = "predictedGroup",
                                    nameScore = "predictedScore",
                                    addToArrow = TRUE, force = TRUE)

# add LSI for RNAseq
project <- addIterativeLSI(project, useMatrix = "GeneIntegrationMatrix", name = "iLSI_RNAseq")

# combine dim-reduced ATAC and RNAseq
project <- addCombinedDims(project, name = "iLSI_Combined", reducedDims = c("iLSI_ATAC", "iLSI_RNAseq"))

# add UMAP embedding on combined reduced dimensionality
project <- addUMAP(project, reducedDims = "iLSI_Combined", name = "UMAP_Combined", verbose = FALSE)

# impute weights (for smoother visualizations)
project <- addImputeWeights(project, reducedDims = "iLSI_Combined")

# add group coverages
## inspect available cell numbers
table(project$Sample)
project <- addGroupCoverages(project, groupBy = "Sample", minCells = 30, maxCells = 250)

# add pseudo-bulk replicates
## requires MACS2 installation
project <- addReproduciblePeakSet(project, groupBy = "Sample", pathToMacs2 = "<PATH_TO_MACS2_INSTALLATION>")

# add peak matrix
project <- addPeakMatrix(project)
getAvailableMatrices(project)

# add motif annotation
project <- addMotifAnnotations(project, motifset = "cisbp", name = "Motif")

# add background peaks
project <- addBgdPeaks(project, method = "chromVAR")

# add deviation matrix
project <- addDeviationsMatrix(project, peakAnnotation = "Motif", force = TRUE)
getAvailableMatrices(project)

# save ArchR project
saveArchRProject(project)
}\if{html}{\out{</div>}}
}

\subsection{4. Save results}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# convert project to MultiAssayExperiment object
MAE <- maw.archr::create.mae.with.multiple.sces.from.archr(outDir)

# inspect object
MAE

# remove unpublished class
ind <- which(names(MAE) == "TileMatrix500")
MAElim <- MultiAssayExperiment::MultiAssayExperiment(
    experiments = c(
    TileMatrix500 = as(experiments(MAE)[[ind]], "SingleCellExperiment"),
    as.list(experiments(MAE)[-ind])
))

# save object
saveMAE("inst/extdata/prostateENZ.h5")
}\if{html}{\out{</div>}}
}
}

\examples{
# check metada of dataset
prostateENZ(metadata = TRUE)
# download data
\dontrun{
prostateENZ()
}


}
\references{
Single-cell ATAC and RNA sequencing reveal pre-existing and persistent cells
associated with prostate cancer relapse.
Taavitsainen \emph{et al.}, \emph{Nature Communications} 2021 Sep 6;12(1):5307
\href{https://pubmed.ncbi.nlm.nih.gov/34489465/}{doi: 10.1038/s41467-021-25624-1}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hematopoiesis.R
\name{hematopoiesis}
\alias{hematopoiesis}
\title{scATAC-seq and unpaired scRNA-seq of hematopoetic cells}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project.
Annotated with the hg19 genome build.
Contains the following experiments:
\itemize{
\item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 17889 rows and 10250 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 22217 rows and 10250 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 150046 rows and 10250 columns}
\item{\strong{TileMatrix500}: SingleCellExperiment with 5762078 rows and 10250 columns}
}
}
\usage{
hematopoiesis(
  metadata = FALSE,
  experiments = c("TileMatrix500", "GeneScoreMatrix", "GeneIntegrationMatrix",
    "PeakMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
Example scATAC-seq data of hematopoietic cells included in ArchR package was integrated with scRNAseq. ScATAC-seq data was obtained from GSE139369 and scRNA-seq obtained from https://jeffgranja.s3.amazonaws.com/ArchR/TestData/scRNA-Hematopoiesis-Granja-2019.rds
}
\section{Data preparation}{


Example scATAC-seq data of hematopoietic cells included in ArchR package was integrated with scRNA-seq. ScATAC-seq data was obtained from GSE139369 and scRNA-seq data was obtained from https://jeffgranja.s3.amazonaws.com/ArchR/TestData/scRNA-Hematopoiesis-Granja-2019.rds"
}

\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\examples{
# check metada of dataset
hematopoiesis(metadata = TRUE)
# download data
\dontrun{
hematopoiesis()
}
}
\references{
Single-cell multiomic analysis identifies regulatory programs in mixed-phenotype acute leukemia
associated with prostate cancer relapse.
Granja \emph{et al.}, \emph{Nature Biotechnology} 2019 Dec;37(12):1458-1465.
\href{https://www.nature.com/articles/s41587-019-0332-7}{doi: 10.1038/s41587-019-0332-7}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/templates.R
\name{templates}
\alias{templates}
\alias{makeMakeData}
\alias{makeMakeMetadata}
\alias{makeR}
\title{documentation templates}
\usage{
makeMakeData(dataset)

makeMakeMetadata(dataset)

makeR(dataset)
}
\arguments{
\item{dataset}{name of data set as character string}
}
\value{
All functions return TRUE invisibly.
}
\description{
Create templates for data set documentation.
}
\details{
Functions to facilitate documenting a new data set.
Each function creates a file and attempts to open it in RStudio for editing.

\code{makeMakeData} creates an Rmarkdown report called \code{inst/scripts/make-data-<dataset>.Rmd}.
\code{makeMakeMetadata} creates an R script called \code{inst/scripts/make-metadata-<dataset>.R}.
\code{makeR} creates an R file called \code{R/<dataset>.Rmd}.
}
\author{
Aleksander Chlebowski
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listDatasets.R
\name{listDatasets}
\alias{listDatasets}
\title{list all available data sets}
\usage{
listDatasets()
}
\value{
A \code{DataFrame} listing all available data sets,
with one data set per row and the following columns:
\itemize{
\item{Call: function call used to access the data set directly}
\item{Author: original data set author}
\item{Title: data set name}
\item{Species: species name}
\item{Lineage: sample lineage}
\item{CellNumber: number of cells in the data set}
\item{Multiome: paired or unpaired}
\item{DiskSize: size of the dataset in storage (also size of the download)}
\item{Version: data set version number or upload date}
}
}
\description{
Summary information for all data sets available in the package.
}
\examples{
listDatasets()

}
\author{
Aleksander Chlebowski
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TEADi_resistance.R
\name{TEADi_resistance}
\alias{TEADi_resistance}
\title{TEAD inhibitor resistance}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project.
Annotated with the Hg38 genome build.
Contains the following experiments:
\itemize{
\item{\strong{TileMatrix}: SingleCellExperiment with 6068436 rows and 4952 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 57765 rows and 4952 columns}
\item{\strong{GeneExpressionMatrix}: SingleCellExperiment with 36451 rows and 4952 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 103723 rows and 4952 column}
\item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 4952 columns}
\item{\strong{TFPeaksDeviationsMatrix}: SingleCellExperiment with 1269 rows and 4952 columns}
\item{\strong{TF_bindingMatrix}: SingleCellExperiment with 1504 rows and 4952 columns}
}
}
\usage{
TEADi_resistance(
  metadata = FALSE,
  experiments = c("TileMatrix", "GeneScoreMatrix", "GeneExpressionMatrix", "PeakMatrix",
    "MotifMatrix", "TFPeaksDeviationsMatrix", "TF_bindingMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
Single cell multiomics on mesothelioma cell line H226 sensitive and resistant to the TEAD inhibitor GNE-7883
}
\section{Data preparation}{


The following workflow was created based on the original code written by Julien Tremblay (julien.tremblay@contractors.roche.com).
\subsection{1. Initiate \code{ArchR} project}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# attach ArchR package
library(ArchR)

# configure ArchR
addArchRGenome("hg38")

# create arrow file from fragment files
## list fragment files
fragments <- <FRAGMENT_FILES>
## assign sample names
names(fragments) <- <SAMPLE_IDs>
## create arrows
createArrowFiles(inputFiles = fragments, sampleNames = names(fragments),
                     minTSS = 4, minFrags = 1000)

# specify output directory
outDir <- <OUTPUT_DIRECTORY>

# locate arrow files
arrows <- <ARROW_FILES>
    
doublet.score <- addDoubletScores(
    input = arrows,
    k = 10, #Refers to how many cells near a 'pseudo-doublet' to count.
    knnMethod = 'UMAP', #Refers to the embedding to use for nearest neighbor search.
    LSIMethod = 1
)

# create ArchR project
project <- ArchRProject(arrows, outDir)
}\if{html}{\out{</div>}}
}

\subsection{2. Add further standard analysis to the ArchR Project}{

This part of code is wrapped by one of the internal libraries.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{
# .h5 files to be downloaded from GEO (GSE247442)
se.rna <- import.archr.10x.se(h5.files = rna.files, sample.names = names(rna.files))

# filter RNA so that only cells which also have ATAC-seq are included
se.rna <- se.rna[,which(colnames(se.rna) \%in\% rownames(ArchR::getCellColData(archr.proj)))]
  
# Filter out scaffolds that don't have at least 3 genes
filtered.chr.names <- names(which(table(seqnames(se.rna)) >= 3));
se.rna <- se.rna[as.character(seqnames(se.rna)) \%in\% filtered.chr.names,];
rowRanges(se.rna) <- keepSeqlevels(rowRanges(se.rna), filtered.chr.names, pruning.mode = 'coarse');

archr.proj <- addGeneExpressionMatrix(
    input = project,
    seRNA = se.rna,
    threads = num.threads
);

# LSI dimensionality reduction
archr.proj <- addIterativeLSI(
    ArchRProj = project,
    useMatrix = 'TileMatrix',
    name = paste0('IterativeLSI_ATAC'),
    seed = 2,
    threads = num.threads
)

# Add clusters to colData
archr.proj <- addClusters(
    input = project,
    reducedDims = paste0('IterativeLSI_ATAC'),
    name = paste0('Clusters',atac.name.suffix),
    seed = 2
)
  
 
}\if{html}{\out{</div>}}
}

\subsection{3. Demultiplex HTO counts}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{
# HTO counts were generated using internal cumulus cellranger workflow and the results are
# available on Gene Expression Omnibus, GSE247442
# This was equivalent to run in the command line:
#generate_count_matrix_ADTs \\
#    /gstore/apps/CellRanger/7.1.0/lib/python/cellranger/barcodes/737K-arc-v1.txt.gz \\
#    ../data/hashing_index.csv \\
#    ../data/hto/raw_reads_LIB5457060_SAM24417357 \\
#    LIB5457060_SAM24417357 \\
#    -p 4 --max-mismatch-feature 2 --feature antibody --max-mismatch-cell 1 --umi-length 12
#
#generate_count_matrix_ADTs \\
#    /gstore/apps/CellRanger/7.1.0/lib/python/cellranger/barcodes/737K-arc-v1.txt.gz \\
#    ../data/hashing_index.csv \\
#    ../data/hto/raw_reads_LIB5457058_SAM24417355 \\
#    LIB5457058_SAM24417355 \\
#    -p 4 --max-mismatch-feature 2 --feature antibody --max-mismatch-cell 1 --umi-length 12

library(zellkonverter)
library(ArchR)
library(DropletUtils)
library(SingleCellExperiment)
library(Matrix)
library(BiocParallel)
library(data.table)
library(dplyr)

hto_files = list.files(hto_path, pattern="*.hashing.csv", recursive=TRUE, full.names=TRUE)
hto_info2 = data.frame(hto_files)
hto_info2$sample = gsub(".*_(SAM\\\\d+).*", "\\\\1", hto_info2$hto_files)


arcseq_files = list.files(paste0(arcseq_info$uri, "/"), pattern="*raw_feature_bc_matrix", recursive=TRUE, full.names=TRUE)
if(!is.null(skip_sample))\{
    arcseq_files = arcseq_files[!grepl("SAM24417356", arcseq_files)]
\}
arcseq_info2 = data.frame(arcseq_files)
arcseq_info2$sample = gsub(".*(SAM\\\\d+).*", "\\\\1", arcseq_info2$arcseq_files)

samples = gsub(".*_(SAM\\\\d+).*", "\\\\1", hto_files)
summary_df_final = NULL
seRNA_objects = list()
#figures = list()
tables = list()
empty_drops = list()
hashed_drops = list()
for(i in 1:length(samples))\{
        summary_df = NULL
      
        curr_sample = samples[i]
        message("Processing ", curr_sample)
        dir.create(paste0(outdir, "/", curr_sample, "/rds"), recursive=TRUE)
      
        curr_arcseq_file = arcseq_info2[arcseq_info2$sample == curr_sample,]$arcseq_files
        curr_hto_file    = hto_info2[hto_info2$sample == curr_sample,]$hto_files
        ###########################
        # import gex              #
        ###########################
        message("...Importing arcseq matrix")
        seRNA <- ArchR::import10xFeatureMatrix(
            input = file.path(curr_arcseq_file),
            names = curr_sample
        )
        names(assays(seRNA)) = "counts"
        # coerce seRNA obs to a SingleCellExperiment because it is a RangedSingleCellExperiment at this point.
        seRNA = as(seRNA, "SingleCellExperiment")
        # Here if you look at the seRNA obj, there are approx 725k single-cells: at this point, they are NOT single cells, but droplets and the
        # vast majority of them are empty (processed below)
      
        ###########################################
        # import HTO and convert to sce object.   #
        ###########################################
        hto = data.frame(fread(curr_hto_file, header=T), check.names=FALSE)
        rownames_hto = hto$Antibody
        hto = hto[,-1]
        # Here remember that each hto also contains a cell barcode sequence. in the next line,
        # we are formatting each cell barcode the way as it is formatted in the seRNA object
        # will be used later to only keep cell barcodes for which we have an assigned hto.
        colnames(hto) = paste0(curr_sample, "#", colnames(hto), "-1")
        hto = as(as.matrix(hto), "dgCMatrix")
        rownames(hto) = rownames_hto
        hto = SingleCellExperiment(assays = list(counts=hto))
      
        # merge GEX and HTO into a SCE
        # Only keep seRNA cells that are found in hto data.
        common_cells = intersect(colnames(hto), colnames(seRNA))

        # Then, narrow selection to common cells.
        hto = hto[, common_cells] # row=features, col=cells
        # Same selection with seRNA obj (remember both SingleCellExperiment and SummarizedExperiment classes have the same structure)
        seRNA = seRNA[, common_cells]
        # coerce seRNA obs to a SingleCellExperiment because it is a RangedSingleCellExperiment at this point.
        seRNA = as(seRNA, "SingleCellExperiment")
        # Add HTO data as altExp attribute, see doc for more details.
        altExp(seRNA, "HTO") = hto
      
        ###################################################
        # Distinguish single cells from empty droplets.   #
        ###################################################
        # Remove all zeros
        # We can't remove rows at this point, only col (i.e. single cells) at this point. because downstream, we'll have to merge various objects. We'll have to do that later downstream.
        seRNA = seRNA[, Matrix::colSums(assays(seRNA)$counts) > 0]
        # Have a look at barcode ranks:
        empty_thresh = 100 # remove more invalid cells.
        bc_ranks = barcodeRanks(counts(seRNA), lower=empty_thresh)
      
        # Add colData to seRNA obj (i.e. bc_ranks hold barcodes in the same order as it was in the seRNA obj)
        colData(seRNA)$BarcodeRank   = bc_ranks$rank
        colData(seRNA)$BarcodeTotal  = bc_ranks$total
        colData(seRNA)$BarcodeFitted = bc_ranks$fitted
      
      
        # Barcodes that contain more than retain total counts are always retained. This ensures that large cells with profiles that 
        # are very similar to the ambient pool are not inadvertently discarded. If retain is not specified, it is set to the total 
        # count at the knee point detected by barcodeRanks. Manual specification of retain may be useful if the knee point was not correctly 
        # identified in complex log-rank curves. Users can also set retain=Inf to disable automatic retention of barcodes with large totals.
      
        curr_rank = 30000
        emp_drops = emptyDrops(counts(seRNA), lower=NULL, niters=100000,
                             test.ambient = TRUE, BPPARAM=SerialParam(), by.rank=curr_rank)
        emp_fdr = 0.01
        is_cell = emp_drops$FDR <= emp_fdr
        is_cell[is.na(is_cell)] = FALSE # NA means its not a cell, so force NAs to FALSE
        empty_drops[[curr_sample]] = emp_drops
      
        seRNA_filt = runUMAP(seRNA_filt, altexp = "HTO", name="UMAP_HTO", exprs_values = "clr")

        seRNA_filt = seRNA[, which(is_cell)]
        
        # Estimate HTO ambient proportions using empty droplets
        hto_mat = assay(altExp(seRNA),"counts")[, which(is_cell)]
        # confirm that by ambient, we mean non-cells... or just really low abundant stuff.
        # Ok, from the documentation, I believe ambient = non-cells HTOs.
        # Get \% of each HTO
        ambient_hto_mat = assay(altExp(seRNA), "counts")[,!is_cell]
        ambient_hto_prop = proportions(rowSums(assay(altExp(seRNA), "counts")[,!is_cell]))
        hash_stats = hashedDrops(hto_mat, ambient=ambient_hto_prop, 
                               doublet.nmads=3,
                               doublet.min=2,
                               confident.nmads=1,
                               confident.min=1
        )

    
    colData(seRNA_filt) = cbind(colData(seRNA_filt), hash_stats)
    colData(seRNA_filt)$library = sapply(strsplit(colnames(seRNA_filt), split = "#"), "[",1)
  
    assay(altExp(seRNA_filt), "logcounts") = log10(assay(altExp(seRNA_filt), "counts")+1)
    # Here see the doc on sweep. But basically for each row the logcount is subtracted from mean log count, which basically is a clr.
    assay(altExp(seRNA_filt), "clr") = sweep(
                                        assay(altExp(seRNA_filt), "logcounts"), 
                                        2,
                                        colMeans(assay(altExp(seRNA_filt), "logcounts")),
                                        "-"
                                      )
    seRNA_filt = runUMAP(seRNA_filt, altexp = "HTO", name="UMAP_HTO", exprs_values = "clr")
    seRNA_filt$hash_assignment = rownames_hto[seRNA_filt$Best]
    # save output
    saveRDS(seRNA_filt, paste0(outdir, "/", curr_sample, "/rds/", curr_sample, "_demultiplex_hto_and_scRNA.rds"))
\}

}\if{html}{\out{</div>}}
}

\subsection{4. Merge HTO data into the ArchR project}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(ArchR)
library(SingleCellExperiment)
library(data.table)

seRNA_objs = list()
for(sample in samples)\{
    curr_seRNA_obj = readRDS(paste0("./output/", sample, "/rds/", sample, "_demultiplex_hto_and_scRNA.rds"))
    seRNA_objs[[sample]] = curr_seRNA_obj
\}

# Here load the multiple seRNA objects from last step and combine them.
seRNA_final = do.call(cbind, seRNA_objs)

seRNA_final$hash_assignment2 = paste0(seRNA_final$library, "_", seRNA_final$hash_assignment)

common = intersect(project$cellNames, colnames(seRNA_final))
project = project[common,]

# Here at this point, we have the archr object and seRNA obj with the same cells.
# add HTO information
for(curr_cell_attribute in colnames(colData(seRNA_final)))\{
    proj = addCellColData(
        ArchRProj = project,
        data = colData(seRNA_final)[common, curr_cell_attribute],
        cells = common,
        name = curr_cell_attribute,
        force = TRUE
    )
\}

#filter out doublets and non-confident calls
project = project[which(project$Confident == TRUE & proj$Doublet == FALSE), ]

# mapping_file.tsv can be dowloaded from GEO (GSE247442)

mapping = data.frame(fread("mapping_file.tsv", header=T, sep="\\t"), check.names=F)
row.names(mapping) = mapping$SampleID
mapping$hash_assignment2 = mapping$SampleID

tmp_df = data.frame(getCellColData(project, select=c("hash_assignment2")))
tmp_df2 = join(tmp_df, mapping, by="hash_assignment2")
project$CNAME = tmp_df2$CNAME
project$TEST_ARTICLE = tmp_df2$TEST_ARTICLE
project$HTO = tmp_df2$HTO
project$Treatment = paste0(project$CNAME, "_", project$TEST_ARTICLE)
project$Treatment_HTO = paste0(project$CNAME, "_", project$TEST_ARTICLE, " (", project$HTO, ")")
project$Treatment_library = paste0(project$Treatment, "_", project$library)

}\if{html}{\out{</div>}}
}

\subsection{5. Cluster annotation}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# mapping_file.tsv can be dowloaded from GEO (GSE247442)
mapping = data.frame(fread("mapping_file.tsv", header=T, sep="\\t"), check.names=F)
samples_vColors = vColors[1:length(unique(mapping[["Treatment_HTO"]]))]
names(samples_vColors) = unique(mapping[["Treatment_HTO"]])

embed = getEmbedding(project, embedding=paste0("UMAP_TileMatrix"))
embed = cbind(embed, getCellColData(project, select=c("Treatment", "HTO", "Treatment_HTO", "Treatment_library", "library", "Clusters_TileMatrix")))
colnames(embed)[1] = "UMAP_dim_1"
colnames(embed)[2] = "UMAP_dim_2"

my_labels_df = data.frame(Treatment_HTO=unique(embed$Treatment_HTO),label=paste0("", unique(embed$Treatment_HTO)))
my_labels = embed \%>\%
    dplyr::group_by(Treatment_HTO) \%>\%
    dplyr::summarize(UMAP_dim_1 = median(UMAP_dim_1), UMAP_dim_2 = median(UMAP_dim_2)) \%>\%
    dplyr::left_join(my_labels_df) \%>\% as.data.frame()

p_init_samples = NULL; p_init_samples = ggplot(embed, aes(x=UMAP_dim_1, y=UMAP_dim_2, color=Treatment_HTO)) +
    geom_point(size=0.3) +
    scale_color_manual(values=samples_vColors) +
    xlab(paste0("UMAP 1")) +
    ylab(paste0("UMAP 2")) +
    #ggtitle(paste0("UMAP; ", curr_signature)) +
    guides(color=guide_legend(ncol=2, override.aes=list(size=1.5))) +
    ggrepel::geom_label_repel(data=my_labels, box.padding=0.0, alpha=0.8, label.size=0.05, fill="gray70", color="black", size=3, aes(label=my_labels[["label"]])) +
    theme_minimal() +
    theme(panel.border=element_rect(fill=NA, linetype="solid", colour = "black", linewidth=0.5), axis.title=element_text(size=8), 
          axis.text=element_text(size=8, angle=0), strip.text.y=element_text(size=8, angle=0), legend.position="bottom",
          legend.text=element_text(size=8, angle=0), plot.title=element_text(size=10), legend.key.size=unit(0.35, 'cm'), legend.title=element_blank())

my_labels_df2 = data.frame(Clusters_TileMatrix=unique(embed$Clusters_TileMatrix),label=paste0("", unique(embed$Clusters_TileMatrix)))
my_labels2 = embed \%>\%
    dplyr::group_by(Clusters_TileMatrix) \%>\%
    dplyr::summarize(UMAP_dim_1 = median(UMAP_dim_1), UMAP_dim_2 = median(UMAP_dim_2)) \%>\%
    dplyr::left_join(my_labels_df2) \%>\% as.data.frame()

clusters_vColors = vColors20[1:length(unique(mapping[["Clusters_TileMatrix"]]))]
names(clusters_vColors) = unique(mapping[["Clusters_TileMatrix"]])

p_init_clusters = NULL; p_init_clusters = ggplot(embed, aes(x=UMAP_dim_1, y=UMAP_dim_2, color=.data[["Clusters_TileMatrix"]])) +
    geom_point(size=0.3) +
    scale_color_manual(values=clusters_vColors) +
    xlab(paste0("UMAP 1")) +
    ylab(paste0("UMAP 2")) +
    #ggtitle(paste0("UMAP; ", curr_signature)) +
    guides(color = guide_legend(override.aes=list(size=1.5))) + guides(fill=guide_legend(ncol=3)) + #guides(fill = guide_legend(override.aes = list(size=4))) +
    ggrepel::geom_label_repel(data=my_labels2, fill="gray70", box.padding=0.0, alpha=0.8, label.size=0.05, color="black", size=3, aes(label=my_labels2[["label"]])) +
    theme_minimal() +
    theme(panel.border=element_rect(fill=NA, linetype="solid", colour = "black", linewidth=0.5), axis.title=element_text(size=8), 
          axis.text=element_text(size=8, angle=0), strip.text.y=element_text(size=8, angle=0), legend.position="bottom",
          legend.text=element_text(size=8, angle=0), plot.title=element_text(size=10), legend.key.size=unit(0.35, 'cm'), legend.title=element_blank())
#p_init_clusters
p1 = ggarrange(p_init_samples, p_init_clusters)
#ggsave(paste0(curr_outdir, "/marker_UMAP_TilleMatrix_sample_vs_cluster_ATAC", ".pdf"), plot=p1, device="pdf", height=5.11, width=9.5, units="in", limitsize=FALSE)
#ggsave(paste0(curr_outdir, "/marker_UMAP_TileMatrix_sample_vs_cluster_ATAC", ".png"), plot=p1, device="png", height=5.11, width=9.5, units="in", limitsize=FALSE)

# C1 = Sensitive_common_small;   Sensitive_side
# C2 = Sensitive_no_agent_big    Sensitive_DMSO
# C3 = Sensitive_agent;          Sensitive_GNE7883
# C4 = Sensitive_agent;          Sensitive_GNE7883
# C5 = Resistant_small;          Resistant_side
# C6 = Resistant_big;            Resistant_main
# C7 = Resistant_big;            Resistant_main 
#From XY: resistant_main (big), resistant_side (small), sensitive_DMSO, sensitive_GNE7883, sensitive_side (common)
project$Clusters_TileMatrix_named = plyr::mapvalues(project$Clusters_TileMatrix,
                                from = paste0("C_TileMatrix",1:7),
                                to = c("Sensitive_side", "Sensitive_DMSO", "Sensitive_GNE7883", 
                                       "Sensitive_GNE7883",  "Resistant_side", "Resistant_main", 
                                       "Resistant_main")
)
}\if{html}{\out{</div>}}
}

\subsection{6. Generate ordinations}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# add reduced dims TileMatrix
project = addIterativeLSI(project, useMatrix='TileMatrix', name='IterativeLSI_TileMatrix', seed=2, force=TRUE, threads=num_threads)
project = addClusters(project, reducedDims='IterativeLSI_TileMatrix', name='Clusters_TileMatrix', seed=2, prefix='C_TileMatrix', force=TRUE, threads=num_threads)
project = addUMAP(project, reducedDims='IterativeLSI_TileMatrix', name='UMAP_TileMatrix', seed=2, force=TRUE, threads=num_threads)

# add reduced dims GeneExpressionMatrix
project = addIterativeLSI(project, useMatrix='GeneExpressionMatrix', name='IterativeLSI_GeneExpressionMatrix', seed=2, force=TRUE, threads=num_threads)
project = addClusters(project, reducedDims='IterativeLSI_GeneExpressionMatrix', name='Clusters_GeneExpressionMatrix', seed=2, force=TRUE, prefix='C_GeneExpressionMatrix', threads=num_threads)
project = addUMAP(project, reducedDims='IterativeLSI_GeneExpressionMatrix', name='UMAP_GeneExpressionMatrix', seed=2, force=TRUE, threads=num_threads)

# Combine both dimensionalities, add cluster to resulting combined dim and add umap
project = addCombinedDims(project, reducedDims=c('IterativeLSI_TileMatrix', 'IterativeLSI_GeneExpressionMatrix'), name='IterativeLSI_Combined')
project = addClusters(project, reducedDims='IterativeLSI_Combined', name='Clusters_Combined', seed=2, force=TRUE, prefix='C_Combined', threads=num_threads)
project = addUMAP(project, reducedDims='IterativeLSI_Combined', name='UMAP_Combined', seed=2, force=TRUE, threads=num_threads)

# Add tSNE embeddings
project = addTSNE(project, reducedDims='IterativeLSI_TileMatrix', name='TSNE_TileMatrix', perplexity=30, seed=2, force=TRUE, threads=num_threads)
project = addTSNE(project, reducedDims='IterativeLSI_GeneExpressionMatrix', name='TSNE_GeneExpressionMatrix', perplexity=30, seed=2, force=TRUE, threads=num_threads)
project = addTSNE(project, reducedDims='IterativeLSI_Combined', name='TSNE_Combined', perplexity=30, seed=2, force=TRUE, threads=num_threads)
project = addTSNE(project, reducedDims='IterativeLSI_ATAC', name='TSNE_ATAC', perplexity=30, seed=2, force=TRUE, threads=num_threads)
}\if{html}{\out{</div>}}
}

\subsection{7. Peak calling}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(MASS)
library(chromVARmotifs)

project = addGroupCoverages(project, groupBy='Clusters_TileMatrix', threads=num_threads)

project = addReproduciblePeakSet(project, groupBy="Clusters_TileMatrix", method="q", cutOff=0.05,
                                    pathToMacs2="/gstore/home/tremblj2/software/macs2/macs2_venv/bin/macs2", 
                                    excludeChr=c('chrMT','chrY'), genomeSize=2.7e9, threads=num_threads)

project = addPeakMatrix(project, binarize=FALSE, threads=num_threads, force=TRUE)

# TF annotation
peaks_anno = genomitory::getFeatures('GMTY162:hg38_motif_bed_granges@REVISION-3')
project = addPeakAnnotations(project, regions=peaks_anno, name='ENCODE_and_cistromeDB_TF_peaks', force=TRUE)
project = addDeviationsMatrix(project, peakAnnotation='ENCODE_and_cistromeDB_TF_peaks', 
                                 matrixName='TFPeaksDeviationsMatrix', threads=num_threads, force=TRUE)

# motif annotation
project = addMotifAnnotations(project, motifSet='cisbp', annoName='Motif', species='Homo sapiens', force=TRUE)
project = addDeviationsMatrix(project, peakAnnotation='Motif', threads=num_threads, force=TRUE)

# TF binding matrix 
project = addPeakAnnotations(project, regions=epiregulon::getTFMotifInfo(genome="hg38"), name="TF_binding", force=T, logFile="addPeakAnnotations")
project = addDeviationsMatrix(project, peakAnnotation="TF_binding", threads=num_threads)
}\if{html}{\out{</div>}}
}
}

\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\examples{
# check metada of dataset
TEADi_resistance(metadata = TRUE)

# download data
 \dontrun{
TEADi_resistance()
}

}
\references{
Manuscript under review...
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve.R
\name{retrieve}
\alias{retrieve}
\title{retrieve data set or its metadata}
\usage{
retrieve(dataset, metadata, experiments, verbose = FALSE)
}
\arguments{
\item{dataset}{character string specifying the data set name}

\item{metadata}{logical flag specifying whether to return the resource or only its metadata}

\item{experiments}{character string specifying which experiments to extract}

\item{verbose}{logical flag specifying loading verbosity}
}
\value{
If \code{metadata = FALSE}, a \code{MultiAssayExperiment},
otherwise an \code{ExperimentHub} object.
}
\description{
Retrieve and return the data or metadata for the currently queried data set.
}
\details{
This is a generic accessor function that is used by user-level accessor functions
to access data sets or their metadata.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colonHealthy.R
\name{colonHealthy}
\alias{colonHealthy}
\title{Single-cell analysis of samples from healthy human colon}
\format{
\code{MultiAssayExperiment} obtained from an \code{ArchR} project. Annotated with the Hg38 genome build.
Contains the following experiments:
\itemize{
\item{\strong{TileMatrix}: SingleCellExperiment with 6062095 rows and 59231 columns}
\item{\strong{GeneIntegrationMatrix}: SingleCellExperiment with 19020 rows and 59231 columns}
\item{\strong{GeneScoreMatrix}: SingleCellExperiment with 24919 rows and 59231 columns}
\item{\strong{MotifMatrix}: SingleCellExperiment with 870 rows and 59231 columns}
\item{\strong{PeakMatrix}: SingleCellExperiment with 406946 rows and 59231 columns}
}
}
\usage{
colonHealthy(
  metadata = FALSE,
  experiments = c("TileMatrix", "GeneIntegrationMatrix", "GeneScoreMatrix",
    "MotifMatrix", "PeakMatrix")
)
}
\arguments{
\item{metadata}{logical flag specifying whether to return data or metadata only}

\item{experiments}{character vector of matrices to return; see \code{Format}}
}
\value{
\code{MultiAssayExperiment} made up of \code{SingleCellExperiment}s
with assays stored as \code{DelayedMatrix} objects.
If \code{metadata = TRUE}, an \code{ExperimentHub} object listing this data set's metadata.
}
\description{
ATACseq and RNAseq data obtained by the colon tissues analysis. Samples were
collected from adult human donors.
}
\section{Data preparation}{


scATAC data was downloaded from Gene Expression Omnibus
(acc. no. \href{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE165659}{GSE165659})
and analyzed with SingleCell ATAC - 10X pipeline v2.0.0
scRNAseq data in form of Seurat objects was downloaded from  https://drive.google.com/drive/folders/12j9ufV1L0uWbUlab-VoXRznDLKDO7PQ. In case
of future change in the data storage location, it will be updated in the readme file in project's Github repository (https://github.com/winstonbecker/scCRC_continuum)

Downstream analysis was performed with the \code{ArchR} package v. 1.0.2:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{library(ArchR)
library(parallel)

catlas_files <- <FRAGMENT_FILES>
outputDir <- <OUTPUT_DIRECTORY>
arrow_files <- createArrowFiles(inputFiles = catlas_files, sampleNames = <SAMPLE_NAMES>)
doubScores <- addDoubletScores(input = arrow_files)

# cerate ArchR project
project <- ArchRProject(arrow_files, outputDirectory = outputDir)

# filtering out doublet cells
project <- filterDoublets(project)

# add Iterative Latent Semantic Indexing reduced-dimensionality space
project <- addIterativeLSI(ArchRProj = project, useMatrix = "TileMatrix", 
                           name = "IterativeLSI", clusterParams = list(resolution = c(0.2), 
                                                    sampleCells = 10000, n.start = 10))

# batch correction
project  <- addHarmony( ArchRProj = project, reducedDims = "IterativeLSI",
    name = "Harmony", groupBy = "Sample")

project <- addClusters( input = project, reducedDims = "IterativeLSI",
    method = "Seurat", name = "Clusters", resolution = 0.8)

# add clusters after Harmony batch correction
project <- addClusters(input = project, reducedDims = "Harmony",
    method = "Seurat", name = "Clusters_Harmony", resolution = 0.8)

# add UMAP embedding 
project <- addUMAP(ArchRProj = project, reducedDims = "IterativeLSI",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI")

project <- addUMAP(ArchRProj = project, reducedDims = "IterativeLSI",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_Harmony")

# add column with log base 10 of the fragment numbers
project <- addCellColData(ArchRProj = project, data = log10(project$nFrags),
                                  name = "log10_nFrags", cells = project$cellNames)




# upload gene expression data
# use files downloaded from 
# https://drive.google.com/drive/folders/12j9ufV1L0uWbUlab-VoXRznDLKDO7PQ_?usp=sharing

data_files <- c("Final_scHTAN_colon_normal_epithelial_220213.rds",
                "Final_scHTAN_colon_immune_220213.rds",
                "Final_scHTAN_colon_stromal_220213.rds")

# define object names
RNAseq_se_names <- gsub(".*scHTAN_|_220213.rds", "", data_files)

# create objects as instances of SingleCellExperiment class
for (i in seq_along(RNAseq_se_names)) assign(RNAseq_se_names[i], Seurat::as.SingleCellExperiment(readRDS(data_files[i])))

# add column with cell types and disease state
for (obj in RNAseq_se_names)\{
    eval(parse(text = paste0("colData(", obj, ")$CellType <-", obj,"@colData@listData$CellType")))
    eval(parse(text = paste0("colData(", obj, ")$DiseaseState <-", obj,"@colData@listData$DiseaseState")))
\}

# uniformize colData columns before merging
shared_cols <- purrr::map(list(colon_immune, colon_stromal, colon_normal_epithelial), colData) \%>\%
    purrr::map(colnames) \%>\%
    purrr::reduce(intersect)

# remove reducedDims since their column names differ across objects
for (obj in RNAseq_se_names)\{
    eval(parse(text = paste0(obj, "@colData <- ", obj, "@colData[,colnames(", obj, "@colData) \%in\% shared_cols]")))
    eval(parse(text = paste0("SingleCellExperiment::reducedDim(", obj, ") <- NULL")))
    eval(parse(text = paste0(obj, "@int_colData@listData <- list()")))
\}

# merge RNAseq data objects

colon_RNAseq <- cbind(colon_immune, colon_normal_epithelial, colon_stromal)


RNA_se <- SummarizedExperiment(assay = list(counts = as(assay(colon_RNAseq, "counts"), "dgCMatrix")),
                               colData = colData(colon_RNAseq), rowData = rowData(colon_RNAseq))


# select samples from healthy donors (no cancer)
RNA_se <- RNA_se[,colData(RNA_se)$DiseaseState == "Normal"]

# RNA integration
project <- addGeneIntegrationMatrix(
    ArchRProj = project,
    useMatrix = "GeneScoreMatrix",
    reducedDims = "IterativeLSI",
    seRNA = RNA_se,
    addToArrow = TRUE,
    groupRNA = "CellType",
    nameCell = "predicted_cell_un",
    nameGroup = "predicted_group_un",
    nameScore = "predicted_score_un")


project <- addGroupCoverages(ArchRProj = project, groupBy = "predicted_group_un")

# add pseudo-bulk replicates
## requires MACS2 installation

project <- addReproduciblePeakSet( ArchRProj = project,
    groupBy = "predicted_group_un", pathToMacs2 = <PATH_TO_MACS2>)


# LSI reduced dimensionality based on the GeneIntegrationMatrix

project <- addIterativeLSI(ArchRProj = project, clusterParams = list(resolution = 0.2,
        sampleCells = 1000, n.start = 10), saveIterations = FALSE,
    useMatrix = "GeneIntegrationMatrix", varFeatures = 2500,
    firstSelection = "variable", binarize = FALSE, name = "LSI_RNA")

# add clusters based on the new reduced-dimensionality space
project <- addClusters( input = project, reducedDims = "LSI_RNA",
    method = "Seurat", name = "Clusters_RNA", resolution = 0.8)

# add UMAP embedding
project <- addUMAP(ArchRProj = project, reducedDims = "LSI_RNA",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI_RNA", metric = "cosine",
    force = TRUE)

# batch correction 
project  <- addHarmony( ArchRProj = project, reducedDims = "LSI_RNA",
    name = "Harmony_RNA", groupBy = "Sample")

# UMAP embedding after batch correction
project <- addUMAP(ArchRProj = project, reducedDims = "Harmony_RNA",
    nNeighbors = 30, minDist = 0.5, name ="UMAP_LSI_RNA_Harmony", metric = "cosine",
    force = TRUE
)

# find clutsters after batch correction
project <- addClusters( input = project, reducedDims = "Harmony_RNA",
    method = "Seurat", name = "Clusters_RNA_Harmony", resolution = 0.8)


# combine reduced-dimensionality spaces produced from ATACseq and RNAseq data
project <- addCombinedDims(project, reducedDims = c("IterativeLSI", "LSI_RNA"),
                            name =  "LSI_Combined")

# add UMAP embedding
project <- addUMAP(ArchRProj = project, name = "UMAP_combined", reducedDims = "LSI_Combined",
        nNeighbors = 30, minDist = 0.5, metric = "cosine")


# find clusters in combined reduced space
project <- addClusters(input = project, reducedDims = "LSI_Combined",
                        method = "Seurat", name = "Clusters_combined",
                        resolution = 0.4)

# add information about sequence motifs recognized by known transcriptions factors
project <- addMotifAnnotations(ArchRProj = project,
                                       motifSet = "cisbp", name = "Motif")


# add background peaks to be compared against during peak variation assessement
project <- addBgdPeaks(project)

# calculate per-cell devations of motif annotations
project <- addDeviationsMatrix(project, peakAnnotation = "Motif")

# save project
saveArchRProject(project, outputDir)

# convert project into MultiAssayExperiment object 
MAE <- maw.archr::create.mae.with.multiple.sces.from.archr(outputDir, tile.sizes = 500)

saveRDS(MAE, <OUTPUT_PATH>)

}\if{html}{\out{</div>}}
}

\section{Data storage and access}{

The \code{MultiAssayExperiments} is split into separate \code{SingleCellExperiment}
objects and they in turn are split into components, all of which are stored in a
single hdf5 file. Data and can be accessed with a special function that extracts
elements of the requested experiment(s), reassembles them, and builds an MAE.
}

\examples{
# check metada of dataset
colonHealthy(metadata = TRUE)
# download data
\dontrun{
colonHealthy()
}

}
\references{
\enumerate{
\item Zhang K, Hocker JD, Miller M, Hou X, Chiou J, Poirion OB, Qiu Y, Li YE,
Gaulton KJ, Wang A, Preissl S, Ren B. A single-cell atlas of
chromatin accessibility in the human genome.
Cell. 2021 Nov 24;184(24):5985-6001.e19. doi: 10.1016/j.cell.2021.10.024.
Epub 2021 Nov 12. PMID: 34774128; PMCID: PMC8664161.
\item Becker, W.R., Nevins, S.A., Chen, D.C. et al. Single-cell analyses
define a continuum of cell state and composition changes in the malignant
transformation of polyps to colorectal cancer. Nat Genet 54, 985–995 (2022).
https://doi.org/10.1038/s41588-022-01088-x
}
}
